#!/usr/bin/env bash

while getopts p: opt; do
    case $opt in
        p) PARAM_FILE_PATH=${OPTARG} ;;
    esac
done

# set -eu -o pipefail # fail on error and report it, debug all lines
sudo true
test $? -eq 0 || exit 1 "You should have sudo privileges to run this script"

if [ -z "$PARAM_FILE_PATH" ] ; then
    read -p "Enter the path of parameter file: " PARAM_FILE_PATH
    if [ -f "$PARAM_FILE_PATH" ]; then
        echo Specified parameter file exists.
        source $PARAM_FILE_PATH
    else
        echo Specified parameter file does not exist.
        exit 1    
    fi  
else
    echo "00_init_control_node.sh: Using supplied arguments..."
    source "$PARAM_FILE_PATH"
fi

set -v

echo BBX_INSTALL_PATH: $BBX_INSTALL_PATH

OSDISTRO=$( $(compgen -G "/etc/*release" > /dev/null) && cat /etc/*release | grep ^NAME | tr -d 'NAME="' || echo "${OSTYPE//[0-9.]/}")
OSVERSION=$( cat /etc/*release | grep ^VERSION_ID | tr -d 'VERSION_ID="'   )

sudo mkdir -p $BBX_INSTALL_PATH/certs
sudo mkdir -p $BBX_INSTALL_PATH/vol_db
sudo mkdir -p $BBX_INSTALL_PATH/vol_media
sudo mkdir -p $BBX_INSTALL_PATH/vol_rs
sudo mkdir -p $BBX_INSTALL_PATH/vol_static

if [[ $OSDISTRO =~ .*"Ubuntu".* ]]; then

sudo apt-get install ufw -y

sudo ufw allow 179/tcp
sudo ufw allow 2379/tcp
sudo ufw allow 2380/tcp
sudo ufw allow 6443/tcp
sudo ufw allow 10250/tcp
sudo ufw allow 10251/tcp
sudo ufw allow 10252/tcp
sudo ufw allow 32767/tcp
# sudo ufw allow 8000/tcp
# sudo ufw allow 8080/tcp
# sudo ufw allow 80/tcp
sudo ufw allow 2049/tcp
sudo ufw allow 5000/tcp

sudo ufw allow 53/tcp
sudo ufw allow 53/udp
sudo ufw allow 5432/tcp

sudo apt-get update -y

sudo apt-get install openssh-server -y
sudo apt-get install net-tools -y 
sudo apt-get install git -y
sudo apt-get install nano -y
sudo apt-get install -y jq

sudo systemctl enable ssh
sudo systemctl start ssh

sudo apt-get install libnss3-tools -y
sudo apt-get install curl -y
sudo apt-get install unzip -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Disable swap permanently. On some versions of Ubuntu, editing /etc/fstab is not sufficient.
# sudo swapoff -a
# sudo rm /swap.img
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab
# systemctl mask swap.img.swap

sudo apt-get update -y

if [[ $OSVERSION = "22.04" ]]; then

# UBUNTU 22

sudo apt-get install apt-transport-https ca-certificates -y 

# echo "deb [signed-by=/etc/apt/keyrings/kubernetes-archive-keyring.gpg] https://apt.kubernetes.io/ kubernetes-xenial main" | sudo tee /etc/apt/sources.list.d/kubernetes.list # deprecated
# sudo curl -fsSLo /etc/apt/keyrings/kubernetes-archive-keyring.gpg https://dl.k8s.io/apt/doc/apt-key.gpg # deprecated

# Download the public signing key for the Kubernetes package repositories. The same signing key is used for all repositories so you can disregard the version in the URL:
curl -fsSL https://pkgs.k8s.io/core:/stable:/v1.28/deb/Release.key | sudo gpg --dearmor -o /etc/apt/keyrings/kubernetes-apt-keyring.gpg

# Add the appropriate Kubernetes apt repository. If you want to use Kubernetes version different than v1.28, replace v1.28 with the desired minor version in the command below:
echo 'deb [signed-by=/etc/apt/keyrings/kubernetes-apt-keyring.gpg] https://pkgs.k8s.io/core:/stable:/v1.28/deb/ /' | sudo tee /etc/apt/sources.list.d/kubernetes.list

sudo apt-get update -y
sudo apt-get install -y kubelet=1.28.0-1.1 kubeadm=1.28.0-1.1 kubectl=1.28.0-1.1
sudo apt-mark hold kubelet kubeadm kubectl

sudo apt-get remove -y docker docker-engine docker.io containerd runc 
sudo apt-get update -y
sudo apt-get install gnupg=2.2.27-3ubuntu2.1 lsb-release=11.1.0ubuntu4 -y

curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg
echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo mkdir -p /etc/apt/keyrings
curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/docker.gpg
echo "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
sudo apt-get update -y
sudo apt-get install docker-ce=5:20.10.17~3-0~ubuntu-jammy -y

sudo apt-get install docker-compose=1.29.2-1 -y
# sudo rm /usr/bin/docker-compose
# sudo curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/bin/docker-compose
# sudo chmod +x /usr/bin/docker-compose

sudo apt-get install iproute2 -y

# sudo apt-get install containerd.io=1.6.24-1 -y --allow-downgrades
sudo apt-get install containerd.io=1.6.24-1 -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable containerd

# EDIT THE TOML FILE /etc/containerd/config.toml and set the systemd cgroup to true 
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo cp /etc/containerd/config.toml /etc/containerd/config.toml.bak

    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2

    sudo sed -i "s/.registry.configs]/.registry.configs]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\"] # edited line\n          [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\".tls] # edited line\n            ca_file = \"\" # edited line\n            cert_file = \"\" # edited line\n            insecure_skip_verify = true # edited line\n            key_file = \"\" # edited line/g" $FILE2
    sudo sed -i "s/.registry.mirrors]/.registry.mirrors]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.mirrors.\"$REGISTRY_URL:5000\"] # edited line\n          endpoint = [\"https:\/\/$REGISTRY_URL:5000\"] # edited line /g" $FILE2
   
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

sudo apt autoremove -y

sudo systemctl enable kubelet
sudo systemctl restart kubelet

# sudo apt-get install nfs-common -y

else # other Ubuntu versions

sudo apt-get install apt-transport-https ca-certificates -y 

# echo "deb [signed-by=/etc/apt/keyrings/kubernetes-archive-keyring.gpg] https://apt.kubernetes.io/ kubernetes-xenial main" | sudo tee /etc/apt/sources.list.d/kubernetes.list # deprecated
# sudo curl -fsSLo /etc/apt/keyrings/kubernetes-archive-keyring.gpg https://dl.k8s.io/apt/doc/apt-key.gpg # deprecated

# Download the public signing key for the Kubernetes package repositories. The same signing key is used for all repositories so you can disregard the version in the URL:
curl -fsSL https://pkgs.k8s.io/core:/stable:/v1.28/deb/Release.key | sudo gpg --dearmor -o /etc/apt/keyrings/kubernetes-apt-keyring.gpg

# Add the appropriate Kubernetes apt repository. If you want to use Kubernetes version different than v1.28, replace v1.28 with the desired minor version in the command below:
echo 'deb [signed-by=/etc/apt/keyrings/kubernetes-apt-keyring.gpg] https://pkgs.k8s.io/core:/stable:/v1.28/deb/ /' | sudo tee /etc/apt/sources.list.d/kubernetes.list

sudo apt-get update -y
sudo apt-get install -y kubelet=1.28.0-1.1 kubeadm=1.28.0-1.1 kubectl=1.28.0-1.1
sudo apt-mark hold kubelet kubeadm kubectl

sudo apt-get remove -y docker docker-engine docker.io containerd runc 
sudo apt-get update -y
sudo apt-get install gnupg lsb-release -y

curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg
echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo mkdir -p /etc/apt/keyrings
curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/docker.gpg
echo "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
sudo apt-get update -y
sudo apt-get install docker-ce -y

sudo apt-get install docker-compose -y
# sudo rm /usr/bin/docker-compose
# sudo curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/bin/docker-compose
# sudo chmod +x /usr/bin/docker-compose

sudo apt-get install iproute2 -y

sudo apt-get install containerd.io -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable containerd

# EDIT THE TOML FILE /etc/containerd/config.toml and set the systemd cgroup to true 
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo cp /etc/containerd/config.toml /etc/containerd/config.toml.bak
    
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2

    sudo sed -i "s/.registry.configs]/.registry.configs]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\"] # edited line\n          [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\".tls] # edited line\n            ca_file = \"\" # edited line\n            cert_file = \"\" # edited line\n            insecure_skip_verify = true # edited line\n            key_file = \"\" # edited line/g" $FILE2
    sudo sed -i "s/.registry.mirrors]/.registry.mirrors]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.mirrors.\"$REGISTRY_URL:5000\"] # edited line\n          endpoint = [\"https:\/\/$REGISTRY_URL:5000\"] # edited line /g" $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

sudo apt autoremove -y

sudo systemctl enable kubelet
sudo systemctl restart kubelet

# sudo apt-get install nfs-common -y

fi

elif [[ $OSDISTRO =~ .*"Red Hat".* ]]; then

# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port={6443,2379,2380,10250,10251,10252}/tcp

sudo firewall-cmd --zone=public --permanent --add-port 179/tcp
sudo firewall-cmd --zone=public --permanent --add-port 32767/tcp
sudo firewall-cmd --zone=public --permanent --add-port 22/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8000/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8080/tcp
sudo firewall-cmd --zone=public --permanent --add-port 80/tcp
sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp

sudo firewall-cmd --zone=public --permanent --add-port 53/tcp
sudo firewall-cmd --zone=public --permanent --add-port 53/udp
sudo firewall-cmd --zone=public --permanent --add-port 5432/tcp

sudo firewall-cmd --add-masquerade --permanent

sudo firewall-cmd --reload

sudo dnf install openssh-server -y 

sudo yum install git -y
sudo yum install nano -y
sudo yum install epel-release -y
# sudo yum update -y
sudo yum install jq -y

sudo systemctl start sshd
sudo systemctl enable sshd

sudo yum install unzip curl -y

# https://github.com/FiloSottile/mkcert
sudo yum install nss-tools -y

if [[ $OSVERSION == "8.6" ]]; then
# RHEL8.6

sudo yum install ca-certificates-2021.2.50-80.0.el8_4.noarch -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc 

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install yum-utils-4.0.21-11.el8.noarch -y

sudo dnf install iproute-tc-5.15.0-4.el8 -y

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

sudo yum install containerd.io -y
sudo yum install docker-ce-3:20.10.9-3.el8 -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable --now containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo cp /etc/containerd/config.toml /etc/containerd/config.toml.bak
    
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2

    sudo sed -i "s/.registry.configs]/.registry.configs]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\"] # edited line\n          [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\".tls] # edited line\n            ca_file = \"\" # edited line\n            cert_file = \"\" # edited line\n            insecure_skip_verify = true # edited line\n            key_file = \"\" # edited line/g" $FILE2
    sudo sed -i "s/.registry.mirrors]/.registry.mirrors]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.mirrors.\"$REGISTRY_URL:5000\"] # edited line\n          endpoint = [\"https:\/\/$REGISTRY_URL:5000\"] # edited line /g" $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet
# sudo systemctl restart kubelet

sudo curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/bin/docker-compose
sudo chmod +x /usr/bin/docker-compose

sudo systemctl enable --now docker.service

# sudo dnf install nfs-utils-1:2.3.3-51.el8 nfs4-acl-tools-0.3.5-3.el8  -y

elif [[ $OSVERSION == "9.2" ]]; then
# RHEL 9.2

sudo yum install ca-certificates-2023.2.60_v7.0.306-90.1.el9_2.noarch -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc 

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install yum-utils-4.3.0-5.el9_2.noarch -y

sudo dnf install iproute-tc-6.1.0-1.el9.x86_64 -y

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

sudo yum install containerd.io-1.6.24-3.1.el9.x86_64 -y
sudo yum install docker-ce-3:24.0.7-1.el9.x86_64 -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable --now containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo cp /etc/containerd/config.toml /etc/containerd/config.toml.bak
    
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2

    sudo sed -i "s/.registry.configs]/.registry.configs]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\"] # edited line\n          [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\".tls] # edited line\n            ca_file = \"\" # edited line\n            cert_file = \"\" # edited line\n            insecure_skip_verify = true # edited line\n            key_file = \"\" # edited line/g" $FILE2
    sudo sed -i "s/.registry.mirrors]/.registry.mirrors]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.mirrors.\"$REGISTRY_URL:5000\"] # edited line\n          endpoint = [\"https:\/\/$REGISTRY_URL:5000\"] # edited line /g" $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet # must be done as root
# sudo systemctl restart kubelet

sudo curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/bin/docker-compose
sudo chmod +x /usr/bin/docker-compose

sudo systemctl enable --now docker.service

# sudo dnf install nfs-utils-1:2.5.4-18.el9.x86_64 nfs4-acl-tools-0.4.2-0.el9.x86_64  -y

else # other versions of RHEL

sudo yum install ca-certificates -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc 

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install yum-utils -y

sudo dnf install iproute-tc -y

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

sudo yum install containerd.io -y
sudo yum install docker-ce -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable --now containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo cp /etc/containerd/config.toml /etc/containerd/config.toml.bak
    
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2

    sudo sed -i "s/.registry.configs]/.registry.configs]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\"] # edited line\n          [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\".tls] # edited line\n            ca_file = \"\" # edited line\n            cert_file = \"\" # edited line\n            insecure_skip_verify = true # edited line\n            key_file = \"\" # edited line/g" $FILE2
    sudo sed -i "s/.registry.mirrors]/.registry.mirrors]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.mirrors.\"$REGISTRY_URL:5000\"] # edited line\n          endpoint = [\"https:\/\/$REGISTRY_URL:5000\"] # edited line /g" $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
sudo setenforce 0
sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet
# sudo systemctl restart kubelet

sudo curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/bin/docker-compose
sudo chmod +x /usr/bin/docker-compose

sudo systemctl enable --now docker.service

# sudo dnf install nfs-utils nfs4-acl-tools  -y

fi

elif [[ $OSDISTRO =~ .*"CentOS".* ]]; then

# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port={6443,2379,2380,10250,10251,10252}/tcp

sudo firewall-cmd --zone=public --permanent --add-port 179/tcp
sudo firewall-cmd --zone=public --permanent --add-port 32767/tcp
sudo firewall-cmd --zone=public --permanent --add-port 22/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8000/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8080/tcp
sudo firewall-cmd --zone=public --permanent --add-port 80/tcp
sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp

sudo firewall-cmd --zone=public --permanent --add-port 53/tcp
sudo firewall-cmd --zone=public --permanent --add-port 53/udp
sudo firewall-cmd --zone=public --permanent --add-port 5432/tcp

sudo firewall-cmd --add-masquerade --permanent

sudo firewall-cmd --reload

sudo dnf install openssh-server -y 

sudo yum install git -y
sudo yum install nano -y
sudo yum install epel-release -y
# sudo yum update -y
sudo yum install jq -y

sudo systemctl start sshd
sudo systemctl enable sshd

sudo yum install unzip -y

sudo yum install nss-tools -y

if [[ $OSVERSION == "7" ]]; then

# Centos 7

echo "Centos 7"

sudo yum install ca-certificates-2023.2.60_v7.0.306-72.el7_9.noarch -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc 

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install -y yum-utils-1.1.31-54.el7_8.noarch
sudo yum install -y iproute-4.11.0-30.el7.x86_64

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

# sudo yum install -y docker-ce 
curl -fsSL https://get.docker.com/ | sh
sudo yum install -y containerd.io.x86_64

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable --now containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo cp /etc/containerd/config.toml /etc/containerd/config.toml.bak
    
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2

    sudo sed -i "s/.registry.configs]/.registry.configs]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\"] # edited line\n          [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\".tls] # edited line\n            ca_file = \"\" # edited line\n            cert_file = \"\" # edited line\n            insecure_skip_verify = true # edited line\n            key_file = \"\" # edited line/g" $FILE2
    sudo sed -i "s/.registry.mirrors]/.registry.mirrors]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.mirrors.\"$REGISTRY_URL:5000\"] # edited line\n          endpoint = [\"https:\/\/$REGISTRY_URL:5000\"] # edited line /g" $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet
# sudo systemctl restart kubelet

sudo curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/bin/docker-compose
sudo chmod +x /usr/bin/docker-compose

sudo systemctl enable --now docker.service

# sudo yum install -y nfs-utils-1.3.0-0.68.el7.2.x86_64 nfs4-acl-tools.x86_64 

elif [[ $OSVERSION == "8" ]]; then

# Centos 8
echo "Centos 8"

# Centos 8 has reached the end of life, and the url to download packages is now different
# Assume the user has already done this if they are using Centos 8
# cd /etc/yum.repos.d/
# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y

# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y
# cd ~/

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port={6443,2379,2380,10250,10251,10252}/tcp

sudo firewall-cmd --zone=public --permanent --add-port 179/tcp
sudo firewall-cmd --zone=public --permanent --add-port 32767/tcp
sudo firewall-cmd --zone=public --permanent --add-port 22/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8000/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8080/tcp
sudo firewall-cmd --zone=public --permanent --add-port 80/tcp
sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp

sudo firewall-cmd --zone=public --permanent --add-port 53/tcp
sudo firewall-cmd --zone=public --permanent --add-port 53/udp
sudo firewall-cmd --zone=public --permanent --add-port 5432/tcp

sudo firewall-cmd --add-masquerade --permanent

sudo firewall-cmd --reload

sudo dnf install openssh-server -y 

sudo yum install git -y
sudo yum install nano -y
sudo yum install epel-release -y
# sudo yum update -y
sudo yum install jq -y

sudo systemctl start sshd
sudo systemctl enable sshd

sudo yum install unzip -y

sudo yum install nss-tools -y

sudo yum install ca-certificates -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc 

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install -y yum-utils
sudo dnf install -y iproute-tc

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

sudo yum install -y containerd.io

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable --now containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo cp /etc/containerd/config.toml /etc/containerd/config.toml.bak
    
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2

    sudo sed -i "s/.registry.configs]/.registry.configs]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\"] # edited line\n          [plugins.\"io.containerd.grpc.v1.cri\".registry.configs.\"$REGISTRY_URL:5000\".tls] # edited line\n            ca_file = \"\" # edited line\n            cert_file = \"\" # edited line\n            insecure_skip_verify = true # edited line\n            key_file = \"\" # edited line/g" $FILE2
    sudo sed -i "s/.registry.mirrors]/.registry.mirrors]\n        [plugins.\"io.containerd.grpc.v1.cri\".registry.mirrors.\"$REGISTRY_URL:5000\"] # edited line\n          endpoint = [\"https:\/\/$REGISTRY_URL:5000\"] # edited line /g" $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet
# sudo systemctl restart kubelet

sudo yum install -y docker-ce 

sudo curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/bin/docker-compose
sudo chmod +x /usr/bin/docker-compose

sudo systemctl enable --now docker.service

# sudo dnf install -y nfs-utils nfs4-acl-tools

else

# other versions of Centos

echo "Unsupported Centos version"

exit 1

fi

else

# other distros

echo "Unsupported Linux version"

exit 1

fi


# aggregator routing needs to be enabled  in the kube-apiserver for the metrics server to work correctly
cat <<EOF | sudo tee $HOME/kubeadm-config.yaml
kind: ClusterConfiguration
apiVersion: kubeadm.k8s.io/v1beta3
kubernetesVersion: v1.28.0
apiServer:
  extraArgs:
    enable-aggregator-routing: "true"
---
kind: KubeletConfiguration
apiVersion: kubelet.config.k8s.io/v1beta1
cgroupDriver: systemd
failSwapOn: false
featureGates:
  NodeSwap: true
memorySwap:
  swapBehavior: LimitedSwap
EOF

# mkdir -p ~/export
# mkdir -p ~/export/calico
# mkdir -p ~/export/coredns
# mkdir -p ~/export/bitnami
# mkdir -p ~/export/nginxinc

# the kubelet version cannot be higher than the control plane version: 
# sudo kubeadm config images pull
# sudo docker save k8s.gcr.io/kube-apiserver:v1.23.6 -o ~/export/kube-apiserver:v1.23.6  
# sudo docker save k8s.gcr.io/kube-controller-manager:v1.23.6 -o ~/export/kube-controller-manager:v1.23.6
# sudo docker save k8s.gcr.io/kube-scheduler:v1.23.6 -o ~/export/kube-scheduler:v1.23.6 
# sudo docker save k8s.gcr.io/kube-proxy:v1.23.6 -o ~/export/kube-proxy:v1.23.6
# sudo docker save k8s.gcr.io/pause:3.6 -o ~/export/pause:3.6 
# sudo docker save k8s.gcr.io/etcd:3.5.1-0 -o ~/export/etcd:3.5.1-0
# sudo docker save k8s.gcr.io/coredns/coredns:v1.8.6 -o ~/export/coredns/coredns:v1.8.6


sudo kubeadm init --config $HOME/kubeadm-config.yaml


sudo rm -rf $HOME/.kube
mkdir -p $HOME/.kube
sudo cp -i /etc/kubernetes/admin.conf $HOME/.kube/config
sudo chown $(id -u):$(id -g) $HOME/.kube/config
export KUBECONFIG=$HOME/.kube/config
echo 'export KUBECONFIG=$HOME/.kube/config' >> $HOME/.bashrc

sudo systemctl daemon-reload
sudo systemctl restart kubelet

sleep 10

for node in $(kubectl get nodes --output=jsonpath={.items..metadata.name}); do echo $node && kubectl label nodes $node node-type=master; done
kubectl label namespace kube-system namespace=kube-system

kubectl taint nodes --all node-role.kubernetes.io/master- &>/dev/null # suppress error messages where this taint doesnt already exist
kubectl taint nodes --all node-role.kubernetes.io/control-plane-
# kubectl taint nodes NODE node-role.kubernetes.io/master:NoSchedule
# kubectl taint nodes NODE node-role.kubernetes.io/control-plane:NoSchedule
# kubeadm token create --print-join-command


# Kubernetes is now installed, and the cluster is set up, but the node is not ready yet because it does not have a CNI 
# Install calico from  manifest -- then the node will be ready
curl https://raw.githubusercontent.com/projectcalico/calico/v3.24.0/manifests/calico.yaml -O
kubectl apply -f calico.yaml

# curl -fsSL -o get_helm.sh https://raw.githubusercontent.com/helm/helm/main/scripts/get-helm-3
# chmod 700 get_helm.sh
# ./get_helm.sh

# specify the version of helm to use
curl -o helm-v3.9.3-linux-amd64.tar.gz https://get.helm.sh/helm-v3.9.3-linux-amd64.tar.gz
tar -zxvf helm-v3.9.3-linux-amd64.tar.gz
sudo mv linux-amd64/helm /usr/local/bin/helm


sudo systemctl restart docker
sudo systemctl restart containerd
sudo systemctl restart kubelet

sudo usermod -a -G docker $USER

# echo 'sudo swapoff -a' >> $HOME/.bashrc

# THE SYSTEM MUST BE RESTARTED AFTER RUNNING THIS SCRIPT
