#!/usr/bin/env bash
echo "$#"

# RUN THIS SCRIPT ON THE CONTROL PLANE NODE

export KUBECONFIG=$HOME/.kube/config

((count = 100))
while [[ $count -ne 0 ]] ; do
    kubectl get nodes 
    rc=$?
    if [[ $rc -eq 0 ]] ; then
        ((count = 1))
    fi
    ((count = count - 1))
done

if [[ $rc -ne 0 ]] ; then
    echo "Could not connect to kubectl after 100 attempts - stopping."
    exit 120
fi

node=$(kubectl get nodes --output=jsonpath='{.items[*].metadata.name}')
echo $node

node_ip=$(kubectl get nodes -o jsonpath='{.items[*].status.addresses[?(@.type=="InternalIP")].address}')
echo $node_ip

if [ -z "$1" ] ; then

    echo "100_ci_start.sh: Need to specify command line arguments"
    exit 1    
else
    echo "100_ci_start.sh: Using supplied arguments..."

    source "$1"
fi

source ./000_bbx_version.sh

echo BEETLEBOX_VERSION: $BEETLEBOX_VERSION

echo BBX_INSTALL_PATH: $BBX_INSTALL_PATH
echo CONTROL_NODE_FQDN: $CONTROL_NODE_FQDN

echo NFS_SOURCE_PATH: $NFS_SOURCE_PATH
echo NFS_SERVER: $NFS_SERVER
echo NFS_SERVER_IP: $NFS_SERVER_IP

echo REGISTRY_NODE_PATH: $REGISTRY_NODE_PATH
echo REGISTRY_IP: $REGISTRY_IP
echo REGISTRY_NODE_NAME: $REGISTRY_NODE_NAME
echo REGISTRY_FQDN: $REGISTRY_FQDN
echo REGISTRY_URL: $REGISTRY_URL

helm install --set global.localPath=$BBX_INSTALL_PATH/ --set db.environment.data.NFS_SOURCE=$NFS_SOURCE_PATH --set global.nfs.localPath=$NFS_SOURCE_PATH --set db.environment.data.REGISTRY_SERVER=$REGISTRY_URL:5000 --set global.volumeNode=$BBX_DEPLOY_NODE --set global.deploymentNode=$BBX_DEPLOY_NODE --set global.nfs.server=$NFS_SERVER --set db.environment.data.NFS_SERVER=$NFS_SERVER --set global.imageRegistry=$REGISTRY_URL:5000 --set db.environment.data.WORKFLOW_STATE=KUBERNETES --set db.environment.data.DJANGO_SECRET_KEY="SElhaShoLIum" --set db.environment.data.DJANGO_ALLOWED_HOSTS=* --set imageCredentials.registry=$REGISTRY_URL beetleboxci beetleboxci

kubectl wait pods  --selector=io.kompose.service=web --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-worker --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-scheduler --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-server --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=nginx --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=db --for condition=Ready --timeout=300s
