#!/usr/bin/env bash

# THIS SCRIPT INSTALLS SOME APPLICATIONS NEEDED FOR THE NFS TO WORK
# THE NFS WILL BE ACCESSIBLE BY OTHER NODES ON THE CLUSTER

# set -eu -o pipefail # fail on error and report it, debug all lines
sudo true
test $? -eq 0 || exit 1 "You should have sudo privileges to run this script"

OSDISTRO=$( $(compgen -G "/etc/*release" > /dev/null) && cat /etc/*release | grep ^NAME | tr -d 'NAME="' || echo "${OSTYPE//[0-9.]/}")
OSVERSION=$( cat /etc/*release | grep ^VERSION_ID | tr -d 'VERSION_ID="'   )

if [[ $OSDISTRO =~ .*"Ubuntu".* ]]; then

sudo apt-get install ufw -y

sudo ufw allow 2049/tcp

sudo apt-get update -y

sudo apt-get install net-tools -y 
sudo apt-get install libnss3-tools -y
sudo apt-get update -y

if [[ $OSVERSION = "22.04" ]]; then

# UBUNTU 22

sudo apt-get install iproute2 -y

sudo apt-get install nfs-kernel-server -y
sudo apt-get install nfs-common -y

else # other Ubuntu versions

sudo apt-get install iproute2 -y
sudo apt autoremove -y

sudo apt-get install nfs-server -y
sudo apt-get install nfs-common -y

fi

elif [[ $OSDISTRO =~ .*"Red Hat".* ]]; then

# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --add-masquerade --permanent
sudo firewall-cmd --reload

sudo yum install nss-tools -y

if [[ $OSVERSION == "8.6" ]]; then
# RHEL8.6

sudo dnf install iproute-tc-5.15.0-4.el8 -y
sudo dnf install nfs-utils-1:2.3.3-51.el8 nfs4-acl-tools-0.3.5-3.el8  -y

elif [[ $OSVERSION == "9.2" ]]; then
# RHEL 9.2

sudo dnf install iproute-tc-6.1.0-1.el9.x86_64 -y
sudo dnf install nfs-utils-1:2.5.4-18.el9.x86_64 nfs4-acl-tools-0.4.2-0.el9.x86_64  -y

else # other versions of RHEL

sudo dnf install iproute-tc -y
sudo dnf install nfs-utils nfs4-acl-tools  -y

fi

elif [[ $OSDISTRO =~ .*"CentOS".* ]]; then


if [[ $OSVERSION == "7" ]]; then

# Centos 7
echo "Centos 7"

# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --add-masquerade --permanent
sudo firewall-cmd --reload

sudo yum install nss-tools -y

sudo yum install -y iproute-4.11.0-30.el7.x86_64
sudo yum install -y nfs-utils-1.3.0-0.68.el7.2.x86_64 nfs4-acl-tools.x86_64 0:0.3.3-21.el7

elif [[ $OSVERSION == "8" ]]; then

# Centos 8
echo "Centos 8"

# Centos 8 has reached the end of life, and the url to download packages is now different
# Assume the user has already done this if they are using Centos 8
# cd /etc/yum.repos.d/
# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y

# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y
# cd ~/

# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --add-masquerade --permanent
sudo firewall-cmd --reload

sudo yum install nss-tools -y

sudo dnf install -y iproute-tc
sudo dnf install -y nfs-utils nfs4-acl-tools

else

# other versions of Centos

echo "Unsupported Centos version"

exit 1

fi

else

# other distros

echo "Unsupported Linux version"

exit 1

fi

