#!/usr/bin/env bash

# THE NFS WILL BE ACCESSIBLE BY OTHER NODES ON THE CLUSTER
# THIS SCRIPT IS TO BE RUN ON THE NFS SERVER TO ALLOW ACCESS TO OTHER NODES

while getopts p: opt; do
    case $opt in
        p) PARAM_FILE_PATH=${OPTARG} ;;
    esac
done

# set -eu -o pipefail # fail on error and report it, debug all lines
sudo true
test $? -eq 0 || exit 1 "You should have sudo privileges to run this script"

if [ -z "$PARAM_FILE_PATH" ] ; then
    read -p "Enter the path of parameter file: " PARAM_FILE_PATH
    if [ -f "$PARAM_FILE_PATH" ]; then
        echo Specified parameter file exists
        source $PARAM_FILE_PATH
    else
        echo Specified parameter file does not exist.
        exit 1    
    fi  
else
    echo "11_create_nfs.sh: Using supplied arguments..."
    source "$PARAM_FILE_PATH"
fi

export KUBECONFIG=$HOME/.kube/config

echo NFS_SOURCE_PATH: $NFS_SOURCE_PATH
echo NFS_SERVER_FQDN: $NFS_SERVER_FQDN
echo NFS_SERVER_NAME: $NFS_SERVER_NAME
echo NFS_SERVER_IP: $NFS_SERVER_IP
echo CONTROL_NODE_FQDN: $CONTROL_NODE_FQDN

OSDISTRO=$( $(compgen -G "/etc/*release" > /dev/null) && cat /etc/*release | grep ^NAME | tr -d 'NAME="' || echo "${OSTYPE//[0-9.]/}")
OSVERSION=$( cat /etc/*release | grep ^VERSION_ID | tr -d 'VERSION_ID="'   )

sudo mkdir -p $NFS_SOURCE_PATH

# Set up source path to be accessible as a network file system by 
sudo bash -c "cat << EOF >> /etc/exports
$NFS_SOURCE_PATH $NFS_SERVER_IP(rw,no_subtree_check,no_root_squash)
$NFS_SOURCE_PATH $NFS_SERVER_FQDN(rw,no_subtree_check,no_root_squash)
$NFS_SOURCE_PATH $NFS_SERVER_NAME(rw,no_subtree_check,no_root_squash)
$NFS_SOURCE_PATH $NFS_SERVER_NAME.local(rw,no_subtree_check,no_root_squash)
$NFS_SOURCE_PATH $CONTROL_NODE_FQDN(rw,no_subtree_check,no_root_squash)
EOF"

# sudo bash -c "cat << EOF >> /etc/exports
# $NFS_SOURCE_PATH 10.0.2.15(rw,no_subtree_check,no_root_squash) # only for VirtualBox VMs using NAT
# EOF"

if [[ $OSDISTRO =~ .*"Ubuntu".* ]]; then

sudo systemctl enable --now nfs-server
sudo exportfs -arv 
sudo exportfs  -s

elif [[ $OSDISTRO =~ .*"Red Hat".* ]]; then

sudo systemctl enable --now nfs-server

sudo exportfs -arv 
sudo exportfs  -s

sudo firewall-cmd --permanent --add-service=nfs
sudo firewall-cmd --permanent --add-service=rpc-bind
sudo firewall-cmd --permanent --add-service=mountd
sudo firewall-cmd --reload

elif [[ $OSDISTRO =~ .*"CentOS".* ]]; then

sudo systemctl enable --now nfs-server

sudo exportfs -arv 
sudo exportfs  -s

sudo firewall-cmd --permanent --add-service=nfs
sudo firewall-cmd --permanent --add-service=rpc-bind
sudo firewall-cmd --permanent --add-service=mountd
sudo firewall-cmd --reload

fi



