#!/usr/bin/env bash

# This script should be run on the node where the registry pod will be
# This generates the SSL certificates and places it in the correct location for them to the mounted inside the registry container
# The certificates need to be manually copied to every node that needs to access the registry.
# domain.crt must also be placed at the following location on the control node, in order for it to be included in the web app container when it is built
# $BBX_INSTALL_PATH/certs/domain.crt

while getopts p: opt; do
    case $opt in
        p) PARAM_FILE_PATH=${OPTARG} ;;
    esac
done

# set -eu -o pipefail # fail on error and report it, debug all lines
sudo true
test $? -eq 0 || exit 1 "You should have sudo privileges to run this script"

if [ -z "$PARAM_FILE_PATH" ] ; then
    read -p "Enter the path of parameter file: " PARAM_FILE_PATH
    if [ -f "$PARAM_FILE_PATH" ]; then
        echo Specified parameter file exists.
        source $PARAM_FILE_PATH
    else
        echo Specified parameter file does not exist.
        exit 1    
    fi  
else
    echo "40_create_certs.sh: Using supplied arguments..."
    source "$PARAM_FILE_PATH"
fi

export KUBECONFIG=$HOME/.kube/config

echo REGISTRY_NODE_PATH: $REGISTRY_NODE_PATH
echo REGISTRY_FQDN: $REGISTRY_FQDN
echo REGISTRY_NODE_NAME: $REGISTRY_NODE_NAME
echo REGISTRY_IP: $REGISTRY_IP

OSDISTRO=$( $(compgen -G "/etc/*release" > /dev/null) && cat /etc/*release | grep ^NAME | tr -d 'NAME="' || echo "${OSTYPE//[0-9.]/}")
OSVERSION=$( cat /etc/*release | grep ^VERSION_ID | tr -d 'VERSION_ID="'   )

sudo mkdir -p $REGISTRY_NODE_PATH/registry
sudo mkdir -p $REGISTRY_NODE_PATH/certs # will be mounted inside the registry container

if [[ $OSDISTRO =~ .*"Ubuntu".* ]]; then

sudo apt-get install ufw -y
sudo ufw allow 5000/tcp

sudo apt-get update -y

sudo apt-get install net-tools -y 
sudo apt-get install libnss3-tools -y

# https://github.com/FiloSottile/mkcert
# curl -JLO "https://dl.filippo.io/mkcert/latest?for=linux/amd64"
curl -JLO "https://dl.filippo.io/mkcert/v1.4.4?for=linux/amd64"
# curl -JLO  "https://github.com/FiloSottile/mkcert/releases/download/v1.4.4/mkcert-v1.4.4-linux-arm64"
sudo chmod +x mkcert-v*-linux-amd64
sudo mv mkcert-v*-linux-amd64 /usr/local/bin/mkcert
mkcert -install

sudo apt-get update -y


if [[ $OSVERSION = "22.04" ]]; then

# UBUNTU 22

sudo apt-get install apt-transport-https ca-certificates -y 

else # other Ubuntu versions

sudo apt-get install apt-transport-https ca-certificates -y 

fi

elif [[ $OSDISTRO =~ .*"Red Hat".* ]]; then

# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp
sudo firewall-cmd --add-masquerade --permanent
sudo firewall-cmd --reload

# https://github.com/FiloSottile/mkcert
sudo yum install nss-tools -y

# curl -JLO "https://dl.filippo.io/mkcert/latest?for=linux/amd64"
curl -JLO "https://dl.filippo.io/mkcert/v1.4.4?for=linux/amd64"
# curl -JLO  "https://github.com/FiloSottile/mkcert/releases/download/v1.4.4/mkcert-v1.4.4-linux-arm64"
sudo chmod +x mkcert-v*-linux-amd64
sudo mv mkcert-v*-linux-amd64 /usr/local/bin/mkcert
mkcert -install


if [[ $OSVERSION == "8.6" ]]; then
# RHEL8.6

sudo yum install ca-certificates-2021.2.50-80.0.el8_4.noarch -y

elif [[ $OSVERSION == "9.2" ]]; then
# RHEL 9.2

sudo yum install ca-certificates-2023.2.60_v7.0.306-90.1.el9_2.noarch -y

else # other versions of RHEL

sudo yum install ca-certificates -y

fi

elif [[ $OSDISTRO =~ .*"CentOS".* ]]; then


if [[ $OSVERSION == "7" ]]; then

# Centos 7

echo "Centos 7"


# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp
sudo firewall-cmd --add-masquerade --permanent
sudo firewall-cmd --reload

sudo yum install nss-tools -y

# curl -JLO "https://dl.filippo.io/mkcert/latest?for=linux/amd64"
curl -JLO "https://dl.filippo.io/mkcert/v1.4.4?for=linux/amd64"
# curl -JLO  "https://github.com/FiloSottile/mkcert/releases/download/v1.4.4/mkcert-v1.4.4-linux-arm64"
sudo chmod +x mkcert-v*-linux-amd64
sudo mv mkcert-v*-linux-amd64 /usr/local/bin/mkcert
mkcert -install

sudo yum install ca-certificates-2023.2.60_v7.0.306-72.el7_9.noarch -y

elif [[ $OSVERSION == "8" ]]; then

# Centos 8
echo "Centos 8"


# Centos 8 has reached the end of life, and the url to download packages is now different
# Assume the user has already done this if they are using Centos 8
# cd /etc/yum.repos.d/
# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y

# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y
# cd ~/


# sudo yum update -y

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp
sudo firewall-cmd --add-masquerade --permanent
sudo firewall-cmd --reload

sudo yum install nss-tools -y

# curl -JLO "https://dl.filippo.io/mkcert/latest?for=linux/amd64"
curl -JLO "https://dl.filippo.io/mkcert/v1.4.4?for=linux/amd64"
# curl -JLO  "https://github.com/FiloSottile/mkcert/releases/download/v1.4.4/mkcert-v1.4.4-linux-arm64"
sudo chmod +x mkcert-v*-linux-amd64
sudo mv mkcert-v*-linux-amd64 /usr/local/bin/mkcert
mkcert -install

sudo yum install ca-certificates -y

else

# other versions of Centos

echo "Unsupported Centos version"

exit 1

fi

else

# other distros

echo "Unsupported Linux version"

exit 1

fi

# certificates to access registry
sudo mkdir -p $REGISTRY_NODE_PATH/certs

mkcert localhost 127.0.0.1 $REGISTRY_IP $REGISTRY_FQDN $REGISTRY_NODE_NAME $REGISTRY_NODE_NAME.local

mv localhost+*-key.pem domain.key
mv localhost+*.pem domain.crt

sudo cp domain.key $REGISTRY_NODE_PATH/certs/domain.key
sudo cp domain.crt $REGISTRY_NODE_PATH/certs/domain.crt

# Copy certificates into default directories in OS where Kubernetes and Docker can access them. 
# the location of the certs folder must be specified as a variable for the registry container, e.g. $REGISTRY_NODE_PATH in this instance
sudo mkdir -p /etc/docker/certs.d/$REGISTRY_NODE_NAME:5000/
sudo cp $REGISTRY_NODE_PATH/certs/domain.crt /etc/docker/certs.d/$REGISTRY_NODE_NAME:5000/domain.crt 

if [[ $OSDISTRO =~ .*"Ubuntu".* ]]; then

sudo cp $REGISTRY_NODE_PATH/certs/domain.crt /usr/local/share/ca-certificates/
sudo update-ca-certificates

elif [[ $OSDISTRO =~ .*"Red Hat".* ]]; then

sudo cp $REGISTRY_NODE_PATH/certs/domain.crt /etc/ssl/certs 
sudo cp $REGISTRY_NODE_PATH/certs/domain.crt /etc/pki/tls/certs/ 
sudo cp $REGISTRY_NODE_PATH/certs/domain.crt /etc/pki/ca-trust/source/anchors/  
sudo update-ca-trust extract ##########

elif [[ $OSDISTRO =~ .*"CentOS".* ]]; then

# sudo cp $REGISTRY_NODE_PATH/certs/domain.crt /etc/pki/tls/certs/ ##########
sudo cp $REGISTRY_NODE_PATH/certs/domain.crt /etc/pki/ca-trust/source/anchors/  ##########
sudo update-ca-trust force-enable ##########
sudo update-ca-trust extract ##########

fi

sudo systemctl restart docker
sudo systemctl restart containerd

# THE SYSTEM SHOULD BE RESTARTED AFTER RUNNING THIS SCRIPT

