#!/usr/bin/env bash

# RUN THIS SCRIPT ON CONTROL NODE

while getopts p: opt; do
    case $opt in
        p) PARAM_FILE_PATH=${OPTARG} ;;
    esac
done

# set -eu -o pipefail # fail on error and report it, debug all lines
sudo true
test $? -eq 0 || exit 1 "You should have sudo privileges to run this script"

if [ -z "$PARAM_FILE_PATH" ] ; then
    read -p "Enter the path of parameter file: " PARAM_FILE_PATH
    if [ -f "$PARAM_FILE_PATH" ]; then
        echo Specified parameter file exists.
        source $PARAM_FILE_PATH
    else
        echo Specified parameter file does not exist.
        exit 1    
    fi  
else
    echo "41_create_registry.sh: Using supplied arguments..."
    source "$PARAM_FILE_PATH"
fi

export KUBECONFIG=$HOME/.kube/config

((count = 100))
while [[ $count -ne 0 ]] ; do
    kubectl get nodes 
    rc=$?
    if [[ $rc -eq 0 ]] ; then
        ((count = 1))
    fi
    ((count = count - 1))
done

if [[ $rc -ne 0 ]] ; then
    echo "Could not connect to kubectl after 100 attempts - stopping."
    exit 120
fi

node=$(kubectl get nodes --output=jsonpath='{.items[*].metadata.name}')
echo $node

node_ip=$(kubectl get nodes -o jsonpath='{.items[*].status.addresses[?(@.type=="InternalIP")].address}')
echo $node_ip

echo REGISTRY_NODE_PATH: $REGISTRY_NODE_PATH # this is the local path on the server where the registry will be held
echo REGISTRY_FQDN: $REGISTRY_FQDN
echo REGISTRY_URL: $REGISTRY_URL
echo REGISTRY_NODE_NAME: $REGISTRY_NODE_NAME
echo REGISTRY_IP: $REGISTRY_IP

# Uninstall the registry if it is already installed:
# echo "INFO: Any error message that says the registry is not loaded or is already installed may be ignored"

helm uninstall registry &>/dev/null
kubectl wait pods -n registry --selector=app.kubernetes.io/name=registry --for delete --timeout=120s &>/dev/null
sleep 90

# install the registry -- this command will simply give an error if it is already installed. 
helm install --set volumeNode=$REGISTRY_NODE_NAME --set deploymentNode=$REGISTRY_NODE_NAME --set localPath=$REGISTRY_NODE_PATH/ registry registry 

# wait until the registry is up and running before proceeeding
kubectl wait pods -n registry --selector=app.kubernetes.io/name=registry --for condition=Ready --timeout=300s

# Unauthenticated pull from public registry 
# AWS public registries are rate limited

# docker pull ubuntu:20.04
# docker pull centos:7.9.2009

# docker tag ubuntu:20.04 $REGISTRY_URL:5000/ubuntu:20.04
# docker tag centos:7.9.2009 $REGISTRY_URL:5000/centos:7.9.2009

# docker push "$REGISTRY_URL:5000"/ubuntu:20.04
# docker push "$REGISTRY_URL:5000"/centos:7.9.2009

# docker rmi "$REGISTRY_URL:5000"/ubuntu:20.04
# docker rmi "$REGISTRY_URL:5000"/centos:7.9.2009