#!/usr/bin/env bash

# RUN THIS SCRIPT ON THE CONTROL PLANE NODE

while getopts up: opt; do
    case $opt in
        u) UPDATE=true ;;
        p) PARAM_FILE_PATH=${OPTARG} ;;
    esac
done

# set -eu -o pipefail # fail on error and report it, debug all lines
sudo true
test $? -eq 0 || exit 1 "You should have sudo privileges to run this script"



source ./000_bbx_version.sh

if [ -z "$PARAM_FILE_PATH" ] ; then
    read -p "Enter the path of parameter file: " PARAM_FILE_PATH
    if [ -f "$PARAM_FILE_PATH" ]; then
        echo Specified parameter file exists
        source $PARAM_FILE_PATH
    else
        echo Specified parameter file does not exist.
        exit 1    
    fi  
else
    echo "60_install_app.sh: Using supplied arguments..."
    source "$PARAM_FILE_PATH"
fi

if [ -z ${UPDATE+x} ] ; then
    # CLEAN INSTALLATION
    echo "Installing BeetleboxCI in 10 seconds. Press Ctrl+C to stop..."
else
    # UPDATE
    echo "Updating BeetleboxCI in 10 seconds. Press Ctrl+C to stop..."
fi

sleep 10

export KUBECONFIG=$HOME/.kube/config

((count = 100))
while [[ $count -ne 0 ]] ; do
    kubectl get nodes 
    rc=$?
    if [[ $rc -eq 0 ]] ; then
        ((count = 1))
    fi
    ((count = count - 1))
    sleep 1
done

if [[ $rc -ne 0 ]] ; then
    echo "Could not connect to kubectl after 100 attempts - stopping."
    exit 120
fi

node=$(kubectl get nodes --output=jsonpath='{.items[*].metadata.name}')
echo $node

node_ip=$(kubectl get nodes -o jsonpath='{.items[*].status.addresses[?(@.type=="InternalIP")].address}')
echo $node_ip

echo BEETLEBOX_VERSION: $BEETLEBOX_VERSION

echo BBX_DEPLOY_NODE: $BBX_DEPLOY_NODE
echo BBX_INSTALL_PATH: $BBX_INSTALL_PATH
echo CONTROL_NODE_FQDN: $CONTROL_NODE_FQDN

echo NFS_SOURCE_PATH: $NFS_SOURCE_PATH
echo NFS_SERVER: $NFS_SERVER
echo NFS_SERVER_IP: $NFS_SERVER_IP

echo REGISTRY_NODE_PATH: $REGISTRY_NODE_PATH
echo REGISTRY_CERT_PATH: $REGISTRY_CERT_PATH
echo REGISTRY_IP: $REGISTRY_IP
echo REGISTRY_NODE_NAME: $REGISTRY_NODE_NAME
echo REGISTRY_FQDN: $REGISTRY_FQDN
echo REGISTRY_URL: $REGISTRY_URL


################ CLEAN INSTALLATIONS ONLY ################

if [ -z ${UPDATE+x} ] ; then

    helm uninstall beetleboxci &>/dev/null #uninstall if there is already an installation

    # These wait commands may give errors -- but it is not an issue and the error messages are suppressed
    kubectl wait pods  --selector=io.kompose.service=web --for delete --timeout=120s &>/dev/null
    kubectl wait pods  --selector=io.kompose.service=redis-worker --for delete --timeout=120s &>/dev/null
    kubectl wait pods  --selector=io.kompose.service=redis-scheduler --for delete --timeout=120s &>/dev/null
    kubectl wait pods  --selector=io.kompose.service=redis-server --for delete --timeout=120s &>/dev/null
    kubectl wait pods  --selector=io.kompose.service=nginx --for delete --timeout=120s &>/dev/null
    kubectl wait pods  --selector=io.kompose.service=db --for delete --timeout=120s &>/dev/null

fi

#########################################################

sudo crictl --runtime-endpoint=unix:///run/containerd/containerd.sock rmi  --prune

sudo rm -rf $BBX_INSTALL_PATH/vol_static/*

# Install the metrics server -- this file has been edited to add kubelet-insecure-tls=true to the command line arguments of metrics
kubectl apply -f metrics.yaml

sudo cp $REGISTRY_CERT_PATH/certs/domain.crt ./app/domain.crt # needed for the webserver to access the registry to display the list of images in the local registry

REGISTRY="$REGISTRY_URL:5000" TAG=$BEETLEBOX_VERSION docker-compose -f docker-compose.yml build
REGISTRY="$REGISTRY_URL:5000" TAG=$BEETLEBOX_VERSION docker-compose -f docker-compose.yml push

if [ -z ${UPDATE+x} ] ; then
    # CLEAN INSTALLATION
    helm install --set global.localPath=$BBX_INSTALL_PATH/ --set db.environment.data.BEETLEBOX_VERSION=$BEETLEBOX_VERSION --set db.environment.data.NFS_SOURCE=$NFS_SOURCE_PATH --set global.nfs.localPath=$NFS_SOURCE_PATH --set db.environment.data.REGISTRY_SERVER=$REGISTRY_URL:5000 --set global.volumeNode=$BBX_DEPLOY_NODE --set global.deploymentNode=$BBX_DEPLOY_NODE --set global.nfs.server=$NFS_SERVER --set db.environment.data.NFS_SERVER=$NFS_SERVER --set global.imageRegistry=$REGISTRY_URL:5000 --set db.environment.data.WORKFLOW_STATE=KUBERNETES --set db.environment.data.DJANGO_SECRET_KEY="SElhaShoLIum" --set db.environment.data.DJANGO_ALLOWED_HOSTS=* --set imageCredentials.registry=$REGISTRY_URL beetleboxci beetleboxci
else
    # UPDATE
    helm upgrade --set global.localPath=$BBX_INSTALL_PATH/ --set db.environment.data.BEETLEBOX_VERSION=$BEETLEBOX_VERSION --set db.environment.data.NFS_SOURCE=$NFS_SOURCE_PATH --set global.nfs.localPath=$NFS_SOURCE_PATH --set db.environment.data.REGISTRY_SERVER=$REGISTRY_URL:5000 --set global.volumeNode=$BBX_DEPLOY_NODE --set global.deploymentNode=$BBX_DEPLOY_NODE --set global.nfs.server=$NFS_SERVER --set db.environment.data.NFS_SERVER=$NFS_SERVER --set global.imageRegistry=$REGISTRY_URL:5000 --set db.environment.data.WORKFLOW_STATE=KUBERNETES --set db.environment.data.DJANGO_SECRET_KEY="SElhaShoLIum" --set db.environment.data.DJANGO_ALLOWED_HOSTS=* --set imageCredentials.registry=$REGISTRY_URL beetleboxci beetleboxci
fi

sleep 10

docker rmi "$REGISTRY_URL:5000"/web:$BEETLEBOX_VERSION
docker rmi "$REGISTRY_URL:5000"/redis:6.2.5-alpine
docker rmi "$REGISTRY_URL:5000"/nginx-unprivileged:1.21-alpine
docker rmi "$REGISTRY_URL:5000"/bitnami/postgresql:14.2.0 

kubectl wait pods  --selector=io.kompose.service=web --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-worker --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-scheduler --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-server --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=nginx --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=db --for condition=Ready --timeout=300s

WEBPOD=$(kubectl get pods --selector=io.kompose.service=web --output=jsonpath='{.items[*].metadata.name}')
echo $WEBPOD

WEBPODPHASE=$(kubectl get pods --selector=io.kompose.service=web --output=jsonpath='{.items[*].status.phase}')
echo $WEBPODPHASE

echo "collectstatic"
kubectl exec -it $WEBPOD -c web -- python3 manage.pyc collectstatic --noinput


## kubectl exec -it $WEBPOD -c web -- python3 manage.pyc makemigrations

echo "showmigrations"
kubectl exec -it $WEBPOD -c web -- python3 manage.pyc showmigrations 

echo "migrate"
kubectl exec -it $WEBPOD -c web -- python3 manage.pyc migrate --noinput

echo "showmigrations"
kubectl exec -it $WEBPOD -c web -- python3 manage.pyc showmigrations 

echo "create_public_keys"
kubectl exec -it $WEBPOD -c web -- python3 manage.pyc create_public_keys 

############### UPGRADES ###################
if [ -z ${UPDATE+x} ] ; then
    :
else
    sleep 60
    kubectl rollout restart deployment/beetleboxci-web
    kubectl rollout restart deployment/beetleboxci-nginx
    kubectl rollout restart deployment/beetleboxci-redis-scheduler
    kubectl rollout restart deployment/beetleboxci-redis-worker
    kubectl rollout restart deployment/beetleboxci-db
    kubectl rollout restart statefulset/beetleboxci-redis-server
    sleep 60
fi


#########################################################

kubectl wait pods  --selector=io.kompose.service=web --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-worker --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-scheduler --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=redis-server --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=nginx --for condition=Ready --timeout=300s
kubectl wait pods  --selector=io.kompose.service=db --for condition=Ready --timeout=300s


# Set up encryption for secrets:
ENC_CONFIG=/etc/kubernetes/pki/encryption.yaml

if [ ! -f "$ENC_CONFIG" ]; then

    ENC=$(head -c 32 /dev/urandom | base64)

    sudo bash -c "cat << EOF >> $ENC_CONFIG
apiVersion: apiserver.config.k8s.io/v1
kind: EncryptionConfiguration
resources:
  - resources:
    - secrets
    providers:
    - secretbox:
        keys:
        - name: key1
          secret: $ENC
    - identity: {}
EOF"

    sudo chmod 600 $ENC_CONFIG

fi

API_CONFIG=/etc/kubernetes/manifests/kube-apiserver.yaml
# API_SERVER=$(kubectl get pods -n kube-system --selector=component=kube-apiserver --output=jsonpath='{.items[*].metadata.name}')

if [ -f "$API_CONFIG" ]; then
    # check if an encryption provider is already defined in the API server
    if  sudo grep -q "encryption-provider-config=" "$API_CONFIG" ; then
        echo 'Encryption provider already exists'; 
    else
        sudo sed -i "s/- kube-apiserver/- kube-apiserver\n    - --encryption-provider-config=\/etc\/kubernetes\/pki\/encryption.yaml/g" $API_CONFIG
        
        # kubectl delete po -n kube-system $API_SERVER
        sleep 60

        ((count = 100))
        while [[ $count -ne 0 ]] ; do
            kubectl get nodes 
            rc=$?
            if [[ $rc -eq 0 ]] ; then
                ((count = 1))
            fi
            ((count = count - 1))
            sleep 1
        done

        # Convert existing secrets to use the current encryption provider
        kubectl get secrets --all-namespaces -o json | kubectl replace -f -
    fi
fi




