let clientSecret, stripe, cardElement;
const params = new URLSearchParams(window.location.search);
const subscriptionId = params.get('subscriptionId');

document.addEventListener('DOMContentLoaded', async () => {

  fetch(update_pm_get_sec, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      'X-CSRFToken': getCookie('csrftoken'),
    },
    credentials: 'same-origin',
    body: JSON.stringify({
      subscriptionId: subscriptionId,
    }),
  })
  .then((response) => response.json())
  .then((data) => {
    
    if(data.error) {
      console.error('Error:', data.error);
    }     
    else{
      clientSecret = data.clientSecret
    }
  })  

  fetch(update_pm_get_key, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      'X-CSRFToken': getCookie('csrftoken'),
    },
    credentials: 'same-origin',
  
  })
  .then((resp) => resp.json())
  .then((data) => {
    
    if(data.error) {
      console.error('Error:', data.error.message);
    }    
    else {
      stripe = Stripe(data.publishableKey)
      const elements = stripe.elements();

      cardElement = elements.create('card', {
        hidePostalCode: true,
      });    
      cardElement.mount('#card-element');      
    }
    
  });
  
  var myPostalCodeField = document.querySelector('#postal_code');
  myPostalCodeField.addEventListener('change', function(event) {
    cardElement.update({value: {postalCode: event.target.value}});
  });   
  
  const btn = document.querySelector('#submit-payment-btn');
  btn.addEventListener('click', async (e) => {
    e.preventDefault();
    
    if (btn.form.checkValidity() === false) {
      e.preventDefault();
      e.stopPropagation();
      btn.form.classList.add('was-validated');  
    }
    else {
      
      // button is disabled
      btn.classList.add('spin');
      btn.disabled = true;
      
      btn.form.firstElementChild.disabled = true;    
    
      const line1Input = document.getElementById('line1').value;
      const line2Input = document.getElementById('line2').value;
      const cityInput = document.getElementById('city').value;
      const postal_codeInput = document.getElementById('postal_code').value;
      const stateInput = document.getElementById('state').value;
      const countryInput = document.getElementById('country').value;
      const nameInput = document.getElementById('name').value;
      
      
      stripe.confirmCardPayment(clientSecret, {
        payment_method: {
          card: cardElement,
          billing_details: {
            name: nameInput,
            address: {
              line1: line1Input,
              line2: line2Input,
              city: cityInput,
              postal_code: postal_codeInput,
              state: stateInput,
              country: countryInput
            }
          },
        },

      }).then((result) => {
        if(result.error) {
          alert(result.error.message); // this creates a pop up message
          
          btn.classList.remove('spin');
          btn.disabled = false;
          btn.form.firstElementChild.disabled = false;        
          
        } else {
          
          set_address(line1Input, line2Input, cityInput, postal_codeInput, stateInput, countryInput)           
          set_new_def_pm(result.paymentIntent)
          
          setMessage('Success! Redirecting to your account in 3s.');
          
          setTimeout(() => {
            window.location.href = user_profile;
          }, 3 * 1000);        
          
        }
      });    
      
    }
    
  });
  
  const setMessage = (message) => {
    const messageDiv = document.querySelector('#messages');
    messageDiv.innerHTML += "<br>" + message;
  }  
  
});


function set_new_def_pm(paymentIntent) {
  
  return fetch(set_new_default_pm, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      'X-CSRFToken': getCookie('csrftoken'),
    },
    credentials: 'same-origin',
    body: JSON.stringify({
      paymentIntent: paymentIntent
    }),
  })
    .then((response) => response.json())
    
}


function set_address(line1Input, line2Input, cityInput, postal_codeInput, stateInput, countryInput) {
  
  return fetch(updatepm_set_customer_address, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      'X-CSRFToken': getCookie('csrftoken'),
    },
    credentials: 'same-origin',
    body: JSON.stringify({
      line1Input: line1Input,
      line2Input: line2Input,
      cityInput: cityInput,
      postal_codeInput: postal_codeInput,
      stateInput: stateInput,
      countryInput: countryInput
    }),
  })
    .then((response) => response.json())
    
}



function getCookie(name) {
    let cookieValue = null;
    if (document.cookie && document.cookie !== '') {
        const cookies = document.cookie.split(';');
        for (let i = 0; i < cookies.length; i++) {
            const cookie = cookies[i].trim();
            // Does this cookie string begin with the name we want?
            if (cookie.substring(0, name.length + 1) === (name + '=')) {
                cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                break;
            }
        }
    }
    return cookieValue;
}




