#!/usr/bin/env bash

# This script should be run on the control plane node - which has kubectl set up to access the Kubernetes API server. 
# If the user has manually set up kubectl on the registry node too, then this script can be run on the registry node.

curl -v -sSL -X DELETE "https://{{registry}}/v2/{{name}}/manifests/$(
    curl -sSL -I \
        -H "Accept: application/vnd.docker.distribution.manifest.v2+json" \
        "https://{{registry}}/v2/{{name}}/manifests/$(
            curl -sSL "https://{{registry}}/v2/{{name}}/tags/list" | jq -r '.tags[0]'
        )" \
    | awk '$1 == "docker-content-digest:" { print $2 }' \
    | tr -d $'\r' \
)"

REGPOD=$(kubectl get pods -n registry --selector=app.kubernetes.io/name=registry --output=jsonpath='{.items[*].metadata.name}')
kubectl exec -it -n registry  $REGPOD -- bin/registry garbage-collect --delete-untagged /etc/docker/registry/config.yml

# The registry pod needs to be restarted for the changes to take effect
# Deleting the registry pod will automatically restart it. 
kubectl delete po -n registry $REGPOD

# The user may need to manually run the following command on the node which hosts the registry in order to delete any orphaned files after an image has been deleted. 
# sudo rm -rf REGISTRY_DIRECTORY/docker/registry/v2/repositories/{{name}}


