#!/usr/bin/env bash

echo "$#"

if [ -z "$1" ] ; then
    echo "00_init_control_node.sh: Using default settings..."

    BBX_INSTALL_PATH=/mnt/beetleboxci
else
    echo "00_init_control_node.sh: Using supplied arguments..."

    source "$1"
fi

echo BBX_INSTALL_PATH: $BBX_INSTALL_PATH

OSDISTRO=$( $(compgen -G "/etc/*release" > /dev/null) && cat /etc/*release | grep ^NAME | tr -d 'NAME="' || echo "${OSTYPE//[0-9.]/}")
OSVERSION=$( cat /etc/*release | grep ^VERSION_ID | tr -d 'VERSION_ID="'   )

sudo mkdir -p $BBX_INSTALL_PATH/certs

if [[ $OSDISTRO =~ .*"Ubuntu".* ]]
then

sudo apt-get install ufw -y

sudo ufw allow 179/tcp
sudo ufw allow 2379/tcp
sudo ufw allow 2380/tcp
sudo ufw allow 6443/tcp
sudo ufw allow 10250/tcp
sudo ufw allow 10251/tcp
sudo ufw allow 10252/tcp
sudo ufw allow 32767/tcp
sudo ufw allow 22/tcp
# sudo ufw allow 8000/tcp
# sudo ufw allow 8080/tcp
# sudo ufw allow 80/tcp
sudo ufw allow 2049/tcp
sudo ufw allow 5000/tcp

sudo ufw allow 53/tcp
sudo ufw allow 53/udp
sudo ufw allow 5432/tcp

sudo apt update -y
sudo apt install openssh-server -y
sudo apt install net-tools curl -y 

sudo systemctl enable ssh
sudo systemctl start ssh

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF


# sudo swapoff -a
# sudo rm /swap.img
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab
# systemctl mask swap.img.swap

sudo apt-get update -y


if [[ $OSVERSION = "22.04" ]] then

# UBUNTU 22

sudo apt-get install -y apt-transport-https ca-certificates

# echo "deb [signed-by=/etc/apt/keyrings/kubernetes-archive-keyring.gpg] https://apt.kubernetes.io/ kubernetes-xenial main" | sudo tee /etc/apt/sources.list.d/kubernetes.list # deprecated
# sudo curl -fsSLo /etc/apt/keyrings/kubernetes-archive-keyring.gpg https://dl.k8s.io/apt/doc/apt-key.gpg # deprecated

# Download the public signing key for the Kubernetes package repositories. The same signing key is used for all repositories so you can disregard the version in the URL:
curl -fsSL https://pkgs.k8s.io/core:/stable:/v1.28/deb/Release.key | sudo gpg --dearmor -o /etc/apt/keyrings/kubernetes-apt-keyring.gpg

# Add the appropriate Kubernetes apt repository. If you want to use Kubernetes version different than v1.28, replace v1.28 with the desired minor version in the command below:
echo 'deb [signed-by=/etc/apt/keyrings/kubernetes-apt-keyring.gpg] https://pkgs.k8s.io/core:/stable:/v1.28/deb/ /' | sudo tee /etc/apt/sources.list.d/kubernetes.list

sudo apt-get update -y
sudo apt-get install -y kubelet=1.28.0-00 kubeadm=1.28.0-00 kubectl=1.28.0-00
sudo apt-mark hold kubelet kubeadm kubectl

sudo apt-get remove -y docker docker-engine docker.io containerd runc

sudo apt-get install gnupg=2.2.27-3ubuntu2.1 lsb-release=11.1.0ubuntu4 -y

curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg
echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo apt-get update -y

sudo apt-get install iproute2 -y

sudo apt-get install containerd.io=1.6.24-1 -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable containerd

# EDIT THE TOML FILE /etc/containerd/config.toml and set the systemd cgroup to true 
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

sudo apt autoremove -y

sudo systemctl enable kubelet
sudo systemctl restart kubelet

# sudo apt install -y nfs-common

else

sudo apt-get install -y apt-transport-https ca-certificates

# echo "deb [signed-by=/etc/apt/keyrings/kubernetes-archive-keyring.gpg] https://apt.kubernetes.io/ kubernetes-xenial main" | sudo tee /etc/apt/sources.list.d/kubernetes.list # deprecated
# sudo curl -fsSLo /etc/apt/keyrings/kubernetes-archive-keyring.gpg https://dl.k8s.io/apt/doc/apt-key.gpg # deprecated

# Download the public signing key for the Kubernetes package repositories. The same signing key is used for all repositories so you can disregard the version in the URL:
curl -fsSL https://pkgs.k8s.io/core:/stable:/v1.28/deb/Release.key | sudo gpg --dearmor -o /etc/apt/keyrings/kubernetes-apt-keyring.gpg

# Add the appropriate Kubernetes apt repository. If you want to use Kubernetes version different than v1.28, replace v1.28 with the desired minor version in the command below:
echo 'deb [signed-by=/etc/apt/keyrings/kubernetes-apt-keyring.gpg] https://pkgs.k8s.io/core:/stable:/v1.28/deb/ /' | sudo tee /etc/apt/sources.list.d/kubernetes.list

sudo apt-get update -y
sudo apt-get install -y kubelet=1.28.0-00 kubeadm=1.28.0-00 kubectl=1.28.0-00
sudo apt-mark hold kubelet kubeadm kubectl

sudo apt-get remove -y docker docker-engine docker.io containerd runc
sudo apt-get install gnupg lsb-release -y

curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg
echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo apt-get update -y

sudo apt-get install iproute2 -y

sudo apt-get install containerd.io -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable containerd

# EDIT THE TOML FILE /etc/containerd/config.toml and set the systemd cgroup to true 
FILE2=/etc/containerd/config.toml
if [ -f "$FILE2" ]; then
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

sudo apt autoremove -y

sudo systemctl enable kubelet
sudo systemctl restart kubelet

# sudo apt install -y nfs-common

fi

elif [[ $OSDISTRO =~ .*"Red Hat".* ]] then

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port={6443,2379,2380,10250,10251,10252}/tcp

sudo firewall-cmd --zone=public --permanent --add-port 179/tcp
sudo firewall-cmd --zone=public --permanent --add-port 10250/tcp
sudo firewall-cmd --zone=public --permanent --add-port 32767/tcp
sudo firewall-cmd --zone=public --permanent --add-port 22/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8000/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8080/tcp
sudo firewall-cmd --zone=public --permanent --add-port 80/tcp
sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp

sudo firewall-cmd --zone=public --permanent --add-port 53/tcp
sudo firewall-cmd --zone=public --permanent --add-port 53/udp
sudo firewall-cmd --zone=public --permanent --add-port 5432/tcp

sudo firewall-cmd --reload

sudo dnf install openssh-server curl -y 

sudo systemctl start sshd
sudo systemctl enable sshd

if [[ $OSVERSION == "8.6" ]]  then
# RHEL8.6

sudo yum install ca-certificates-2021.2.50-80.0.el8_4.noarch -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install yum-utils-4.0.21-11.el8.noarch -y
sudo dnf install iproute-tc-5.15.0-4.el8 -y

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

sudo yum install containerd.io-1.6.7-3.1.el8 -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE0" ]; then
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet
# sudo systemctl restart kubelet

sudo systemctl enable --now docker.service

elif [[ $OSVERSION == "9.2" ]] then
# RHEL9.2

sudo yum install ca-certificates-2023.2.60_v7.0.306-90.1.el9_2.noarch -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install yum-utils-4.3.0-5.el9_2.noarch -y
sudo dnf install iproute-tc-6.1.0-1.el9.x86_64 -y

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

sudo yum install containerd.io-1.6.24-3.1.el9.x86_64 -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable --now containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE0" ]; then
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet
# sudo systemctl restart kubelet

sudo systemctl enable --now docker.service

else

sudo yum install ca-certificates -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install yum-utils -y
sudo yum install iproute-tc -y

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

sudo yum install containerd.io -y
sudo yum install docker-ce -y

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable --now containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE0" ]; then
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes

sudo systemctl enable --now kubelet
# sudo echo 'KUBELET_EXTRA_ARGS="--fail-swap-on=false"' > /etc/sysconfig/kubelet
# sudo systemctl restart kubelet

sudo systemctl enable --now docker.service

fi

elif [[ $OSDISTRO =~ .*"CentOS".* ]] then

# Centos 8 has reached the end of life, and the url to download packages is now different
# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*

sudo yum install firewalld -y 
sudo systemctl start firewalld
sudo systemctl enable firewalld

sudo firewall-cmd --zone=public --permanent --add-port 179/tcp
sudo firewall-cmd --zone=public --permanent --add-port 6443/tcp
sudo firewall-cmd --zone=public --permanent --add-port 10250/tcp
sudo firewall-cmd --zone=public --permanent --add-port 32767/tcp
sudo firewall-cmd --zone=public --permanent --add-port 22/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8000/tcp
sudo firewall-cmd --zone=public --permanent --add-port 8080/tcp
sudo firewall-cmd --zone=public --permanent --add-port 80/tcp
sudo firewall-cmd --zone=public --permanent --add-port 2049/tcp
sudo firewall-cmd --zone=public --permanent --add-port 5000/tcp

sudo firewall-cmd --zone=public --permanent --add-port 53/tcp
sudo firewall-cmd --zone=public --permanent --add-port 53/udp
sudo firewall-cmd --zone=public --permanent --add-port 5432/tcp

sudo firewall-cmd --reload

sudo yum install openssh-server curl -y 

sudo systemctl start sshd
sudo systemctl enable sshd

if [[ $OSVERSION == "7" ]] then

sudo yum install ca-certificates-2023.2.60_v7.0.306-72.el7_9.noarch -y

cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install -y yum-utils
sudo yum install -y iproute-4.11.0-30.el7.x86_64

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

# sudo yum install -y docker-ce 
curl -fsSL https://get.docker.com/ | sh
sudo yum install -y containerd.io.x86_64

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE0" ]; then
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes
sudo systemctl enable --now kubelet

sudo systemctl enable --now docker.service

elif [[ $OSVERSION == "8" ]] then

# Centos 8 has reached the end of life, and the url to download packages is now different
# Assume the user has already done this if they are using Centos 8
# cd /etc/yum.repos.d/
# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y

# sudo sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
# sudo sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
# sudo yum update -y
# cd ~/

sudo yum install ca-certificates -y


cat <<EOF | sudo tee /etc/modules-load.d/k8s.conf
br_netfilter
EOF

cat <<EOF | sudo tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF

sudo sysctl --system

# sudo swapoff -a
# sudo sed -i '/ swap / s/^\(.*\)$/#\1/g' /etc/fstab

sudo yum remove docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine podman runc

cat <<EOF | sudo tee /etc/modules-load.d/containerd.conf
overlay
br_netfilter
EOF

# Setup required sysctl params, these persist across reboots.
cat <<EOF | sudo tee /etc/sysctl.d/99-kubernetes-cri.conf
net.bridge.bridge-nf-call-iptables  = 1
net.ipv4.ip_forward                 = 1
net.bridge.bridge-nf-call-ip6tables = 1
EOF

# Apply sysctl params without reboot
sudo sysctl --system

sudo yum install -y yum-utils
sudo dnf install -y iproute-tc

sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo

# sudo yum install -y docker-ce 
curl -fsSL https://get.docker.com/ | sh
sudo yum install -y containerd.io.x86_64

sudo rm /etc/containerd/config.toml

sudo mkdir -p /etc/containerd
containerd config default | sudo tee /etc/containerd/config.toml

sudo systemctl enable containerd

# EDIT THE TOML FILE /etc/containerd/config.toml
FILE2=/etc/containerd/config.toml
if [ -f "$FILE0" ]; then
    sudo sed -i '/SystemdCgroup/s/false/true/g' $FILE2
fi

sudo systemctl daemon-reload
sudo systemctl restart containerd

# deprecated
cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
[kubernetes]
name=Kubernetes
baseurl=https://packages.cloud.google.com/yum/repos/kubernetes-el7-\$basearch
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
exclude=kubelet kubeadm kubectl
EOF

# # new repo url 
# # This overwrites any existing configuration in /etc/yum.repos.d/kubernetes.repo
# cat <<EOF | sudo tee /etc/yum.repos.d/kubernetes.repo
# [kubernetes]
# name=Kubernetes
# baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
# enabled=1
# gpgcheck=1
# gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key
# EOF

# Set SELinux in permissive mode (effectively disabling it)
# sudo setenforce 0
# sudo sed -i 's/^SELINUX=enforcing$/SELINUX=permissive/' /etc/selinux/config

sudo yum install -y kubelet-1.28.0 kubeadm-1.28.0 kubectl-1.28.0 --disableexcludes=kubernetes
sudo systemctl enable --now kubelet

sudo systemctl enable --now docker.service

else

# other versions of Centos

echo "Unsupported Centos version"

exit 1

fi 

else

# other distros

echo "Unsupported Linux version"

exit 1

fi

