import React, { useMemo } from 'react';

import {getCookie} from "./shared_functions.js"
import running_icon from './img/beetlebox-running-icon.png';

import { Modal, Container, Row, Col, Alert} from 'react-bootstrap';
import {Breadcrumb} from './shared_classes.js'

import { TextField } from '@material-ui/core';

import CodeMirror from '@uiw/react-codemirror';

// import { basicSetup } from "codemirror"
import { StreamLanguage } from "@codemirror/language"
import { linter, lintGutter } from "@codemirror/lint"
import { yaml } from "@codemirror/legacy-modes/mode/yaml"
import parser from "js-yaml";

import debounce from 'lodash.debounce'

import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 
// const queryClient2 = new QueryClient();

const twentyFourHoursInMs = 1000 * 60 * 60 * 24;
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      refetchOnWindowFocus: false,
      refetchOnmount: false,
      refetchOnReconnect: false,
      retry: false,
      staleTime: twentyFourHoursInMs,
    },
  },
});

export default class ConfigEditorView extends React.Component {
  constructor(props) {
    super(props);
    var pathArray = window.location.pathname.split('/');

    // this.config_url = window.location.protocol + '//' + window.location.host + '/api/config?pipeline=' + window.location.search.split('pipeline=')[1] + '&branch=' + window.location.search.split('branch=')[1];
    this.config_url = window.location.protocol + '//' + window.location.host + '/api/config'
  }
  
  render() {
    
    const queryString = window.location.search;
    const urlParams = new URLSearchParams(queryString);
    const pipeline_id = urlParams.get('pipeline')
    const branch = urlParams.get('branch')

    const conf_url = this.config_url + '?pipeline=' + pipeline_id + '&branch=' + branch
    
    return (
      <div>
        <QueryClientProvider client={queryClient}>
          <FetchConfig config_url={conf_url} pipeline_id={pipeline_id} branch={branch} />
        </QueryClientProvider>
      </div>
    );
    
  }
}

function FetchConfig(props) {
  const { isLoading, error, data } = useQuery('config_data', () =>
     fetch(props.config_url).then(res =>
       res.json()
     ),
  );
  
  if (isLoading) return (<div className="container-xl px-4 mt-5">
  <div className='row justify-content-md-center'>
    <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
  </div>
  </div>);
  
  if (error) return(
    <div>
    <h4>An error has occurred: {error.message}</h4>
    </div>   
  );
  if (data.error) return( 
    <div>
    <h4> {data.error.name}: {data.error.message} </h4>
    </div>
  );
  
  let pageList=[['Pipelines', '/pipelines'], [data.pipeline_name, '/pipelines/' + props.pipeline_id], [data.pipeline_name + ' (' + props.branch + ') Editor', window.location.pathname + '?pipeline=' + props.pipeline_id + '&branch=' + props.branch ]];
  
  return (
  <Container fluid className="px-4 mt-5">
    <Row>
        <Breadcrumb pageList={pageList} />
    </Row>  
    <Row>
        <h1 className="mb-0">Config for {data.pipeline_name} ({props.branch})</h1>
    </Row>

    
    <Row className="mx-0">
    
      <ConfigText config_data={data.config_text} example_data={data.example_text} pipeline_id={props.pipeline_id} branch={props.branch}/>
      
    </Row>  
    
  </Container>
  );
}

class ConfigText extends React.Component {
  constructor(props) {
    super(props);

    if  (this.props.config_data.length<1){
        this.state = {
          show: true, // show pop up message about example config text
          name: 'CodeMirror',
          code: this.props.config_data,
          commit: '',
          error_text: '',
          info_text: '',
        };
    } else{
        this.state = {
          show: false, // don't show pop up message about example config text
          name: 'CodeMirror',
          code: this.props.config_data,
          commit: '',
          error_text: '',
          info_text: '',
        };
    }
    
    this.handleSubmit = this.handleSubmit.bind(this);    
    this.updateCommit = this.updateCommit.bind(this);
    this.debouncedupdateCommit = debounce(this.debouncedupdateCommit, 500)
    this.debouncedUpdateCode = debounce(this.debouncedUpdateCode, 500)    
  }
  
  handleYes(){
    this.setState({code:this.props.example_data});
    this.setState({show:false});
  }    
  handleNo(){
    this.setState({show:false});
  }

  handleCodeChange = (newCode) => {
    // Immediately update the state
    this.setState({
      code: newCode
    })
    // Execute the debounced method
    this.debouncedUpdateCode(newCode)
  }  
  debouncedUpdateCode = (newCode) => {
    // Delayed logic goes here
    this.handleCodeChange(newCode)
  }

  updateCommit(event) {    
		event.preventDefault()
    
    this.setState({
      commit: event.target.value
      
    }); 
    event.preventDefault()
  }	
  debouncedupdateCommit = (event) => {
    // Delayed logic goes here
    this.updateCommit(event)
  }
	
  handleSubmit(e) {
    e.preventDefault();
    
    var commit;
    if (this.state.commit == ''){
      commit = 'config.yaml updated by BeetleboxCI'
    } else {
      commit = this.state.commit
    }
    
    let data = JSON.stringify({
        name: "push_branch_",
        br_: this.props.branch,
        id_: this.props.pipeline_id,
        conf_data_: this.state.code,
        commit_message_: commit
        
    });
      
    const requestOptions = {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'X-CSRFToken': getCookie('csrftoken'),
          },
          credentials: 'same-origin',
          body: data
    };

    this.setState({error_text: ""});
    this.setState({info_text: "Saving..."});
          
    fetch('', requestOptions)
            .then(response => response.json())
            .then((data) => {
                // console.log(data)

                this.setState({info_text: ""});                 
                if (data.config_error){
                  this.setState({error_text: data.error_details});
                // alert(data.error_details)
                  
                } else if (data.push_error){
                  this.setState({error_text:"Error while pushing to remote repository. Please ensure that you have the correct permissions to push to this repository."});

                } else {
                  window.location.href = data.redirect;
                }
            });
  }	
  
  render(){    
    
    let default_config_modal =     <>
  
      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header >
          <Modal.Title>Empty config file</Modal.Title>
        </Modal.Header>
        <Modal.Body> The config file for this branch is currently empty. Do you want to add an example config file for the '{this.props.branch}'' branch?</Modal.Body>
        <Modal.Footer>
          <button className="btn btn-primary btn-sm"  onClick={() => this.handleYes()}>
              Yes
          </button>          
          <button className="btn btn-primary btn-sm"  onClick={() => this.handleNo()}>
              No
          </button>
        </Modal.Footer>
      </Modal>
    </>;    
    
    let error_output = <div></div>;
    if (this.state.error_text.length!=0){
      error_output =<div>
            <Alert key="danger" variant="danger">
              {this.state.error_text}
            </Alert>
        </div>;
    }

    let info_output = <div></div>;
    if (this.state.info_text.length!=0){
      info_output =<div>
            <Alert key="primary" variant="primary"><img src={running_icon} alt="loading_icon" style={ {width: '20px'}} />&nbsp;{this.state.info_text}</Alert>
        </div>;
    }    
    
    const yamlLinter = linter((view) => {
      const diagnostics = [];
    
      try {
        parser.load(view.state.doc);
      } catch (e) {
        const loc = e.mark;
        const from = loc ? loc.position : 0;
        const to = from;
        const severity = "error";
    
        diagnostics.push({
          from,
          to,
          message: e.message,
          severity
        });
      }
    
      return diagnostics;
    });

    let code_mirror=<CodeMirror
    value={this.state.code}
    height="500px"
    theme="dark"                
    onChange={this.debouncedUpdateCode}                
    extensions={[StreamLanguage.define(yaml) , lintGutter(), yamlLinter]}                
    />

    var config_card = <div className="card p-0 mb-4 mt-4">
        <div className="card-header" style={{color: "black" }}>
          <Row >
            <Col md="auto me-auto"> config.yaml </Col>
            <Col md="auto">  
              <form onSubmit={this.handleSubmit}>
                <button className="btn btn-primary btn-sm" type="submit" id="push_branch_" name="push_branch_">Save and commit</button>
              </form>
            </Col>
          </Row>
        </div>
    
        <div className="card-body p-3">
        
            <TextField 

              fullWidth
              multiline={true} 
              minRows={3} 
              id="filled-basic" 
              placeholder="config.yaml updated by BeetleboxCI" 
              label="Enter a commit message (optional)" 
              variant="filled" 
              onChange={this.debouncedupdateCommit} 
            />
          
            <Row className="mt-3">
            {error_output}
            </Row>
            
            <Row className="mt-1">
            {info_output}
            </Row>

            <Row>          
            {code_mirror}              
            </Row>
        </div>
    </div>
    return(<React.Fragment>
    
    {default_config_modal}
    {config_card}
    
    </React.Fragment>);
  }
}






