import React from 'react';

// import ellipse from './img/ellipse.png';
import running_icon from './img/beetlebox-running-icon.png';
// import static_running_icon from './img/beetleboxci-static-running-icon.svg';
import pending_icon from './img/beetleboxci-pending-icon.png';
import static_pending_icon from './img/beetleboxci-static-pending-icon.svg';
import failure_icon from './img/beetleboxci-failure-icon.svg';
import pass_icon from './img/beetleboxci-pass-icon.svg';
import stop_icon from './img/stop.svg';

// import clipboard from './img/clipboard.svg'
// import download from './img/download.svg'
// import user_portal_example from './img/user-portal-example.png';
// import license_example from './img/license-example.png';
import documentation_example from './img/documentation-example.png';
import introduction_welcome_image from './img/introduction-welcome-image.png';

// import styles from './custom.css'

import StopWatch from "./stopwatch.js"
import {formatDate, getCookie, humanFileSize} from "./shared_functions.js"

import { Modal, Button, Table, Container, Row, Col, Card, Alert, Pagination} from 'react-bootstrap';

import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 


export default class DashboardView extends React.Component {
  constructor(props) {
    super(props);
    var pathArray = window.location.pathname.split('/');

    this.update_check_url = window.location.protocol + '//' + window.location.host + '/api/dashboard_update_check';
    this.resource_url = window.location.protocol + '//' + window.location.host + '/api/dashboard_resources';
    this.dashboard_url = window.location.protocol + '//' + window.location.host + '/api/dashboard_pipelines';

  }
  
  
  render() {
    return (
      <div>
          <QueryClientProvider client={queryClient}>
            <Container fluid className="px-4 mt-5">
              <Row>
                  <h1 className="mb-4">Dashboard</h1>
              </Row>              
              <FetchUpdateInfo update_check_url={this.update_check_url} />
              <FetchResourceInfo resource_url={this.resource_url} />
              <FetchPipelineInfo dashboard_url={this.dashboard_url} />
            </Container>
          </QueryClientProvider>
      </div>
    );
  }
}

const queryClient = new QueryClient();

function FetchUpdateInfo(params) {

  const fetchData = async () => {
    const res = await fetch(params.update_check_url);
    return res.json();
  };

  const { isLoading, error, data } = useQuery("update_check_data", 
                                              fetchData);  

  let error_alert_system=<div></div>;
  let update_alert_system=<div></div>;    

  if (isLoading) {

  
  }
 
  else if (error) return(
      <div>
      <h4>An error has occurred: {error.message}</h4>
      </div>   
  );

  else if (data.error) {

    return( 
      <div>
      <h4> {data.error.name}: {data.error.message} </h4>
      </div>
    );
  }

  else {   
    
    if (data.critical_error_message!='None'){
      error_alert_system=<div>
          <Alert key="danger" variant="danger">
            {data.critical_error_message}
          </Alert>
      </div>;
    
    } 
    if (data.update_set==true){
        
      update_alert_system=<div>
          <Alert key="success" variant="success">
          BeetleboxCI {data.server_version} available (please stop all running jobs before updating): <a href={"https://bbx-public-bucket.s3.amazonaws.com/beetleboxci/BeetleboxCIv"+data.server_version_url_string+".run"}>Download here</a>
          </Alert>
      </div>;
    }  

  }

  return (   
  <div>
    {error_alert_system}
    {update_alert_system}        
  </div>
  );
}

function FetchResourceInfo(params) {
  const fetchData = async () => {
    const res = await fetch(params.resource_url);
    return res.json();
  };

  const { isLoading, error, data } = useQuery("resource_data", fetchData);


  let error_alert_system=<div></div>;
  let update_alert_system=<div></div>;  

  let resource_card=<div></div>;    
  let storage_card=<div></div>;  
  let status_card=<div></div>;  
  let users_card=<div></div>;  

  let compute_info=<div></div>;  
  let storage_info=<div></div>;  
  let status_info=<div></div>;  
  let user_info=<div></div>;    

  if (isLoading) {

    compute_info=<React.Fragment>
    <Row>
    <div className="h5"><img src={running_icon} alt="loading_icon" style={ {width: '40px'}} />&nbsp;&nbsp;Loading</div>
    </Row>
    </React.Fragment>

    storage_info=<React.Fragment>
    <Row>
    <div className="h5"><img src={running_icon} alt="loading_icon" style={ {width: '40px'}} />&nbsp;&nbsp;Loading</div>
    </Row>
    </React.Fragment>

    status_info=<React.Fragment>
    <Row>
    <div className="h5"><img src={running_icon} alt="loading_icon" style={ {width: '40px'}} />&nbsp;&nbsp;Loading</div>
    </Row>
    </React.Fragment>

    user_info=<React.Fragment>
    <Row>
    <div className="h5"><img src={running_icon} alt="loading_icon" style={ {width: '40px'}} />&nbsp;&nbsp;Loading</div>
    </Row>
    </React.Fragment>    
  
  }
 
  else if (error) return(
      <div>
      <h4>An error has occurred: {error.message}</h4>
      </div>   
  );

  else if (data.error) {

    return( 
      <div>
      <h4> {data.error.name}: {data.error.message} </h4>
      </div>
    );
  }

  else {

    compute_info=<React.Fragment>
      <Row>
      <Col sm={4}>
          <div className="h7">CPU: </div>                
      </Col>
      <Col sm={8}>
          <div className="h5">{data.CPU_usage}/{data.CPU_allocatable}</div>
      </Col>
      </Row>
      <Row>
          <Col sm={4}>
              <div className="h7">RAM: </div>                
          </Col>
          <Col sm={8}>
              <div className="h5">{data.RAM_usage}/{data.RAM_allocatable}</div>
          </Col>
      </Row>
    </React.Fragment>

    let container_human_storage=humanFileSize(data.container_size);
    let artifact_human_storage=humanFileSize(data.artifact_size);
    let volume_human_storage=humanFileSize(data.volume_size);
    let total_human_storage=humanFileSize(data.total_size);

    storage_info=<React.Fragment>
      <Row>

          <div className="h3">{total_human_storage}</div>
      </Row>             
      <Row>
          <Col>
              <div className="h7">Artifacts: </div>                
          </Col>
          <Col>
              <div className="h5">{artifact_human_storage}</div>
          </Col>
      </Row>   
      <Row>
          <Col>
              <div className="h7">Volumes: </div>                
          </Col>
          <Col>
              <div className="h5">{volume_human_storage}</div>
          </Col>
      </Row>

    </React.Fragment>

    status_info=<React.Fragment>
      <Row>
          <Col>
              <div className="h7">{data.pipelines}</div>                
          </Col>
      </Row>

    </React.Fragment>
    
    user_info=<React.Fragment>
      <Row>
          <Col>
              <div className="h7">Team Members: {data.team_members} </div>                
          </Col>
      </Row>
    </React.Fragment>
  }  

  resource_card= 
  <Col xl="3" className="mb-4 align-items-start">
      <Card className="h-100">
      
          <Card.Header className="card-header">
              Compute Usage
          </Card.Header>    
          
          <Card.Body className="card-body">
              <Row>
                  <Col xs={12} md={8}>
                    {compute_info}
                  </Col>
                  <Col xs={6} md={4}>
                      <Row style={{ height: "100px" }} className="align-items-center">                        
                          <svg xmlns="http://www.w3.org/2000/svg" fill = "#0061f2" viewBox="0 0 512 512" width="64" height="64">
                              <path d="M160 352h192V160H160V352zM448 176h48C504.8 176 512 168.8 512 160s-7.162-16-16-16H448V128c0-35.35-28.65-64-64-64h-16V16C368 7.164 360.8 0 352 0c-8.836 0-16 7.164-16 16V64h-64V16C272 7.164 264.8 0 256 0C247.2 0 240 7.164 240 16V64h-64V16C176 7.164 168.8 0 160 0C151.2 0 144 7.164 144 16V64H128C92.65 64 64 92.65 64 128v16H16C7.164 144 0 151.2 0 160s7.164 16 16 16H64v64H16C7.164 240 0 247.2 0 256s7.164 16 16 16H64v64H16C7.164 336 0 343.2 0 352s7.164 16 16 16H64V384c0 35.35 28.65 64 64 64h16v48C144 504.8 151.2 512 160 512c8.838 0 16-7.164 16-16V448h64v48c0 8.836 7.164 16 16 16c8.838 0 16-7.164 16-16V448h64v48c0 8.836 7.164 16 16 16c8.838 0 16-7.164 16-16V448H384c35.35 0 64-28.65 64-64v-16h48c8.838 0 16-7.164 16-16s-7.162-16-16-16H448v-64h48C504.8 272 512 264.8 512 256s-7.162-16-16-16H448V176zM384 368c0 8.836-7.162 16-16 16h-224C135.2 384 128 376.8 128 368v-224C128 135.2 135.2 128 144 128h224C376.8 128 384 135.2 384 144V368z"/>
                          </svg>  
                      </Row>
                  </Col>
              </Row>
          </Card.Body>
      </Card>
  </Col>;

  storage_card= 
      <Col xl="3" className="mb-4 align-items-stretch">
          <Card className="h-100">
          
              <Card.Header className="card-header">
                  Storage Used
              </Card.Header>            
              
              <Card.Body className="card-body">
                  <Row>
                      <Col xs={12} md={8}>
                        {storage_info}
                      </Col>
                      <Col xs={6} md={4}>
                          <Row style={{ height: "100px" }} className="align-items-center">                        
                      
                              <svg className="bi me-3" xmlns="http://www.w3.org/2000/svg" fill = "#0061f2" viewBox="0 0 576 512" width="64" height="64">
                                  <path d="M160 48C160 21.49 181.5 0 208 0H256V80C256 88.84 263.2 96 272 96H304C312.8 96 320 88.84 320 80V0H368C394.5 0 416 21.49 416 48V176C416 202.5 394.5 224 368 224H208C181.5 224 160 202.5 160 176V48zM96 288V368C96 376.8 103.2 384 112 384H144C152.8 384 160 376.8 160 368V288H208C234.5 288 256 309.5 256 336V464C256 490.5 234.5 512 208 512H48C21.49 512 0 490.5 0 464V336C0 309.5 21.49 288 48 288H96zM416 288V368C416 376.8 423.2 384 432 384H464C472.8 384 480 376.8 480 368V288H528C554.5 288 576 309.5 576 336V464C576 490.5 554.5 512 528 512H368C341.5 512 320 490.5 320 464V336C320 309.5 341.5 288 368 288H416z"/>
                              </svg>  
                          </Row>
                      </Col>
                  </Row>
              </Card.Body>
          </Card>
      </Col>;

  status_card= 
      <Col xl="3" className="mb-4 pr-0 align-items-stretch">
          <Card className="h-100">
          
              <Card.Header className="card-header">
                  Currently Running
              </Card.Header>
              
              <Card.Body className="card-body">
                  <Row>
                      <Col xs={12} md={8}>
                        {status_info}
                      </Col>
                      <Col xs={6} md={4}>
                          <Row style={{ height: "100px" }} className="align-items-center">
                              <svg xmlns="http://www.w3.org/2000/svg" fill = "#0061f2" viewBox="0 0 448 512" width="64" height="64">
                                  <path d="M400 224h-44l-26.12-53.25c-12.5-25.5-35.38-44.25-61.75-51L197 98.63C189.5 96.84 181.1 95.97 174.5 95.97c-20.88 0-41.33 6.81-58.26 19.78L76.5 146.3C68.31 152.5 64.01 162 64.01 171.6c0 17.11 13.67 32.02 32.02 32.02c6.808 0 13.67-2.158 19.47-6.616l39.63-30.38c5.92-4.488 13.01-6.787 19.53-6.787c2.017 0 3.981 .2196 5.841 .6623l14.62 4.25l-37.5 87.5C154.1 260.3 152.5 268.8 152.5 277.2c0 22.09 11.49 43.52 31.51 55.29l85 50.13l-27.5 87.75c-.9875 3.174-1.458 6.388-1.458 9.55c0 13.65 8.757 26.31 22.46 30.58C265.6 511.5 268.9 512 272 512c13.62 0 26.25-8.75 30.5-22.5l31.75-101c1.211-4.278 1.796-8.625 1.796-12.93c0-16.57-8.661-32.51-23.55-41.44l-61.13-36.12l31.25-78.38l20.25 41.5C310.9 277.4 327.9 288 345.1 288H400c17.62 0 32-14.38 32-32C432 238.3 417.6 224 400 224zM288 96c26.5 0 48-21.5 48-48s-21.5-48-48-48s-48 21.5-48 48S261.5 96 288 96zM129.8 317.5L114.9 352H48c-17.62 0-32 14.38-32 32s14.38 32 32 32h77.5c19.25 0 36.5-11.5 44-29.12l8.875-20.5l-10.75-6.25C150.4 349.9 137.6 334.8 129.8 317.5z"/>
                              </svg>                    
                          </Row>
                      </Col>
                  </Row>
              </Card.Body>
          </Card>
      </Col>;
  
  users_card= 
      <Col xl="3" className="mb-4 pr-0 align-items-stretch">
          <Card className="h-100">
              <Card.Header className="card-header">
                  Total Users
              </Card.Header>        
              
              <Card.Body className="card-body">
                  <Row>
                      <Col xs={12} md={8}>
                        {user_info} 
                      </Col>
                      <Col xs={6} md={4}>
                          <Row style={{ height: "100px" }} className="align-items-center">
                              <svg xmlns="http://www.w3.org/2000/svg" fill = "#0061f2" viewBox="0 0 640 512" width="64" height="64">
                                  <path d="M184 88C184 118.9 158.9 144 128 144C97.07 144 72 118.9 72 88C72 57.07 97.07 32 128 32C158.9 32 184 57.07 184 88zM208.4 196.3C178.7 222.7 160 261.2 160 304C160 338.3 171.1 369.8 192 394.5V416C192 433.7 177.7 448 160 448H96C78.33 448 64 433.7 64 416V389.2C26.16 371.2 0 332.7 0 288C0 226.1 50.14 176 112 176H144C167.1 176 190.2 183.5 208.4 196.3V196.3zM64 245.7C54.04 256.9 48 271.8 48 288C48 304.2 54.04 319.1 64 330.3V245.7zM448 416V394.5C468 369.8 480 338.3 480 304C480 261.2 461.3 222.7 431.6 196.3C449.8 183.5 472 176 496 176H528C589.9 176 640 226.1 640 288C640 332.7 613.8 371.2 576 389.2V416C576 433.7 561.7 448 544 448H480C462.3 448 448 433.7 448 416zM576 330.3C585.1 319.1 592 304.2 592 288C592 271.8 585.1 256.9 576 245.7V330.3zM568 88C568 118.9 542.9 144 512 144C481.1 144 456 118.9 456 88C456 57.07 481.1 32 512 32C542.9 32 568 57.07 568 88zM256 96C256 60.65 284.7 32 320 32C355.3 32 384 60.65 384 96C384 131.3 355.3 160 320 160C284.7 160 256 131.3 256 96zM448 304C448 348.7 421.8 387.2 384 405.2V448C384 465.7 369.7 480 352 480H288C270.3 480 256 465.7 256 448V405.2C218.2 387.2 192 348.7 192 304C192 242.1 242.1 192 304 192H336C397.9 192 448 242.1 448 304zM256 346.3V261.7C246 272.9 240 287.8 240 304C240 320.2 246 335.1 256 346.3zM384 261.7V346.3C393.1 335 400 320.2 400 304C400 287.8 393.1 272.9 384 261.7z"/>
                              </svg>                 
                          </Row>
                      </Col>
                  </Row>
              </Card.Body>            
          </Card>
      </Col>; 

  return (
   
  <div>
    <Row>
    {resource_card}
    {storage_card}
    {status_card}
    {users_card}
    </Row>    
  </div>
  );
}

function FetchPipelineInfo(params) {

    const fetchData = async () => {
      const res = await fetch(params.dashboard_url);
      return res.json();
    };

    const { isLoading, error, data } = useQuery("dashboard_data", 
                                                fetchData);    
  
    if (isLoading) return (<div className="container-xl px-4 mt-5">
        <div className='row justify-content-md-center'>
        <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
        </div>
    </div>)
   
    if (error) return(
        <div>
        <h4>An error has occurred: {error.message}</h4>
        </div>   
    );
    if (data.error) return( 
        <div>
        <h4> {data.error.name}: {data.error.message} </h4>
        </div>
    );
   
    let live_pipeline_table =<div className="card p-0 mb-4">
        <div className="card-header">
            Currently running workflows
        </div>                  
        <div className="card-body p-0">
            <WorkflowRowsPaginated workflows={data.live_workflows}/>

        </div>   
    </div>;

    if (data.live_workflows.length==0) {
        live_pipeline_table=  <div className="card p-0 mb-4">
            <div className="card-header">
                Currently running workflows
            </div>
            <div className="card-body p-0">
                <Table className="table dataTable" id="pipeline-table" role="grid">
                    <tbody>
                        <tr role="row" className="odd" >
                            <td>There are no workflows currently running</td>
                        </tr>
                    </tbody>            
                </Table>
            </div>
        </div>;
    }

    let recent_pipeline_table =<div className="card p-0 mb-4">
        <div className="card-header">
            Most recent workflows
        </div>                  
        <div className="card-body p-0">
            <Table className="table dataTable" id="pipeline-table" role="grid">
                <thead>
                <tr>
                    <th style={{width: "16.66%"}}> Workflow Name </th>
                    <th style={{width: "16.66%"}}> Pipeline </th>
                    <th style={{width: "16.66%"}}> Run Time </th>
                    <th style={{width: "16.66%"}}> Last Run </th>
                    <th style={{width: "16.66%"}}> Status </th>
                </tr>
                </thead>
                <WorkflowRows workflows={data.recent_workflows}/>
            </Table>
        </div>
    </div>;

    if (data.recent_workflows.length==0) {
        recent_pipeline_table=  <div className="card p-0 mb-4">
            <div className="card-header">
                Most recent workflows
            </div>
            <div className="card-body p-0">
                <Table className="table dataTable" id="pipeline-table" role="grid">
                    <tbody>
                        <tr role="row" className="odd" >
                            <td>There are no recent workflows.</td>
                        </tr>
                    </tbody>
                </Table>        
            </div>
        </div>;
    }    
  
    return (
     
    <div>
      
      <IntroductionScreen token_set={data.token_set} dashboard_url={params}/>
      
      <Row className="mx-0">
        {live_pipeline_table}

      </Row>

      <Row className="mx-0">
        {recent_pipeline_table}

      </Row>      
      
    </div>
    );
 }


class IntroductionScreen extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.dashboard_url=this.props.dashboard_url.dashboard_url;
    if  (this.props.token_set){
        this.state = {
          show: false,
          body: 1,
          user_token : '',
          license_key : ''
        };
    } else{
        this.state = {
          show: true,
          body: 1,
          user_token : '',
          license_key : ''
        };
    }
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }

  setBodyNumber = (number) => {
    this.setState({body:number})
  }

  setLicenseKey = (event) => {
    this.setState({license_key: event.target.value})
  }
    
  render(){
    let body=<div></div>
    if (this.state.body==1){
        
        body=<React.Fragment>
                <Modal.Header>
                    <Modal.Title>Welcome to BeetleboxCI</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <Container>
                        Congratulations on taking the first steps to automating your devices.
                        <div className="m-3">
                            <img src={introduction_welcome_image}  alt="user_portal_example" style={ { maxWidth: '80%'}} />
                        </div>
                    </Container>
                    </Modal.Body>
                <Modal.Footer>
                    <SendSignal dashboard_url={this.props.dashboard_url} handleClick={this.setBodyNumber}  />
                </Modal.Footer>
            </React.Fragment>
            
    } else if (this.state.body==2){
        
        body=<React.Fragment>
                <Modal.Header>
                    <Modal.Title>You are all set!</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <Container>
                          If you need  help learning, please visit the official documentation for a full range of tutorials and other resources.
                        <div className="mb-3">
                            <img src={documentation_example}  alt="Documentation Example" style={ { maxWidth: '100%', border: '1px solid #021a40' }} />
                        </div>
                        <a href="https://docs.beetleboxci.com/">Link to official documentation</a>
                    </Container>
                </Modal.Body>
                <Modal.Footer>
                
                    <Button variant="primary" onClick={() => this.handleClose()}>Close</Button> 
                    
                </Modal.Footer>
            </React.Fragment>    
        
    }
    let introduction_screen =<React.Fragment>
      <Modal show={this.state.show} size="lg" onHide={() => this.handleClose()} centered backdrop="static" keyboard={false}>
        {body}
      </Modal>
    </React.Fragment>;
    return(<React.Fragment>
    {introduction_screen}
    </React.Fragment>);
    }
}

function SendSignal(props) {
    
    function handleSubmit(e) {
        e.preventDefault();
        let data = JSON.stringify({
            signal: 0
        });
         const requestOptions = {
                method: 'POST',
                headers: {
                  'Content-Type': 'application/json',
                  'X-CSRFToken': getCookie('csrftoken'),
                },
                credentials: 'same-origin',
                body: data
                };
            fetch(props.dashboard_url.dashboard_url, requestOptions)
                // .then(response => response.json())
                .then();
                
                
        props.handleClick(2)        

    }
   
    let NextButton=<div></div>
    NextButton=<form onSubmit={handleSubmit}>
      <button className="btn btn-primary " type="submit" >Next</button>
    </form>    
  

  
  return (
     <div>
        <Modal.Footer>
            {NextButton}
        </Modal.Footer>
     </div>
  );
 }
 
class StatusNotification extends React.Component {
    render(){
      let stopwatch_status='RU';
      let status_image='None';
      let status_span=<span className="border"></span>;
      let icon=this.props.icon
       if (this.props.status=='PA'){
        status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '25px'}} />;
        stopwatch_status='PA';
        status_span=<div className="border rounded border-success text-success" style={{"padding": "5px", "backgroundColor": "#EAFAF4"}}>{status_image} {this.props.text}</div>;
       } else if (this.props.status=='PE'){
        status_image=<img src={running_icon} alt="loading_icon" style={ {width: '25px'}} />;
        stopwatch_status='RU';
        status_span=<div className="border rounded border-primary text-primary" style={{"padding": "5px", "backgroundColor": "#F4F8FF"}}>{status_image} {this.props.text}</div>;
       } else if (this.props.status=='FA'){
        stopwatch_status='FA';
        status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '25px'}} />;
        status_span=<div className="border rounded border-danger text-danger" style={{"padding": "5px", "backgroundColor": "#F8DFE2"}}>{status_image} {this.props.text}</div>;
      }
      return(status_span);
    }
} 

class WorkflowRowsPaginated extends React.Component {
    
    render(){
      let workflows=this.props.workflows;
      let workflow_rows=[];
      
      // console.log(workflows)
          for (var i = 0; i < workflows.length; i++) {
            
                // console.log(workflows[i].runtime)
                
                // let trigger_date = new Date(workflows[i].last_run_rawdata);
                // let trigger_date_string=formatDate(trigger_date);
                let stopwatch_status='RU';
                let status_image='None';
                let status_span=<span className="border"></span>;
                if (workflows[i].status=='ID'){
                  status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '25px'}} />;
                  stopwatch_status='ID';
                  status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} Idle</div>;
    
                 } else if (workflows[i].status=='IN'){
                  status_image=<img src={static_pending_icon} alt="static_pending_icon" style={ {width: '25px'}} />;
                  stopwatch_status='ID';
                  status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} Idle</div>;
                  
                 } else if (workflows[i].status=='SC'){
                  status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '25px'}} />;
                  stopwatch_status='PE';
                  status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} {workflows[i].schedule}</div>;
                  log_button = 'N/A';
                  
                 } else if (workflows[i].status=='PA'){
                  status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '25px'}} />;
                  stopwatch_status='PA';
                  status_span=<div className="border rounded border-success text-success" style={{"padding": "5px", "width": "150px", "backgroundColor": "#EAFAF4"}}>{status_image} Passed</div>;
                 } else if (workflows[i].status=='PE'){
                  status_image=<img src={running_icon} alt="loading_icon" style={ {width: '25px'}} />;
                  stopwatch_status='RU';
                  status_span=<div className="border rounded border-primary text-primary" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F4F8FF"}}>{status_image} Running</div>;
                 } else if (workflows[i].status=='FA'){
                  stopwatch_status='FA';
                  status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '25px'}} />;
                  status_span=<div className="border rounded border-danger text-danger" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F8DFE2"}}>{status_image} Failed</div>;                
                }
                workflow_rows.push(
                  <tr role="row" className="odd" key={i}>
                    <td className="align-middle"><a href={workflows[i].workflow_url}>{workflows[i].workflow_name}</a></td>
                    <td className="align-middle"><StopWatch status={stopwatch_status} start_time={workflows[i].runtime}></StopWatch></td>
                    <td className="align-middle">{workflows[i].last_run}</td>
                    <td className="align-middle">{status_span}</td>
                    <td><StopButton workflow_id={workflows[i].workflow_id}></StopButton> </td>                
                  </tr>
                  );
          }
          return(
                  <PaginatedTable rows={workflow_rows} number_of_rows_per_page="5" />
              );
      
    }
}


class StopButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }
    
    render(){
    let stop_confirmation_screen =     <>
      <Button variant="primary btn-sm  px-2 p-1"  onClick={() => this.handleShow()} >
        <img src={stop_icon} alt="stop_icon" /> 
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header closeButton>
          <Modal.Title>Stop Workflow</Modal.Title>
        </Modal.Header>
        <Modal.Body>Are you sure you want to stop all current jobs in this workflow?</Modal.Body>
        <Modal.Footer>
        <Form workflow_id={this.props.workflow_id} onHide={this.handleClose}  closeModal={() => this.handleClose()}/> 

        </Modal.Footer>
      </Modal>
    </>;
    return(<div>
    {stop_confirmation_screen}
    </div>);
    }
}

function Form(props) {
  function handleSubmit(e) {
    e.preventDefault();
    let data = JSON.stringify({
        name: "stop_workflow_",
        id_: props.workflow_id
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            // .then(response => response.json())
            .then();
    props.closeModal();
  }

  return (
    <form onSubmit={handleSubmit}>
      <button className="btn btn-danger " type="submit" id="stop_workflow_" name="stop_workflow_">Yes</button>
    </form>
  );
}



class PaginatedTable extends React.Component{
    constructor(props) {
      super(props);
      this.state = {
        currentPageIndex: 0,
        modalEllipseShow : false
      };
    }
    
  
    handleEllipseClose(){
      this.setState({modalEllipseShow : false});
    }
    
    handleEllipseOpen(){
      this.setState({modalEllipseShow : true});
    }
    
    handleSubmit(event){
      let rows=this.props.rows;
      let userSetPage=parseInt(event.target.pageNumber.value-1);
      let number_of_pages=Math.ceil(rows.length / this.props.number_of_rows_per_page);
      if (number_of_pages >= userSetPage) {
        this.setState({currentPageIndex : parseInt(event.target.pageNumber.value-1)});
      }
      this.setState({modalEllipseShow : false});
      event.preventDefault();
    }
    
    
    paginationEllipseClicked(number){
      this.handleEllipseOpen();
    }
    
    paginationClicked(number){
      this.setState({currentPageIndex : number-1});
    }
    
    paginationFirstClicked(){
      this.setState({currentPageIndex : 0});
    }
    
    paginationPreviousClicked(){
      if (this.state.currentPageIndex==0){
        this.setState({currentPageIndex : 0});
      } else{
        let tempNumber=this.state.currentPageIndex-1;
        this.setState({currentPageIndex : tempNumber});
      }
    }
    
    paginationNextClicked(number_of_pages){
      if (this.state.currentPageIndex==number_of_pages-1){
        this.setState({currentPageIndex : number_of_pages-1});
      } else{
        let tempNumber=this.state.currentPageIndex+1;
        this.setState({currentPageIndex : tempNumber});
      }  }
    
    paginationLastClicked(number_of_pages){
      this.setState({currentPageIndex : number_of_pages-1});
    }
    
    render(){
      let rows=this.props.rows;
      let current_page_list=[];
      let starting_index=this.props.number_of_rows_per_page*this.state.currentPageIndex;
      for (var i = 0; i < this.props.number_of_rows_per_page; i++) {
        let current_index=starting_index+i;
        if (current_index < rows.length){
          current_page_list.push(rows[current_index]);
        }
      }
      
      let number_of_pages=Math.ceil(rows.length / this.props.number_of_rows_per_page);
      let items = [];
      if (number_of_pages<8){
        for (let number = 1; number <= number_of_pages; number++) {
          items.push(
            <Pagination.Item key={number} onClick={() => this.paginationClicked(number)} active={number === parseInt(this.state.currentPageIndex+1)}>
              {number}
            </Pagination.Item>,
          );
        }
      } else {
        for (let number = 1; number <= 8; number++) {
          items.push(
            <Pagination.Item key={number} onClick={() => this.paginationClicked(number)} active={number === parseInt(this.state.currentPageIndex+1)}>
              {number}
            </Pagination.Item>,
          );
        }
        if (8<this.state.currentPageIndex+1){
            items.push(<Pagination.Ellipsis key="ellipse1" onClick={() => this.paginationEllipseClicked()}/>);
            items.push(<Pagination.Item key="current" onClick={() => this.paginationClicked(this.state.currentPageIndex+1)} active={true}>{this.state.currentPageIndex+1} </Pagination.Item>);
            if ((this.state.currentPageIndex+1)!=number_of_pages){
            items.push(<Pagination.Ellipsis key="ellipse2" onClick={() => this.paginationEllipseClicked()}/>);
            }
        }
        else{
          items.push(<Pagination.Ellipsis key="ellipse1" onClick={() => this.paginationEllipseClicked()}/>);
  
        }
  
  
      }
      
      
      return(
        <div>
        <Row>
                  <Table className="dataTable" id="projectsTable" role="grid">
                      <thead>
                      <tr>
                        <th style={{width: "16.66%"}}> Workflow </th>
                        <th style={{width: "16.66%"}}> Run Time </th>
                        <th style={{width: "16.66%"}}> Last Run </th>
                        <th style={{width: "16.66%"}}> Status </th>
                        <th style={{width: "8.333%"}}> Actions </th>
                      </tr>
                  </thead>
      <tbody>
      {current_page_list}
     </tbody>
      </Table>
      </Row>
      <Row className="justify-content-md-center">
        <Col md="auto">
        <Pagination>
          <Pagination.First onClick={() => this.paginationFirstClicked()}  />
          <Pagination.Prev onClick={() => this.paginationPreviousClicked()}  />
          {items}
          <Pagination.Next onClick={() => this.paginationNextClicked(number_of_pages)}  />
          <Pagination.Last onClick={() => this.paginationLastClicked(number_of_pages)}  />
        </Pagination>
        </Col>
      </Row>
        
        <Modal size="sm" show={this.state.modalEllipseShow} onHide={ () => this.handleEllipseClose()}>
          <form onSubmit={ (event) => {this.handleSubmit(event)}}>
          <Modal.Header closeButton>
            <Modal.Title>Page Select</Modal.Title>
          </Modal.Header>
          <Modal.Body>
          <label>
            <input type="text" name= "pageNumber" />
          </label>
          </Modal.Body>
          <Modal.Footer>
            <Button variant="danger" onClick={() => this.handleEllipseClose()}>
              Close
            </Button>
            <Button variant="primary" type="submit" value="Submit">
              Submit
            </Button>
          </Modal.Footer>
          </form>
        </Modal>
    </div>
  
  );
    }
  }


class WorkflowRows extends React.Component {
    
    render(){
      let workflows=this.props.workflows;
      let workflow_rows=[]
      for (var i = 0; i < workflows.length; i++) {
            let trigger_date = new Date(workflows[i].last_run);
            let trigger_date_string=formatDate(trigger_date);
            let stopwatch_status='RU';
            let status_image='None'
            let supporting_text='None'
            let status_span=<span className="border"></span>
            if (workflows[i].status=='ID'){
              status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '25px'}} />;
              stopwatch_status='ID';
              status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} Idle</div>

             } else if (workflows[i].status=='IN'){
              status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '25px'}} />;
              stopwatch_status='ID';
              status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} Idle</div>
              
             } else if (workflows[i].status=='SC'){
              status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '25px'}} />;
              stopwatch_status='PE';
              status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} {workflows[i].schedule}</div>
              
             } else if (workflows[i].status=='PA'){
              status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '25px'}} />;
              stopwatch_status='PA';
              status_span=<div className="border rounded border-success text-success" style={{"padding": "5px", "width": "150px", "backgroundColor": "#EAFAF4"}}>{status_image} Passsed</div>
             } else if (workflows[i].status=='PE'){
              status_image=<img src={running_icon} alt="loading_icon" style={ {width: '25px'}} />;
              stopwatch_status='RU';
              supporting_text='Running';
              status_span=<div className="border rounded border-primary text-primary" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F4F8FF"}}>{status_image} Running</div>
             } else if (workflows[i].status=='FA'){
              stopwatch_status='FA';
              status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '25px'}} />;
              supporting_text='Failed';
              status_span=<div className="border rounded border-danger text-danger" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F8DFE2"}}>{status_image} Failed</div>
            }
            workflow_rows.push(
              <tr role="row" className="odd" key={i}>
                <td><a href={workflows[i].workflow_url}>{workflows[i].workflow_name}</a></td>
                <td><a href={workflows[i].project_url}>{workflows[i].project_name}</a></td>
                <td><StopWatch status={stopwatch_status} start_time={workflows[i].runtime}></StopWatch></td>
                <td>{workflows[i].last_run}</td>
                <td>{status_span}</td>

              </tr>
              );

      };
      return(
          <tbody>
              {workflow_rows}
          </tbody>
          );
      }
}

