import React, { useMemo } from 'react';
import ReactDOM from "react-dom";

import {getCookie} from "./shared_functions.js"
import running_icon from './img/beetlebox-running-icon.png';

import { Modal, Form, Button, Table, Container, Row, Col, Card, Alert} from 'react-bootstrap';
import {Breadcrumb} from './shared_classes.js'

import { TextField } from '@material-ui/core';

import CodeMirror from '@uiw/react-codemirror';

import { basicSetup } from "codemirror"
import { StreamLanguage } from "@codemirror/language"
import { linter, lintGutter } from "@codemirror/lint"
import { yaml } from "@codemirror/legacy-modes/mode/yaml"
import parser from "js-yaml";

import debounce from 'lodash.debounce'

import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 

const twentyFourHoursInMs = 1000 * 60 * 60 * 24;
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      refetchOnWindowFocus: false,
      refetchOnmount: false,
      refetchOnReconnect: false,
      retry: false,
      staleTime: twentyFourHoursInMs,
    },
  },
});

export default class DeviceEditorView extends React.Component {
  constructor(props) {
    super(props);
    var pathArray = window.location.pathname.split('/');

    // this.config_url = window.location.protocol + '//' + window.location.host + '/api/deviceeditor?device=' + window.location.search.split('device=')[1] + '&branch=' + window.location.search.split('branch=')[1];
    this.device_url = window.location.protocol + '//' + window.location.host + '/api/deviceeditor?device=' + window.location.search.split('device=')[1] ;
    // this.device_url = window.location.protocol + '//' + window.location.host + '/api/deviceeditor?device=' ;
    // this.device_url = window.location.protocol + '//' + window.location.host + '/api/deviceeditor'
  }
  
  render() {
    
    const queryString = window.location.search;
    const urlParams = new URLSearchParams(queryString);
    const device_id = urlParams.get('device')
    // const branch = urlParams.get('branch')

    // const dev_url = this.device_url + '?device=' + device_id + '&branch=' + branch
    const dev_url = this.device_url
    // const dev_url = this.device_url
    
    return (
      <div>
        <QueryClientProvider client={queryClient}>
          <FetchConfig device_url={dev_url} device_id={device_id} />
        </QueryClientProvider>
      </div>
    );
    
  }
}

function FetchConfig(props) {
  const { isLoading, error, data } = useQuery('device_data', () =>
     fetch(props.device_url).then(res =>
       res.json()
     ),
  );
  
  if (isLoading) return (<div className="container-xl px-4 mt-5">
  <div className='row justify-content-md-center'>
    <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
  </div>
  </div>);
  
  if (error) return(
    <div>
    <h4>An error has occurred: {error.message}</h4>
    </div>   
  );
  if (data.error) return( 
    <div>
    <h4> {data.error.name}: {data.error.message} </h4>
    </div>
  );
  
  // let pageList=[['Devices', '/devices'], [data.device_name, window.location.pathname + '?device=' + props.device_id + '&branch=' + props.branch ]];
  // let pageList=[['Devices', '/devices'], [data.device_name, window.location.pathname ]];
  
  var title
  var pageList
  if (data.device_name == ""){
    pageList=[['Devices', '/devices'], ["Register device", window.location.pathname ]];
    title = <h1 className="mb-0">Config for {data.device_name}</h1>
    title = <h1 className="mb-0">Config for new device</h1>
  }
  else{
    pageList=[['Devices', '/devices'], [data.device_name, window.location.pathname + '?device=' + props.device_id  ]];
    title = <h1 className="mb-0">Config for {data.device_name}</h1>
  }

  return (
  <Container fluid className="px-4 mt-5">
    <Row>
        <Breadcrumb pageList={pageList} />
    </Row>  
    <Row>
        {title}
    </Row>    
    <Row className="mx-0">    
      <ConfigText device_id={data.device_id} device_data={data.device_text} example_data={data.example_text} device_name={props.device_name}/>      
    </Row>    
  </Container>
  );
}

class ConfigText extends React.Component {
  constructor(props) {
    super(props);

    if  (this.props.device_data.length<1){
        this.state = {
          // show: false,
          name: 'CodeMirror',
          code: this.props.device_data,
          commit: '',
          error_text: '',          
        };
    } else{
        this.state = {
          // show: false,
          name: 'CodeMirror',
          code: this.props.device_data,
          commit: '',
          error_text: '',                   
        };
    }
    
    this.handleSubmit = this.handleSubmit.bind(this);
    
    this.updateCommit = this.updateCommit.bind(this);
    this.debouncedupdateCommit = debounce(this.debouncedupdateCommit, 500)

    // this.updateCode   = this.updateCode.bind(this);
    
    this.debouncedUpdateCode = debounce(this.debouncedUpdateCode, 500)
    
  }
  
  // handleYes(){
  //   this.setState({code:this.props.example_data});
  //   this.setState({show:false});
  // }    
  // handleNo(){
  //   this.setState({show:false});
  // }  
  
   

  handleCodeChange = (newCode) => {
    // Immediately update the state
    this.setState({
      code: newCode
    })
    // Execute the debounced method
    this.debouncedUpdateCode(newCode)
  }  
  debouncedUpdateCode = (newCode) => {
    // Delayed logic goes here
    this.handleCodeChange(newCode)
  }


  updateCommit(event) {    
		event.preventDefault()
    
    this.setState({
      commit: event.target.value
      
    }); 
    event.preventDefault()
  }	
  debouncedupdateCommit = (event) => {
    // Delayed logic goes here
    this.updateCommit(event)
  }
	
  handleSubmit(e) {
    e.preventDefault();
    
    var commit;
    if (this.state.commit == ''){
      commit = 'Device file updated by BeetleboxCI'
    } else {
      commit = this.state.commit
    }
    
    let data = JSON.stringify({
        name: "push_branch_",
        br_: this.props.branch,
        id_: this.props.device_id,
        device_data_: this.state.code,
        commit_message_: commit
        
    });
      
    const requestOptions = {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'X-CSRFToken': getCookie('csrftoken'),
          },
          credentials: 'same-origin',
          body: data
          };
          
    fetch('', requestOptions)
            .then(response => response.json())
            .then((data) => {
                // console.log(data)
                 
                 if (data.config_error){
                   this.setState({error_text: data.error_details});
                  // alert(data.error_details)
                   
                 } else if (data.push_error){
                   this.setState({error_text:"Error while pushing to remote repository. Please ensure that you have the correct permissions to push to this repository."});

                 } else {
                   window.location.href = data.redirect;
                 }
            });
  }	
  
  render(){    
    
    let default_device_modal =     <>
  
      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header >
          <Modal.Title>Empty device file</Modal.Title>
        </Modal.Header>
        <Modal.Body> The device file for this device is currently empty. Do you want to add an example device file for the '{this.props.device_name}'' device?</Modal.Body>
        <Modal.Footer>
          <button className="btn btn-primary btn-sm"  onClick={() => this.handleYes()}>
              Yes
          </button>          
          <button className="btn btn-primary btn-sm"  onClick={() => this.handleNo()}>
              No
          </button>
        </Modal.Footer>
      </Modal>
    </>;    
    
    let error_output = <div></div>;
    if (this.state.error_text.length!=0){
      error_output =<div>
            <Alert key="danger" variant="danger">
              {this.state.error_text}
            </Alert>
        </div>;
    }
    
    const yamlLinter = linter((view) => {
      const diagnostics = [];
    
      try {
        parser.load(view.state.doc);
      } catch (e) {
        const loc = e.mark;
        const from = loc ? loc.position : 0;
        const to = from;
        const severity = "error";
    
        diagnostics.push({
          from,
          to,
          message: e.message,
          severity
        });
      }
    
      return diagnostics;
    });


    let code_mirror=<CodeMirror
    value={this.state.code}
    height="500px"
    theme="dark"                
    onChange={this.debouncedUpdateCode}                
    extensions={[StreamLanguage.define(yaml) , lintGutter(), yamlLinter]}                
    />





    var device_card = <div className="card p-0 mb-4 mt-4">
        <div className="card-header" style={{color: "black" }}>
          <Row >
            <Col md="auto me-auto"> Device file </Col>
            <Col md="auto">  
              <form onSubmit={this.handleSubmit}>
                <button className="btn btn-primary btn-sm" type="submit" id="push_branch_" name="push_branch_">Save</button>
              </form>
            </Col>
          </Row>
        </div>
    
        <div className="card-body p-3">
        
            {/* <TextField 

              fullWidth
              multiline={true} 
              minRows={3} 
              id="filled-basic" 
              placeholder="device.yaml updated by BeetleboxCI" 
              label="Enter a commit message (optional)" 
              variant="filled" 
              onChange={this.debouncedupdateCommit} 
            /> */}
          
            <Row className="mt-3">
            {error_output}
            </Row>
            
            <Row>
          
              {code_mirror}
              
            </Row>
        </div>
    </div>
    return(<React.Fragment>
    
    {/* {default_device_modal} */}
    {device_card}
    
    </React.Fragment>);
  }
}






