import React from 'react';
import ReactDOM from "react-dom";

import {LogButton} from './shared_classes.js'
import {getCookie} from "./shared_functions.js"

import running_icon from './img/beetlebox-running-icon.png';
import pending_icon from './img/beetleboxci-pending-icon.png';
import static_pending_icon from './img/beetleboxci-static-pending-icon.svg';
import failure_icon from './img/beetleboxci-failure-icon.svg';
import pass_icon from './img/beetleboxci-pass-icon.svg';
import warning_icon from './img/beetleboxci-warning-icon.svg';

import { Modal, Button, Table, Container, Row} from 'react-bootstrap';


import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 
const queryClient = new QueryClient();

export default class DeviceView extends React.Component {
  constructor(props) {
    super(props);
    var pathArray = window.location.pathname.split('/');
    this.device_url = window.location.protocol + '//' + window.location.host + '/api/device';
  } 
  
  render() {
    return (
      <div>
        <QueryClientProvider client={queryClient}>
          <FetchPipelines device_url={this.device_url} />
        </QueryClientProvider>
      </div>
    );
  }
}

function FetchPipelines(api_url) {
   const { isLoading, error, data } = useQuery('device_data', () =>
     fetch(api_url.device_url).then(res =>
       res.json()
     ),
      {
      // Refetch the data every three second
      refetchInterval: 3000,
    }
   );
   if (isLoading) return (<div className="container-xl px-4 mt-5">
  <div className='row justify-content-md-center'>
    <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
  </div>
 </div>);

 if (error) return 'An error has occurred: ' + error.message;
 if (data.error) return data.error.name + ': ' + data.error.message; 
   let device_card=<div></div>;
   if (data.devices.length==0){
       device_card=<div className="card card-waves mb-4 mt-5">
          <div className="card-body p-5">
            <div className="row align-items-center justify-content-between">
              <div className="col">
                <h2 className="text-primary">Welcome to the devices page</h2>
                <p className="text-gray-700">There are currently no registered devices</p>
                <a className="btn btn-primary btn-xl" href={data.register_device_url}>Register Device</a>


              </div>
            </div>
          </div>
         </div>
   } else {
       device_card= <div className="card p-0 mb-4 mt-4">
          <div className="card-body p-0">
            
            <Table className="table dataTable" id="deviceTable" role="grid">
                <thead>
                    <tr>
                      <th style={{width: "25%"}}> Device </th>
                      <th style={{width: "25%"}}> Status </th>
                      <th style={{width: "25%"}}> User </th>
                      <th style={{width: "12.5%"}}> Log </th>
                      <th style={{width: "12.5%"}}> Actions </th>
                    </tr>
                </thead>
              <DeviceRows devices={data.devices}/>
            </Table>
          </div>
          <a className="btn btn-primary btn-xl" href={data.register_device_url}>Register Device</a>

      </div>
   } 
   return (     
    <Container fluid className="px-4 mt-5">
        <Row>
            <h1 className="mb-4">Devices</h1>  
        </Row>
        <Row className="mx-0">
            {device_card}           
        </Row>  
    </Container>
   );
 }  

class DeviceRows extends React.Component {
  
  render(){
    let devices=this.props.devices;
    let device_rows=[];    

    for (var i = 0; i < devices.length; i++) {

      let log_button = <LogButton output={devices[i].output}></LogButton>;

      let status_image='None';
      let status_span=<span className="border"></span>;
      
      if (devices[i].device_status=='AV'){
        status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '25px'}} />;
        status_span=<div className="border rounded border-success text-success" style={{"padding": "5px", "width": "150px", "backgroundColor": "#EAFAF4"}}>{status_image} Available</div>;

      } else if (devices[i].device_status=='BU'){
        status_image=<img src={running_icon} alt="loading_icon" style={ {width: '25px'}} />;
        status_span=<div className="border rounded border-primary text-primary" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F4F8FF"}}>{status_image} Busy</div>;

      } else if (devices[i].device_status=='ER'){
        status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '25px'}} />;
        status_span=<div className="border rounded border-danger text-danger" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F8DFE2"}}>{status_image} Error</div>;        

      } else if (devices[i].device_status=='NR'){
        status_image=<img src={warning_icon} alt="warning_icon" style={ {width: '25px'}} />;
        status_span=<div className="border rounded border-warning text-warning" style={{"padding": "5px", "width": "150px", "backgroundColor": "#fffff1"}}>{status_image} No response</div>;        

      } else if (devices[i].device_status=='UN'){
        status_image=<img src={static_pending_icon} alt="static_pending_icon" style={ {width: '25px'}} />;
        status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F4F4F4"}}>{status_image} Unknown</div>;        

      }

      device_rows.push(
      <tr role="row" className="odd" key={i}>
        <td className="align-middle">{devices[i].device_name}</td>
        <td className="align-middle">  {status_span} </td>
        <td className="align-middle">{devices[i].device_user}</td>
        <td className="align-middle">{log_button}</td>
        <td className="align-middle"><RefreshButton device_id={devices[i].device_id} device_status_check_enabled={devices[i].device_status_check_enabled}  ></RefreshButton>&nbsp;<DeregisterDeviceButton device_name={devices[i].device_name}></DeregisterDeviceButton>&nbsp;<EditConfigForm device_id={devices[i].device_id} device_name={devices[i].device_name} device_prompt={devices[i].device_prompt} device_transfer={devices[i].device_transfer} device_connection={devices[i].device_connection}></EditConfigForm>  </td>
      </tr>
      );            
    }    
    return(
        <tbody>
            {device_rows}
        </tbody>
        );    
  }
}

class RefreshButton extends React.Component {
  constructor(props) {
    super(props);
    // this.handleSubmit = this.handleSubmit.bind(this);

  }
  handleSubmit() {
    // e.preventDefault();
    let data = JSON.stringify({
      name: "refresh_device_",
      device_id: this.props.device_id
  });
  const requestOptions = {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      'X-CSRFToken': getCookie('csrftoken'),
    },
    credentials: 'same-origin',
    body: data
    };
  fetch('', requestOptions)
      .then();         
  } 
    
  render(){    
    return(
      <React.Fragment>
        <Button variant="primary btn-sm" disabled={!this.props.device_status_check_enabled} onClick={()=> {this.handleSubmit()}} >
          <svg width="12" height="12" aria-hidden="true" focusable="false" data-prefix="fas" data-icon="sync" className="svg-inline--fa fa-sync fa-w-16" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="currentColor" d="M440.65 12.57l4 82.77A247.16 247.16 0 0 0 255.83 8C134.73 8 33.91 94.92 12.29 209.82A12 12 0 0 0 24.09 224h49.05a12 12 0 0 0 11.67-9.26 175.91 175.91 0 0 1 317-56.94l-101.46-4.86a12 12 0 0 0-12.57 12v47.41a12 12 0 0 0 12 12H500a12 12 0 0 0 12-12V12a12 12 0 0 0-12-12h-47.37a12 12 0 0 0-11.98 12.57zM255.83 432a175.61 175.61 0 0 1-146-77.8l101.8 4.87a12 12 0 0 0 12.57-12v-47.4a12 12 0 0 0-12-12H12a12 12 0 0 0-12 12V500a12 12 0 0 0 12 12h47.35a12 12 0 0 0 12-12.6l-4.15-82.57A247.17 247.17 0 0 0 255.83 504c121.11 0 221.93-86.92 243.55-201.82a12 12 0 0 0-11.8-14.18h-49.05a12 12 0 0 0-11.67 9.26A175.86 175.86 0 0 1 255.83 432z"></path></svg>              
        </Button>
      </React.Fragment>
    );
  }
}

// button to go to the device editor page to edit an existing device
class EditConfigForm extends React.Component {
  constructor(props) {
    super(props);
    // this.handleSubmit = this.handleSubmit.bind(this);

  }
  handleSubmit() {
    // e.preventDefault();
    let data = JSON.stringify({
        name: "edit_device_config_",
        device_name: this.props.device_name,
        device_id: this.props.device_id,
        branch_: "main", 
        edit_config_: 1,
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            .then(response =>  {
              // console.log(response)
              window.location.href = response.url;
              });             
  } 
    
  render(){

    // let branches=this.props.branches;
    let edit_device_button =     <>
      <Button variant="primary btn-sm"  onClick={()=> {this.handleSubmit()}} >
        <svg xmlns="http://www.w3.org/2000/svg" width="12" height="12" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-settings"><circle cx="12" cy="12" r="3"></circle><path d="M19.4 15a1.65 1.65 0 0 0 .33 1.82l.06.06a2 2 0 0 1 0 2.83 2 2 0 0 1-2.83 0l-.06-.06a1.65 1.65 0 0 0-1.82-.33 1.65 1.65 0 0 0-1 1.51V21a2 2 0 0 1-2 2 2 2 0 0 1-2-2v-.09A1.65 1.65 0 0 0 9 19.4a1.65 1.65 0 0 0-1.82.33l-.06.06a2 2 0 0 1-2.83 0 2 2 0 0 1 0-2.83l.06-.06a1.65 1.65 0 0 0 .33-1.82 1.65 1.65 0 0 0-1.51-1H3a2 2 0 0 1-2-2 2 2 0 0 1 2-2h.09A1.65 1.65 0 0 0 4.6 9a1.65 1.65 0 0 0-.33-1.82l-.06-.06a2 2 0 0 1 0-2.83 2 2 0 0 1 2.83 0l.06.06a1.65 1.65 0 0 0 1.82.33H9a1.65 1.65 0 0 0 1-1.51V3a2 2 0 0 1 2-2 2 2 0 0 1 2 2v.09a1.65 1.65 0 0 0 1 1.51 1.65 1.65 0 0 0 1.82-.33l.06-.06a2 2 0 0 1 2.83 0 2 2 0 0 1 0 2.83l-.06.06a1.65 1.65 0 0 0-.33 1.82V9a1.65 1.65 0 0 0 1.51 1H21a2 2 0 0 1 2 2 2 2 0 0 1-2 2h-.09a1.65 1.65 0 0 0-1.51 1z"></path></svg>            
      </Button>
    </>;
    return(<React.Fragment>
    {edit_device_button}
    </React.Fragment>);
  }
}
  
class DeregisterDeviceButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }  
  handleShow(){
      this.setState({show:true});
  } 
  handleClose(){
      this.setState({show:false});
  }    
    render(){
    let delete_confirmation_screen =     <>
      <Button variant="primary btn-sm"  onClick={() => this.handleShow()} >
        <svg  xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-trash-2"><polyline points="3 6 5 6 21 6"></polyline><path d="M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6m3 0V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2"></path><line x1="10" y1="11" x2="10" y2="17"></line><line x1="14" y1="11" x2="14" y2="17"></line></svg> 
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header closeButton>
          <Modal.Title>Deregister Device</Modal.Title>
        </Modal.Header>
        <Modal.Body>
          Are you sure you want to deregister this device? 
        </Modal.Body>
        <Modal.Footer>
        <DeleteForm device_name={this.props.device_name} onHide={this.handleClose}  closeModal={() => this.handleClose()}/> 

        </Modal.Footer>
      </Modal>
    </>;
    return(<React.Fragment>
    {delete_confirmation_screen}
    </React.Fragment>);
    }
}

function DeleteForm(props) {
  function handleSubmit(e) {
    e.preventDefault();
    let data = JSON.stringify({
        name: "deregister_device_",
        device: props.device_name
    });
        
    const requestOptions = {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'X-CSRFToken': getCookie('csrftoken'),
          },
          credentials: 'same-origin',
          body: data
          };
      fetch('', requestOptions)
          .then(response => {
              // console.log(response)
              if (!response.ok) {
                throw new Error(response);
              }
              // window.location.href = response.url;
          }) 
    props.closeModal();
  }

  return (
    <form onSubmit={handleSubmit}>
      <button className="btn btn-primary " type="submit" id="deregister_device_" name="deregister_device_">Submit</button>
    </form>
  );
}
 
 
