import React from 'react';
import ReactDOM from "react-dom";


import {getCookie} from "./shared_functions.js"
import running_icon from './img/beetlebox-running-icon.png';

import { Modal, Form, Button, Table, Container, Row, Col, Card, Alert} from 'react-bootstrap';


import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 
 const queryClient = new QueryClient();


export default class ImageView extends React.Component {
  constructor(props) {
    super(props);
    var pathArray = window.location.pathname.split('/');

    this.image_url = window.location.protocol + '//' + window.location.host + '/api/image';

  }
  
  
  render() {
    return (
      <div>
        <QueryClientProvider client={queryClient}>
          <FetchPipelines image_url={this.image_url} />
        </QueryClientProvider>
      </div>
    );
  }
}


function FetchPipelines(api_url) {
   const { isLoading, error, data } = useQuery('image_data', () =>
     fetch(api_url.image_url).then(res =>
       res.json()
     ),
      {
      // Refetch the data every three second
      refetchInterval: 3000,
    }
   );
   if (isLoading) return (<div className="container-xl px-4 mt-5">
  <div className='row justify-content-md-center'>
    <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
  </div>
 </div>);

 if (error) return 'An error has occurred: ' + error.message;
 if (data.error) return data.error.name + ': ' + data.error.message;
 
   let image_card=<div></div>;
   if (data.images.length==0){
       image_card=<div className="card card-waves mb-4 mt-5">
          <div className="card-body p-5">
            <div className="row align-items-center justify-content-between">
              <div className="col">
                <h2 className="text-primary">Welcome to BeetleboxCI, your account is ready!</h2>
                <p className="text-gray-700">To get started please check out our documentation or create a new image.</p>
                <a className="btn btn-primary p-3" href="#">
                  Create your first image
                <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-arrow-right ms-1"><line x1="5" y1="12" x2="19" y2="12"></line><polyline points="12 5 19 12 12 19"></polyline></svg>
                </a>
              </div>
            <div className="col d-none d-lg-block mt-xxl-n4"><img className="img-fluid px-xl-4 mt-4" src="/static/assets/img/Intro-pic.png"></img></div>
            </div>
          </div>
         </div>
   } else {
       image_card= <div className="card p-0 mb-4 mt-4">
          <div className="card-body p-0">
            
            <Table className="table dataTable" id="imageTable" role="grid">
                <thead>
                    <tr>
                      <th > Image </th>
                      <th > Version </th>
                      <th > Full image path </th>                      
                      <th > Actions </th>

                    </tr>
                </thead>

            <ImageRows images={data.images}/>
            
            </Table>
            
            
          </div>

          <ImageButton></ImageButton>        
          
      </div>
   }
   

 
   return (
     
    <Container fluid className="px-4 mt-5">
        <Row>
            <h1 className="mb-4">Images</h1>

  
        </Row>

        <Row className="mx-0">
            {image_card}
            
        </Row>
        
        
    </Container>
   );
 }
 

  
class ImageButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }
    
    render(){
    let add_image_screen =     <>
      <Button variant="primary btn-sm"  onClick={() => this.handleShow()} >
        Add Image
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()} backdrop="static">
          <AddImageForm onHide={this.handleClose}  closeModal={() => this.handleClose()} />
      </Modal>
    </>;
    return(<React.Fragment>
    {add_image_screen}
    </React.Fragment>);
    }
}


class AddImageForm extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      image_name: '',
      public_reg: false
    };
    this.handleInput = this.handleInput.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  handleInput(event) {
    
    const target = event.target;
    const value = target.type === 'checkbox' ? target.checked : target.value;
    const name = target.name;
    
    this.setState({
      [name]: value    
      
    });
    
  }

  handleSubmit(event) {
    
    let data = JSON.stringify({
      
        name: "add_image_",
        image: this.state.image_name,
        public_reg: this.state.public_reg
    });
    var filename;
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)

          .then(response => {
              // console.log(response)
              if (!response.ok) {
                throw new Error(response);
              }
              // window.location.href = response.url;
              filename = "add_image.sh"
              return response.blob()
          })
          .then(blob => {

              // console.log(blob)
              const url = window.URL.createObjectURL(blob);
              const a = document.createElement('a');
              a.style.display = 'none';
              a.href = url;
              a.download = filename;
              document.body.appendChild(a);
              a.click();
              document.body.removeChild(a);
              window.URL.revokeObjectURL(url);
              a.remove();  //afterwards we remove the element again
            })
     
    
    // alert('An image was submitted: ' + this.state.image_name);

    event.preventDefault();
    
    this.props.closeModal();
  }
  
  


  render() {
    
    let imageform=<>

      <form onSubmit={this.handleSubmit}>
      
      
        <Modal.Header closeButton>
          <Modal.Title>Add Image</Modal.Title>
        </Modal.Header>
        
        <Modal.Body>
        
          <Row className="mb-3">
            <Col>
              To generate a script to add an image from an external repository, enter the full path of the image, including the registry location, the image name and the tag. For example:
              
            </Col>
          </Row>
          
          <Row className="mb-3">
            <Col>
              reg.location/img_name:latest
            </Col>
          </Row>          
          
          <Row className="mb-3">
            <Col>
              <label>
                Image:&nbsp;
                <input name="image_name" type="text"  placeholder="reg.location/img_name:latest" value={this.state.image_name}  onChange={this.handleInput} />
                
              </label>
            </Col>
            
          </Row>
          
          <Row>
            <Col>
              <label>
                Public registry&nbsp;
                <input name="public_reg" type="checkbox"  checked={this.state.public_reg} onChange={this.handleInput} />
                
              </label>
            </Col>
            
          </Row>          
            
        </Modal.Body>
        
        <Modal.Footer>
          <input className="btn btn-primary " type="submit" value="Submit" />
  
        </Modal.Footer>   
        
      </form>
    
    </>;    
    
    return(<React.Fragment>
      {imageform}
    </React.Fragment>);


  }
}
 


class ImageRows extends React.Component {
  
  render(){
    let images=this.props.images;
    let image_rows=[];
    

    for (var i = 0; i < images.length; i++) {
      
      if ((images[i].image_name!='web') && (images[i].image_name!='redis') && (images[i].image_name!='nginx-unprivileged') && (images[i].image_name!='bitnami/postgresql')) {

        image_rows.push(
        <tr role="row" className="odd" key={i}>
          <td className="align-middle">{images[i].image_name}</td>
          <td className="align-middle">{images[i].image_version}</td>
          <td className="align-middle">
            {images[i].full_image_path}
            <CopyButton full_image_path={images[i].full_image_path}></CopyButton>
          </td>
          
          <td className="align-middle"><DeleteImageButton image_name={images[i].image_name}></DeleteImageButton> </td>

        </tr>
        );
        
      }
            
    }
    
    return(
        <tbody>
            {image_rows}
        </tbody>
        );
    
  }
}

// navigator.clipboard only works through https, and execCommand can be used as a fallback
async function copyToClipboard(textToCopy) {
  // Navigator clipboard api needs a secure context (https)
  if (navigator.clipboard && window.isSecureContext) {
      await navigator.clipboard.writeText(textToCopy);
  } else {
      // Use the 'out of viewport hidden text area' trick
      const textArea = document.createElement("textarea");
      textArea.value = textToCopy;
          
      // Move textarea out of the viewport so it's not visible
      textArea.style.position = "absolute";
      textArea.style.left = "-999999px";
          
      document.body.prepend(textArea);
      textArea.select();

      try {
          document.execCommand('copy');
      } catch (error) {
          console.error(error);
      } finally {
          textArea.remove();
      }
  };
}

class CopyButton extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      textToCopy: this.props.full_image_path                 
    };
  }
  

  render(){
  let copy_btn =     <>
    <Button variant="btn-sm"  onClick={() => {copyToClipboard(this.props.full_image_path)}}>
      <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-copy"><rect x="9" y="9" width="13" height="13" rx="2" ry="2"></rect><path d="M5 15H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h9a2 2 0 0 1 2 2v1"></path></svg>
    </Button>
  </>;
  return(<React.Fragment>
    {copy_btn}
  </React.Fragment>);
  }

}
   
  
class DeleteImageButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }
    
    render(){
    let delete_confirmation_screen =     <>
      <Button variant="primary btn-sm"  onClick={() => this.handleShow()} >
        <svg  xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-trash-2"><polyline points="3 6 5 6 21 6"></polyline><path d="M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6m3 0V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2"></path><line x1="10" y1="11" x2="10" y2="17"></line><line x1="14" y1="11" x2="14" y2="17"></line></svg> 
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header closeButton>
          <Modal.Title>Delete Image</Modal.Title>
        </Modal.Header>
        <Modal.Body>
          To generate a script to delete this image from the registry, click submit. You can then either place this script on the (virtual) machine where BeetleboxCI is installed and run the script, or copy-and-paste the code from the script into a terminal on the (virtual) machine and run it directly. 
        </Modal.Body>
        <Modal.Footer>
        <DeleteForm image_name={this.props.image_name} onHide={this.handleClose}  closeModal={() => this.handleClose()}/> 

        </Modal.Footer>
      </Modal>
    </>;
    return(<React.Fragment>
    {delete_confirmation_screen}
    </React.Fragment>);
    }
}

function DeleteForm(props) {
  function handleSubmit(e) {
    e.preventDefault();
    let data = JSON.stringify({
        name: "delete_image_",
        image: props.image_name
    });
    
    var filename;
    
    const requestOptions = {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'X-CSRFToken': getCookie('csrftoken'),
          },
          credentials: 'same-origin',
          body: data
          };
      fetch('', requestOptions)
          .then(response => {
              // console.log(response)
              if (!response.ok) {
                throw new Error(response);
              }
              // window.location.href = response.url;
              filename = "delete_image.sh"
              return response.blob()
          })
          .then(blob => {

              // console.log(blob)
              const url = window.URL.createObjectURL(blob);
              const a = document.createElement('a');
              a.style.display = 'none';
              a.href = url;
              a.download = filename;
              document.body.appendChild(a);
              a.click();
              document.body.removeChild(a);
              window.URL.revokeObjectURL(url);
              a.remove();  //afterwards we remove the element again
            })      

    props.closeModal();
  }

  return (
    <form onSubmit={handleSubmit}>
      <button className="btn btn-primary " type="submit" id="delete_image_" name="delete_image_">Submit</button>
    </form>
  );
}
     