import React from 'react';
import ReactDOM from "react-dom";
import styled from "styled-components";
// import Editor from './editor.js';
// import AceEditor from "react-ace";
import ellipse from './img/ellipse.png';
import running_icon from './img/beetlebox-running-icon.png';
import static_running_icon from './img/beetleboxci-static-running-icon.svg';
import pending_icon from './img/beetleboxci-pending-icon.png';
import static_pending_icon from './img/beetleboxci-static-pending-icon.svg';
import failure_icon from './img/beetleboxci-failure-icon.svg';
import pass_icon from './img/beetleboxci-pass-icon.svg';
import warning_icon from './img/beetleboxci-warning-icon.svg';
import clipboard from './img/clipboard.svg'
import download from './img/download.svg'
import CodeMirror from '@uiw/react-codemirror';

import {getCookie} from "./shared_functions.js"


import StopWatch from "./stopwatch.js"
import {formatDate, humanFileSize, downloadFile} from "./shared_functions.js"


import { Modal, Table, Container, Row, Col, Card} from 'react-bootstrap';
import {Breadcrumb} from './shared_classes.js'


// import { createWriteStream } from 'streamsaver'
import { WritableStream } from 'web-streams-polyfill/ponyfill';
import streamSaver from 'streamsaver';

import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 
const Button = styled.button`
  color: black;
  height: 100%;
  width: 100%;
  cursor: pointer;
  padding: 0;
  border: none;
  padding-bottom: 10px;
  padding-top: 10px;
  text-align: left;
  
`;

const ClipboardButton = styled.button`
  color: black;
  cursor: pointer;
  border: none;
  text-align: left;
  background: none;
`;

const ButtonToggle = styled(Button)`
  background-color: white;
  border-radius: 4px;
  margin: 2px;
  ${({ active }) =>
    active &&
    `
    background-color: #E3E3E3;
  `}
`;


export default class JobView extends React.Component {
  constructor(props) {
    super(props);
    var pathArray = window.location.pathname.split('/');
    var job_no=pathArray[2];
    let api_url = pathArray[0]
    this.job_url = window.location.protocol + '//' + window.location.host + '/api/job/' + job_no;
    let step_url_base= api_url + 'api/step/';
    this.steps_jsx=[];
  }
  
  
  render() {
    return (
      <div>
          <QueryClientProvider client={queryClient}>
            <FetchJobs job_url={this.job_url} />
          </QueryClientProvider>
      </div>
    );
  }
}

const queryClient = new QueryClient();


function FetchJobs(job_url) {
 const { isLoading, error, data } = useQuery('job_data', () =>
   fetch(job_url.job_url).then(res =>
     res.json()
   ),
    {
    // Refetch the data every three second
    refetchInterval: 3000,
  }
 )


  
  if (isLoading) return (<div className="container-xl px-4 mt-5">
  <div className='row justify-content-md-center'>
    <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
  </div>
  </div>)
  
  if (error) return 'An error has occurred: ' + error.message
  if (data.error) return data.error.name + ': ' + data.error.message;
  
  var status_image, supporting_text;
  var svg_size=40;
  var gif_size=20;
  let stopwatch_status;
  let trigger_date = new Date(data.last_run);
  let trigger_date_string=formatDate(trigger_date);
  
  if (data.workflow_status=='IN'){
    status_image=<img src={static_pending_icon} alt="static_pending_icon" style={ {width: '50px'}} />;
    stopwatch_status='ID';
    trigger_date_string='';  
  } else if (data.status=='PE'||data.status=='PR'){
    status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '50px'}} />
    stopwatch_status='PE';
    trigger_date_string='';
  } else if (data.status=='ID'){
    status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '50px'}} />
    stopwatch_status='ID'
    trigger_date_string='';  
  } else if (data.status=='PD'||data.status=='QU'||data.status=='Q1'||data.status=='Q2'||data.status=='Q3'||data.status=='Q4'||data.status=='Q5'||data.status=='Q6'){
    status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '50px'}} />
    stopwatch_status='QU'
    trigger_date_string='';
  } else if (data.status=='PA'){
    status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '50px'}} />
    stopwatch_status='PA'
    supporting_text='Triggered on:';
  } else if (data.status=='RU'){
    status_image=<img src={running_icon} alt="loading_icon" style={ {width: '50px'}} />
    stopwatch_status='RU';
    supporting_text='Triggered on:';
  } else if (data.status=='ST'){
    status_image=<img src={running_icon} alt="loading_icon" style={ {width: '50px'}} />
    stopwatch_status='RU';
    supporting_text='Triggered on:';  
  } else if (data.status=='FA'){
    stopwatch_status='FA';
    status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '50px'}} />
    supporting_text='Triggered on:';
  }

  let status_card=      <div className="col-xl-3 mb-4 align-items-stretch">
          <div className="card h-100">
          <div className="card-body">
           <div className="row">
              <div className="h3">Status</div>
           </div>
           <div className="row">
            <div className="col-md-auto">
              {status_image}
            </div>
            <div className="col-md-auto align-self-center">
              <StopWatch status={data.status} start_time={data.runtime} status={stopwatch_status}></StopWatch>
            </div>
            </div>
            <div className="row align-self-center">
            <p>
            {supporting_text} <br></br>
            {trigger_date_string} </p>
           </div>
           </div>
          </div>
        </div>
  let result_pass_image=<img src={pass_icon} alt="pass_icon" style={ {width: '18px'}} />;
  let result_fail_image=<img src={failure_icon} alt="fail_icon" style={ {width: '18px'}} />;
  let result_not_run_image=<img src={warning_icon} alt="warning_icon" style={ {width: '18px'}} />;
  let result_running_image=<img src={static_running_icon} alt="running_icon" style={ {width: '18px'}} />;
  let result_pending_image=<img src={static_pending_icon} alt="pending_icon" style={ {width: '18px'}} />;
  
  let results_card=       <div className="col-xl-3 mb-4 align-items-start">
          <div className="card h-100">
            <div className="card-body">
             <div className="row">
                <div className="h3">Results</div>
              </div>
                <div className="row">
                  <div className="col-md-auto">
                    {result_pass_image}
                  </div>
                  <div className="col-md-auto">
                    Passed: {data.number_steps_passed}
                  </div>
                </div>
                  <div className="row">
                    <div className="col-md-auto">
                      {result_fail_image}
                    </div>
                    <div className="col-md-auto">
                      Failed: {data.number_steps_failed}
                    </div>
                  </div>
                  <div className="row">
                    <div className="col-md-auto">
                      {result_not_run_image}
                    </div>
                    <div className="col-md-auto">
                      Not Run: {data.number_steps_not_run}
                    </div>
                  </div>
                  <div className="row">
                    <div className="col-md-auto">
                      {result_running_image}
                    </div>
                    <div className="col-md-auto">
                      Running: {data.number_steps_running}
                    </div>
                  </div>
                  <div className="row">
                    <div className="col-md-auto">
                      {result_pending_image}
                    </div>
                    <div className="col-md-auto">
                      Pending: {data.number_steps_pending}
                    </div>
                  </div>
             </div>
           </div>
         </div>
  let resource_card=     <div className="col-xl-3 mb-4 align-items-start">
        <div className="card h-100">
        <div className="card-body">
        
         <div className="row">
            <div className="h3">Resource Usage</div>
         </div>
         
         <div className="row">
            <div className="h5">Requested</div>
         </div>         
         
         <div className="row">
          <div className="col-md-6">
            CPU: {data.CPU_request} 
          </div>
          <div className="col-md-6">
            RAM: {data.RAM_request} 
          </div>
         </div>
         

         <div className="row">
            <div className="h5">Current Usage</div>
         </div>                  
         
         <div className="row">
          <div className="col-md-6">
            CPU: {data.CPU_usage} 
          </div>
          <div className="col-md-6">
            RAM: {data.RAM_usage} 
          </div>
         </div>         
         
         </div>
        </div>
      </div>

  let human_storage=humanFileSize(data.storage_usage)
  let artifact_card=     <div className="col-xl-3 mb-4 align-items-stretch">
          <div className="card h-100">
          <div className="card-body">
           <div className="row">
              <div className="h3">Artifacts</div>
           </div>
           <div className="row">
              <div className="col">
                <div className="h5">Number of Artifacts</div>
              </div>
            </div>
           <div className="row">
              <div className="col">
                <div className="h7">{data.number_artifacts}</div>
              </div>
            </div>          
            <div className="row">
              <div className="col">
                <div className="h5">Current Usage</div>
              </div>
            </div>
            <div className="row">
              <div className="col">
                <div className="h7">{human_storage}</div>
              </div>
            </div>            
           </div>
          </div>
        </div>
  
  let log_card=  <div className="card">
    <div className="card-body">
      <div>
        <JobRows step_data={data}/>
      </div>
    </div>
  </div>
    
  let artifact_tab_card=  <div className="card">
    <div className="card-body">
      <div>
        <ArtifactTab artifact_data={data}/>
      </div>
    </div>
  </div>

 let artifact_tab=<div></div>;
 
 if (data.artifacts.length!=0) {
    artifact_tab=<li className="nav-item" role="presentation">
      <button className="nav-link" id="profile-tab" data-bs-toggle="tab" data-bs-target="#profile" type="button" role="tab" aria-controls="profile" aria-selected="false">Artifacts</button>
    </li>   
 }

 let pageList=[['Pipelines', data.pipelines_url], [data.project_name, data.project_url], [data.branch, data.project_url + "?branchfilters=" + data.branch], [data.workflow_name, data.workflow_url], [data.job_name, window.location.pathname]];

 return (
  <Container fluid className="px-4 mt-5">
    <Row>
        <Breadcrumb pageList={pageList} />
    </Row>
    <h1 className="mb-4" >{data.job_name}</h1>
    <div className="row">
      {status_card}
      {results_card}
      {resource_card}
      {artifact_card}
    </div>
  <ul className="nav nav-pills" id="myTab" role="tablist">
    <li className="nav-item" role="presentation">
      <button className="nav-link active" id="home-tab" data-bs-toggle="tab" data-bs-target="#home" type="button " role="tab" aria-controls="home" aria-selected="true">Log</button>
    </li>
    
    {artifact_tab}
    
  </ul>
  
  <div className="tab-content mt-2" id="myTabContent">
    <div className="tab-pane fade show active" id="home" role="tabpanel" aria-labelledby="home-tab">{log_card}</div>
    <div className="tab-pane fade" id="profile" role="tabpanel" aria-labelledby="profile-tab">{artifact_tab_card}</div>
  </div>
  </Container>
  ) 
} 
 
 
function DownloadArtifact(props) {
  
  
  function handleClick(e) {
    e.preventDefault();

    let data = JSON.stringify({
        name: "download_artifact_",
        id_: props.artifact_id
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
            
            
            
    fetch('', requestOptions)
    
        .then(res => res.json()) // expecting a json response
        .then((data) => {
           console.log(data)
           window.location.href = data.redirect;
          });    


      // .then(response => {
      //     console.log(response)
          
      //     if (!response.ok) {
      //       throw new Error(response);
      //     }
          

          
      // })

          


         
            
  }

  return (
    <form action=""   action={props.artifact_url} target="_blank"  method="get">  <button className="btn btn-primary btn-sm "  type="submit"  data-bs-toggle="tooltip" data-bs-placement="top" title="Download item"><svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-download"><path d="M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4"></path><polyline points="7 10 12 15 17 10"></polyline><line x1="12" y1="15" x2="12" y2="3"></line></svg></button></form>

  );
} 
 
class ArtifactTab extends React.Component{
  
  render() {
  let artifact_rows=[]
  let artifacts=this.props.artifact_data.artifacts
  for (var i = 0; i < artifacts.length; i++) {
    let artifact=artifacts[i]
    let name=artifacts[i].artifact_name
    
    let download_artifact=<DownloadArtifact artifact_id={artifact.artifact_id} artifact_url={artifact.url} ></DownloadArtifact>;
    
    artifact_rows.push(
      
      <tr role="row" className="odd" key={i}>
        <td>{artifact.artifact_name}</td>
        <td>{humanFileSize(artifact.size)}</td>
        <td>{download_artifact}</td>
      </tr>
      );
  }
  
  let table = <div>
  <table className="table table-bordered table-striped dataTable" id="user-table" role="grid">
      <thead>
      <tr>
          <th> Name </th>
          <th> Size </th>
          <th> Download </th>
      </tr>
  </thead>
  <tbody>
    {artifact_rows}
  </tbody>
  </table>
  </div>
    return (
  <div className="row">
    {table}
  </div>
  );
    
  }
  
}
 

class JobRow extends React.Component {  
     render() {
         var i=this.props.index;
         var status_image;
         var svg_size="15px";
         var gif_size="15px";
         
         if (this.props.workflow_status=='IN'){
           status_image=<img src={static_pending_icon} alt="static_pending_icon" style={ {width: '50px', paddingLeft: "10px", paddingRight: "10px"}} />
         }         
         else if (this.props.status=='PE'){
           status_image=<img src={running_icon} alt="loading_icon" style={ {width: '50px', paddingLeft: "10px", paddingRight: "10px"}} />
         }
         else if (this.props.status=='FA'){
           status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '50px', paddingLeft: "10px", paddingRight: "10px"}} />
         }
         else if (this.props.status=='PA'){
           status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '50px', paddingLeft: "10px", paddingRight: "10px"}} />
         }
         else if (this.props.status=='ID'){
           status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '50px', paddingLeft: "10px", paddingRight: "10px"}} />
         }
         else if (this.props.status=='NR'){
           status_image=<img src={warning_icon} alt="warning_icon" style={ {width: '50px', paddingLeft: "10px", paddingRight: "10px"}} />
         }
         else {
           status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '50px', paddingLeft: "10px", paddingRight: "10px"}} />
         }         
         return (
        <ButtonToggle onClick={(i) => this.props.onClick(i)} active={this.props.active} >
          {status_image}
          {this.props.step_name}
        </ButtonToggle>
    );
  }
}
 
class JobRows extends React.Component {
  constructor(props) {
    super(props);
    this.step= props.step_data.steps.length;
    this.number_of_steps= props.step_data.steps.length;
    var step_array=Array(this.number_of_steps).fill(false);
    step_array[0]=true;
    this.state = {
      JobActiveState: step_array,
      log : this.props.step_data.steps[0].output,
    };
  }


  
  handleClick(i) {
      const JobActiveState = this.state.JobActiveState.slice();
      JobActiveState.fill(false);
      JobActiveState[i] = true;
      this.setState({JobActiveState: JobActiveState,
      });
  }
  
  renderJob(i){

    return (
        <JobRow key={i}
        workflow_status={this.props.step_data.workflow_status}
        step_name={this.props.step_data.steps[i].step_name}
        active={this.state.JobActiveState[i]}
        status={this.props.step_data.steps[i].status}
        index={i}
        onClick={() => this.handleClick(i) }>
        </JobRow>
    );
  }
  
  //  copyToClipboard(log) {
  //   navigator.clipboard.writeText(log);
  // } 
  
  fetchActiveLog(){
      const JobActiveState = this.state.JobActiveState.slice();
      var ActiveState=0
      for (var i = 0; i < this.number_of_steps; i++) {
        if (JobActiveState[i] == true) {
            var log= this.props.step_data.steps[i].output;
            ActiveState=i;
        }
      }
  // console.log(document.getElementsByTagName("*").length)
      return(
        <div className='container'>
          {/* <div className='row'>
            <div className='col pb-1'>
              <ClipboardButton onClick={() => this.copyToClipboard(log)}>
                <img src={clipboard} alt="clipboard" style={ {width: '24px', stroke : "green"}} /> Copy to Clipboard
              </ClipboardButton>
            </div>
          </div> */}
          <div>
          <CodeMirror
            value={log}
            height="500px"
            theme="dark"
          />
        </div>
        </div>
    );
  }
  
  render() {
    var job_rows=[]
    var active_log=<div> {this.fetchActiveLog()}</div>
    
    for (var i = 0; i < this.number_of_steps; i++) {
      job_rows.push(
        <div className="job-row" key={i} >
        {this.renderJob(i)}
        </div>
        );
    }
    return (
    <div className="row">
      <div className="col-md-3 g-0">
        {job_rows}
      </div>
      <div className="col-md-9 ">
        {active_log}
      </div>
    </div>
    );
  }
}
 