import React from 'react';
import running_icon from './img/beetlebox-running-icon.png';

import { Modal, Pagination, Button, Table, Container, Row, Col, Card, Alert, Dropdown, DropdownButton,  ButtonGroup} from 'react-bootstrap';
// import {Breadcrumb} from './shared_classes.js'
import {RefreshButton} from './shared_classes.js'
import {CloseForm, getCookie} from './shared_functions.js'

import git_branch from './img/git-branch-svg-repo.svg';

import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 
 const queryClient = new QueryClient();

export default class PipelinesView extends React.Component {
  constructor(props) {
    super(props);
    var pathArray = window.location.pathname.split('/');

    this.pipelines_outline_url = window.location.protocol + '//' + window.location.host + '/api/pipelines_detail';

  }  
  
  render() {
    return (
      <div>
        <QueryClientProvider client={queryClient}>
          <Container fluid className="px-4 mt-5">
            <Row>
              <h1 className="mb-0">Pipelines</h1>
              <div className="small">
                <span className="fw-500 text-primary">Monitor or create new pipelines</span>
              </div>            
            </Row>            

            <PipelinesOutline pipelines_url={this.pipelines_outline_url} />

          </Container>
        </QueryClientProvider>
      </div>
    );
  }
}

function PipelinesOutline(props) {


  const requestOptions = {
    method: 'GET',
    headers: {
      'Content-Type': 'application/json',
      'name': "get_outline_",
    },
  };

  const fetchData = async () => {
    const res = await fetch(props.pipelines_url, requestOptions);
    return res.json();
  };

  const { isLoading, error, data } = useQuery("pipelines_data", fetchData);



  if (isLoading) return (<div className="container-xl px-4 mt-5">
  <div className='row justify-content-md-center'>
    <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
    </div>
  </div>);

  if (error) return 'An error has occurred: ' + error.message;
  if (data.error) return data.error.name + ': ' + data.error.message;
 
  let pipeline_card=<div></div>;
  if (data.num_pipelines==0){
      pipeline_card=<div className="card card-waves mb-4 mt-5">
        <div className="card-body p-5">
          <div className="row align-items-center justify-content-between">
            <div className="col">
              <h2 className="text-primary">Welcome to BeetleboxCI, your account is ready!</h2>
              <p className="text-gray-700">To get started please check out our documentation or create a new pipeline. Your pipelines will contain workflows that are used to manage the jobs you want to run.</p>
              <a className="btn btn-primary p-3" href={data.create_pipeline_url}>
                Create Pipeline
              <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-arrow-right ms-1"><line x1="5" y1="12" x2="19" y2="12"></line><polyline points="12 5 19 12 12 19"></polyline></svg>
              </a>
            </div>
          <div className="col d-none d-lg-block mt-xxl-n4"><img className="img-fluid px-xl-4 mt-4" src="/static/assets/img/Intro-pic.png"></img></div>
          </div>
        </div>
        </div>
  } else {
      pipeline_card= <div className="card p-0 mb-4 mt-4">
        <div className="card-body p-0">
            <PipelinesRows pipelines_url={props.pipelines_url} num_pipelines={data.num_pipelines} number_of_rows_per_page="10" />          
        </div>
        <a className="btn btn-primary btn-xl" href={data.create_pipeline_url}>Create Pipeline</a>
    </div>
   }
   
  //   let pageList=[['Pipelines', window.location.pathname] ];
 
  return (     
    <div>

        <CreateProjectError error_text={data.create_project_error}></CreateProjectError>
        <Row className="mx-0">
            {pipeline_card}
        </Row>
    </div>
  );
 }
 

class CreateProjectError extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      error_text: ''
    };    
  }
  
  componentDidMount(){
    if (this.props.error_text.length!=0){
      this.setState({error_text:this.props.error_text});
    }
  }
    
  render(){
    
    let create_project_error_output = <div></div>;
        
    if (this.props.error_text.length!=0){
      create_project_error_output =<div>
            <Alert key="danger" variant="danger">
              {this.props.error_text}
            </Alert>
        </div>;
    } 
    else if (this.state.error_text.length!=0){
      create_project_error_output =<div>
            <Alert key="danger" variant="danger">
              {this.state.error_text}
            </Alert>
        </div>;
    }
    
    return(<React.Fragment>
    {create_project_error_output}
    </React.Fragment>);
  }  
}


class PipelinesRows extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      currentPageIndex: 0,
    };
    
  }  
  
  paginationClicked(number){
    this.setState({currentPageIndex : number-1});
  }
  
  paginationFirstClicked(){
    this.setState({currentPageIndex : 0});
  }
  
  paginationPreviousClicked(){
    if (this.state.currentPageIndex==0){
      this.setState({currentPageIndex : 0});
    } else{
      let tempNumber=this.state.currentPageIndex-1;
      this.setState({currentPageIndex : tempNumber});
    }
  }
  
  paginationNextClicked(number_of_pages){
    if (this.state.currentPageIndex==number_of_pages-1){
      this.setState({currentPageIndex : number_of_pages-1});
    } else{
      let tempNumber=this.state.currentPageIndex+1;
      this.setState({currentPageIndex : tempNumber});
    }  
  }
  
  paginationLastClicked(number_of_pages){
    this.setState({currentPageIndex : number_of_pages-1});
  }
 
  render(){

    let number_of_pages=Math.ceil(this.props.num_pipelines / this.props.number_of_rows_per_page);
    let items = [];
    if (number_of_pages<8){
      for (let number = 1; number <= number_of_pages; number++) {
        items.push(
          <Pagination.Item key={number} onClick={() => this.paginationClicked(number)} active={number === parseInt(this.state.currentPageIndex+1)}>
            {number}
          </Pagination.Item>,
        );
      }
    } else {
      for (let number = 1; number <= 8; number++) {
        items.push(
          <Pagination.Item key={number} onClick={() => this.paginationClicked(number)} active={number === parseInt(this.state.currentPageIndex+1)}>
            {number}
          </Pagination.Item>,
        );
      }
      if (8<this.state.currentPageIndex+1){
          items.push(<Pagination.Ellipsis key="ellipse1" onClick={() => this.paginationEllipseClicked()}/>);
          items.push(<Pagination.Item key="current" onClick={() => this.paginationClicked(this.state.currentPageIndex+1)} active={true}>{this.state.currentPageIndex+1} </Pagination.Item>);
          if ((this.state.currentPageIndex+1)!=number_of_pages){
          items.push(<Pagination.Ellipsis key="ellipse2" onClick={() => this.paginationEllipseClicked()}/>);
          }
      }
      else{
        items.push(<Pagination.Ellipsis key="ellipse1" onClick={() => this.paginationEllipseClicked()}/>);

      }

    }        
    
    // Note that the "key" element of the FetchPipelines has to change in order to force it to be rerendered. 
    return(
      <React.Fragment>
        <Row>
          <Table className="table dataTable" id="projectsTable" role="grid">
            <thead>
                <tr>
                  <th style={{width: "25%"}}> Pipeline </th>
                  <th style={{width: "25%"}}> Owner </th>
                  <th style={{width: "12.5%"}}> Last Run </th>
                  <th style={{width: "12.5%"}}> Git Provider </th>
                  <th style={{width: "25%"}}> Actions </th>
                </tr>
            </thead>
            
            <tbody>
            <FetchPipelines pipelines_url={this.props.pipelines_url} page={this.state.currentPageIndex+1} key={this.state.currentPageIndex+1}> </FetchPipelines>
            </tbody>
          </Table>
        </Row>
        <Row className="justify-content-md-center">
          <Col md="auto">
          <Pagination>
            <Pagination.First onClick={() => this.paginationFirstClicked()}  />
            <Pagination.Prev onClick={() => this.paginationPreviousClicked()}  />
            {items}
            <Pagination.Next onClick={() => this.paginationNextClicked(number_of_pages)}  />
            <Pagination.Last onClick={() => this.paginationLastClicked(number_of_pages)}  />
          </Pagination>
          </Col>
        </Row>
        
      </React.Fragment>
      
        );
  }
} 



function FetchPipelines(props) {

  const requestOptions = {
    method: 'GET',
    headers: {
      'Content-Type': 'application/json',
      'name': "get_list_",
      'page': props.page
    },
  };

  // console.log("abcd")

  const { isLoading, error, data } = useQuery('pipeline_data', () =>
    fetch(props.pipelines_url, requestOptions).then(res =>
      res.json()
    ),
    // {
    // refetchInterval: 3000,
    // }
  );

  // const fetchData = async () => {
  //   const res = await fetch(props.pipelines_url, requestOptions);
  //   return res.json();
  // };
  // const { isLoading, error, data } = useQuery("pipeline_list", fetchData);

  if (isLoading) return (
    <React.Fragment>
      <tr>
        <td className="align-middle"><img src={running_icon} alt="loading_icon" style={ {width: '40px'}} /> &nbsp;&nbsp;Loading</td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
      </tr>
    </React.Fragment>
  );

  else if (error) return (
    <React.Fragment>
      <tr>
        <td className="align-middle">An error has occurred:&nbsp;{error.message}</td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
      </tr>
    </React.Fragment>
  )

  else if (data.error) return (
    <React.Fragment>
      <tr>
        <td className="align-middle">{data.error.name}:&nbsp;{data.error.message}</td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
        <td className="align-middle"></td>
      </tr>
    </React.Fragment>
    
  )

  let pipelines=data.pipelines;
  let pipeline_rows=[];
  
  // console.log(pipelines)
    
  for (var i = 0; i < pipelines.length; i++) {
            
    let error_status = <div></div>
      
    if (pipelines[i].status=='CE'){
        error_status=<div style={{ color: 'red' }}>Config error</div>;
    }     

    pipeline_rows.push(
    <tr role="row" className="odd" key={i}>
      <td className="align-middle"><a href={pipelines[i].pipeline_url}>{pipelines[i].pipeline_name}</a>{error_status}</td>
      <td className="align-middle">{pipelines[i].owner}</td>
      <td className="align-middle">{pipelines[i].last_run}</td>
      <td className="align-middle">{pipelines[i].git_provider}</td>
      <td className="align-middle"><InfoButton pipeline_repo={pipelines[i].pipeline_repo}></InfoButton>&nbsp;<DeleteButton pipeline_id={pipelines[i].pipeline_id}></DeleteButton>&nbsp;<RefreshButton pipeline_id={pipelines[i].pipeline_id}></RefreshButton>&nbsp;<ConfigForm branches={pipelines[i].branches} pipeline_id={pipelines[i].pipeline_id}  ></ConfigForm><ConfigMessageScreen display_config_message={pipelines[i].display_config_message} pipeline_id={pipelines[i].pipeline_id} pipeline_name={pipelines[i].pipeline_name} branches={pipelines[i].branches} /></td>
    </tr>
    );    

  }
   
  //   let pageList=[['Pipelines', window.location.pathname] ];
 
  return (     
    <React.Fragment>
        {pipeline_rows}
    </React.Fragment>

  );
 }


class ConfigForm extends React.Component {
  constructor(props) {
    super(props);

  }
  
  handleSubmit(e) {
    // e.preventDefault();
    let data = JSON.stringify({
        name: "config_pipeline_",
        id_: this.props.pipeline_id,
        branch_: e, 
        edit_config_: 1,
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            .then(response =>  {
              // console.log(response)
              window.location.href = response.url;
              });             
  } 
    
  render(){
    
    // console.log(this.props.branches)
    
    let branches=this.props.branches;
    var dropDownItems;  

    if (branches.length > 0){
      dropDownItems = []
      for (var i = 0; i < branches.length; i++) {
        dropDownItems.push(
          <Dropdown.Item key={branches[i].branch_name} eventKey={branches[i].branch_name}> <img src={git_branch} alt="git-branch" style={ {width: '15px'}} />&nbsp;{branches[i].branch_name}</Dropdown.Item>
          );
      }
    } else {
      dropDownItems = <Dropdown.Header><img src={running_icon} alt="loading_icon" style={ {width: '15px'}} />&nbsp; Loading</Dropdown.Header>
    }
  
    return (
      <React.Fragment>
        <DropdownButton 
          title={<svg xmlns="http://www.w3.org/2000/svg" width="12" height="12" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-settings"><circle cx="12" cy="12" r="3"></circle><path d="M19.4 15a1.65 1.65 0 0 0 .33 1.82l.06.06a2 2 0 0 1 0 2.83 2 2 0 0 1-2.83 0l-.06-.06a1.65 1.65 0 0 0-1.82-.33 1.65 1.65 0 0 0-1 1.51V21a2 2 0 0 1-2 2 2 2 0 0 1-2-2v-.09A1.65 1.65 0 0 0 9 19.4a1.65 1.65 0 0 0-1.82.33l-.06.06a2 2 0 0 1-2.83 0 2 2 0 0 1 0-2.83l.06-.06a1.65 1.65 0 0 0 .33-1.82 1.65 1.65 0 0 0-1.51-1H3a2 2 0 0 1-2-2 2 2 0 0 1 2-2h.09A1.65 1.65 0 0 0 4.6 9a1.65 1.65 0 0 0-.33-1.82l-.06-.06a2 2 0 0 1 0-2.83 2 2 0 0 1 2.83 0l.06.06a1.65 1.65 0 0 0 1.82.33H9a1.65 1.65 0 0 0 1-1.51V3a2 2 0 0 1 2-2 2 2 0 0 1 2 2v.09a1.65 1.65 0 0 0 1 1.51 1.65 1.65 0 0 0 1.82-.33l.06-.06a2 2 0 0 1 2.83 0 2 2 0 0 1 0 2.83l-.06.06a1.65 1.65 0 0 0-.33 1.82V9a1.65 1.65 0 0 0 1.51 1H21a2 2 0 0 1 2 2 2 2 0 0 1-2 2h-.09a1.65 1.65 0 0 0-1.51 1z"></path></svg>} 
          variant="primary" 
          size="sm" 
          as={ButtonGroup} 
          onSelect={(eventKey)=> {this.handleSubmit(eventKey)}}
        >
        
          <Dropdown.Header>Branch</Dropdown.Header>
          {dropDownItems}

        </DropdownButton>  
      </React.Fragment>
        
      );
  }
}  

// modal to prompt the user to set up a config file 
class ConfigMessageScreen extends React.Component {
  constructor(props) {
    super(props);
    
    // this.handleSubmitYes = this.handleSubmitYes.bind(this);
    this.handleSubmitNo = this.handleSubmitNo.bind(this);
    
    this.handleClose = this.handleClose.bind(this);
    this.pipeline_id=this.props.pipeline_id
    this.branch=this.props.branch
        
    if  (this.props.display_config_message){
        this.state = {
          show: true,
        };
    } else{
        this.state = {
          show: false,
        };
    }    
     
  }
  
  handleSubmitYes(e) {
    // e.preventDefault();
    let data_yes = JSON.stringify({
        name: "config_pipeline_",
        id_: this.props.pipeline_id,
        branch_: e,
        edit_config_: 1,
    });
    const requestOptions = {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'X-CSRFToken': getCookie('csrftoken'),
          },
          credentials: 'same-origin',
          body: data_yes,
          };
      fetch('', requestOptions)
          .then(response =>  {
            // console.log(response)
            window.location.href = response.url;
            });             
  }  
    
  handleSubmitNo(e) {
      e.preventDefault();
      let data_no = JSON.stringify({
          name: "config_pipeline_",
          id_: this.props.pipeline_id,
          branch_: this.branch,
          edit_config_: 0,
      });    
      const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data_no,
            };
        fetch('', requestOptions)
            .then(response =>  {
              // console.log(response)
              window.location.href = response.url;
              });             
  }    
  
  // handleShow(){
  //     this.setState({show:true});
  // }
 
  handleClose(){
      this.setState({show:false});
  }
    
  render(){
     
    let branches=this.props.branches;
    var dropDownItems;

    if (branches.length > 0){
      dropDownItems = []
      for (var i = 0; i < branches.length; i++) {
        dropDownItems.push(
          <Dropdown.Item key={branches[i].branch_name} eventKey={branches[i].branch_name}> <img src={git_branch} alt="git-branch" style={ {width: '15px'}} />&nbsp;{branches[i].branch_name}</Dropdown.Item>
          );
      }
    } else {
      dropDownItems = <Dropdown.Header><img src={running_icon} alt="loading_icon" style={ {width: '15px'}} />&nbsp; Loading</Dropdown.Header>
    }    
    
    
    let config_confirmation_screen =     <>
  
      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header >
          <Modal.Title>Set up pipeline config for {this.props.pipeline_name}</Modal.Title>
        </Modal.Header>
        <Modal.Body> Would you like to set up the configuration file for the pipeline {this.props.pipeline_name} now?</Modal.Body>
        <Modal.Footer>
        

          
          <Dropdown align="end" onSelect={(eventKey)=> {this.handleSubmitYes(eventKey)}}>
            <Dropdown.Toggle variant="primary" size="sm"  id="dropdown-basic">
              Yes
            </Dropdown.Toggle>
            <Dropdown.Menu>
              <Dropdown.Header>Branch</Dropdown.Header>
              {dropDownItems}
            </Dropdown.Menu>
          </Dropdown>            
          
          
          <form onSubmit={this.handleSubmitNo}>
            <button className="btn btn-primary btn-sm" type="submit" >
                No
            </button>
          </form>           
  
  
        </Modal.Footer>
      </Modal>
    </>;
    return(<React.Fragment>
    {config_confirmation_screen}
    </React.Fragment>);
  }
}

   

class InfoButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }  
  handleShow(){
      this.setState({show:true});
  } 
  handleClose(){
      this.setState({show:false});
  }    
    render(){
      let info_data =     <>
        <Button variant="primary btn-sm"  onClick={() => this.handleShow()} >
        <svg xmlns="http://www.w3.org/2000/svg" width="12" height="12" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-info"><circle cx="12" cy="12" r="10"></circle><line x1="12" y1="16" x2="12" y2="12"></line><line x1="12" y1="8" x2="12.01" y2="8"></line></svg>
        </Button>

        <Modal dialogClassName="modal-xl" show={this.state.show} onHide={() => this.handleClose()}>
          <Modal.Header closeButton>
            <Modal.Title>Pipeline information</Modal.Title>
          </Modal.Header>
          <Modal.Body>
            Repository location: {this.props.pipeline_repo}
          </Modal.Body>
          <Modal.Footer>
          <CloseForm onHide={this.handleClose}  closeModal={() => this.handleClose()}/>           
          </Modal.Footer>
        </Modal>
      </>;
      return(<React.Fragment>
      {info_data}
      </React.Fragment>);
      
    }
}


  
  
class DeleteButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }
    
    render(){
    let delete_confirmation_screen =     <>
      <Button variant="primary btn-sm"  onClick={() => this.handleShow()} >
        <svg  xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="feather feather-trash-2"><polyline points="3 6 5 6 21 6"></polyline><path d="M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6m3 0V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2"></path><line x1="10" y1="11" x2="10" y2="17"></line><line x1="14" y1="11" x2="14" y2="17"></line></svg> 
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header closeButton>
          <Modal.Title>Delete Pipeline</Modal.Title>
        </Modal.Header>
        <Modal.Body>Are you sure you want to delete this pipeline?</Modal.Body>
        <Modal.Footer>
        <DeleteForm pipeline_id={this.props.pipeline_id} onHide={this.handleClose}  closeModal={() => this.handleClose()}/> 

        </Modal.Footer>
      </Modal>
    </>;
    return(<React.Fragment>
    {delete_confirmation_screen}
    </React.Fragment>);
    }
}

function DeleteForm(props) {
  function handleSubmit(e) {
    e.preventDefault();
    let data = JSON.stringify({
        name: "delete_project_",
        id_: props.pipeline_id
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            .then();
    props.closeModal();
  }

  return (
    <form onSubmit={handleSubmit}>
      <button className="btn btn-danger " type="submit" id="delete_project_" name="delete_project_">Yes</button>
    </form>
  );
}
  