import React from 'react';
import ReactDOM from "react-dom";

import StopWatch from "./stopwatch.js"
import git_branch from './img/git-branch-svg-repo.svg';
import git_branch_white from './img/git-branch-white-svg-repo.svg';

import pending_icon from './img/beetleboxci-pending-icon.png';
import static_pending_icon from './img/beetleboxci-static-pending-icon.svg';
import failure_icon from './img/beetleboxci-failure-icon.svg';
import pass_icon from './img/beetleboxci-pass-icon.svg';
import running_icon from './img/beetlebox-running-icon.png';
import play_icon from './img/playbutton.svg';
import config_icon from './img/configbutton.svg';

import stop_icon from './img/stop.svg';

import $ from 'jquery';
import {Button, Modal, Table, Container, Row, Col, Tabs, Pagination, Tab, Dropdown, Card, Alert} from 'react-bootstrap';
import {Breadcrumb, LogButton, RefreshButton} from './shared_classes.js'
import {getCookie, CloseForm} from "./shared_functions.js"

import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query';
 
const queryClient = new QueryClient();


export default class PipelineView extends React.Component {
      constructor(props) {
        super(props);
        var pathArray = window.location.pathname.split('/');
        var pipeline_no=pathArray[2];
        this.pipeline_url = window.location.protocol + '//' + window.location.host + '/api/pipeline/' + pipeline_no;
        const queryString = window.location.search;
        const urlParams = new URLSearchParams(queryString);
        this.urlBranchFilter = urlParams.get('branchfilters');
    }
      
      render(){
        //   var url = new URL(this.pipeline_url);
        // var search_params = url.searchParams;
        // search_params.set('id', '101');
        // change the search property of the main url
        // window.location.search  = search_params.toString();
        // var new_url = url.toString();

// output : http://demourl.com/path?id=101&topic=main
          console.log('brancfilter1' + this.urlBranchFilter);
          return(
      <div>
          <QueryClientProvider client={queryClient}>
            <FetchPipeline pipeline_url={this.pipeline_url} urlBranchFilter={this.urlBranchFilter} />
          </QueryClientProvider>
      </div>
              );
      }
     
}

function FetchPipeline(api_url, urlBranchFilter) {
  let initalBranchState="*allBranches"
  if (api_url.urlBranchFilter != null){
    initalBranchState=api_url.urlBranchFilter;
  }
  const [branchFilter, setBranchFilter] = React.useState(initalBranchState);

   const { isLoading, error, data } = useQuery('workflow_data', () =>
     fetch(api_url.pipeline_url).then(res =>
       res.json()
     ),
      {
      refetchInterval: 3000,
    }
   );

 if (isLoading) return (<div className="container-xl px-4 mt-5">
  <div className='row justify-content-md-center'>
    <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
  </div>
 </div>);
 
 if (error) return(
   <div>
   <h4>An error has occurred</h4>
   </div>   
  );
 if (data.error) return( 
   <div>
   <h4> {data.error.name}: {data.error.message} </h4>
   </div>
  );
 
 let no_workflows=<div></div>;
 if (data.num_of_workflows==0) {
    no_workflows=<div className="card p-0 mb-4">
    <div className="card-body p-5">
      <div className="row align-items-center justify-content-between">
        <div className="col">
          <h2 className="text-danger">Error</h2>
          <p className="text-gray-700">There are no workflows. Please refer to the log of this particular pipeline for more information. Possible solutions to this error are:</p>
          <p className="text-gray-700">1. Initialise the project.</p>
          <p className="text-gray-700">2. Ensure that a config.yaml file is present in the .bbx folder of the git repository.</p>
          <p className="text-gray-700">3. Resolve any errors in the config.yaml file, if the file already exists.</p>

        </div>

      </div>
    </div>
   </div>;  
 }   

 let branch_tables=[];
   for (var i = 0; i < data.branches.length; i++) {
     let branch=data.branches[i];
     
     if ((branch.branch==branchFilter) || ("*allBranches"==branchFilter)) {
       if (branch.workflows.length!=0){
         let branch_table =     <Row className="mx-0" key={i}>
          <div className="card p-0 mb-4">
              <div className="card-body p-0">
                  <Row className="m-3">
                    <Col md="auto" className="p-0">
                      <img src={git_branch} alt="git-branch" style={ {width: '17px'}} />
                    </Col>
                    <Col md="auto me-auto"> 
                        <h4 > {branch.branch} </h4> 
                    </Col>


                  <Col md="auto">  
                    <ConfigButton pipeline_id = {data.pipeline_id} branch_name={branch.branch}></ConfigButton>
                  </Col>
                  <Col md="auto">  
                    <RunButton workflowid={branch.workflows[0].workflow_id}></RunButton> 
                  </Col>                  
                  
                  </Row>
                  <Row>
                    <Tabs
                      id="controlled-tab-example"
                      defaultActiveKey="history"
                      className="mx-3"
                      variant="pills"
                    >
                    <Tab eventKey="history" title="Workflows">
                      <WorkflowRows workflows={branch.workflows}/>
                    </Tab>
                    
                    </Tabs>
                    </Row>
                  </div>
              </div>
          </Row>;
          
          
        branch_tables.push(branch_table);
       } 
       else{
         let branch_table =     <Row className="mx-0" key={i}>
          <div className="card p-0 mb-4">
              <div className="card-body p-0">
                  <Row className="m-3">
                  <Col md="auto" className="p-0"><img src={git_branch} alt="git-branch" style={ {width: '20px'}} /></Col>
                  <Col md="auto me-auto"> <h4 > {branch.branch} </h4> </Col>
                  </Row>
                  <Row>
                    <Col md="auto" className="p-5">
                      <h2 className="text-danger">Error</h2>
                      <p className="text-gray-700">There are no workflows in this branch. Please refer to the log of this particular pipeline for more information. Possible solutions to this error are:</p>
                      <p className="text-gray-700">1. Initialise the project.</p>
                      <p className="text-gray-700">2. Ensure that a config.yaml file is present in the .bbx folder of this branch in the git repository.</p>
                      <p className="text-gray-700">3. Resolve any errors in the config.yaml file, if the file already exists.</p>
            
                    </Col>                  

                  </Row>
              </div>
          </div>
          </Row>;
          
        branch_tables.push(branch_table);

       }
     }
   }
 
 let pageList=[['Pipelines', '/pipelines'], [data.pipeline_name, window.location.pathname]];
 if ("*allBranches"!=branchFilter){
   pageList.push([branchFilter, window.location.pathname + '?branchfilters=' + branchFilter]);
 } 

 let error_status = <div></div>
 let git_error_status = <div></div>
  
 if (data.status=='CE'){
    error_status=<div>
      <Alert key="danger" variant="danger">
        Configuration error — Please check pipeline log for more information. 
      </Alert>
    </div>;    
  } 
  else if (data.status=='PE'){
    error_status=<div>
      <Alert key="primary" variant="primary"><img src={running_icon} alt="loading_icon" style={ {width: '20px'}} />&nbsp;Processing...</Alert>
    </div>;    
  }       
  else if (data.status=='FA'){
    error_status=<div>
      <Alert key="danger" variant="danger">
        Pipeline error
      </Alert>
    </div>;    
  }     
  if (data.git_error==true){
    git_error_status=<div>
      <Alert key="warning" variant="warning">
        Git integration failure — Please check pipeline log and git integration settings. Refresh the pipeline to reset the error.
      </Alert>
    </div>;    
  }     

 return( <Container fluid className="px-4 mt-5">  
    <Row>
        <Breadcrumb pageList={pageList} />
    </Row>
    <Row>
        <h1 className="mb-3" >{data.pipeline_name}</h1>
    </Row>
    <Row className="mb-3">
      <Col md="auto">
        <BranchDropdown  branches= {data.branches} branchFilter={branchFilter} handleSelect={(newBranchFilter)=>{setBranchFilter(newBranchFilter)}} /> 
      </Col>
      <Col md="auto">
        <PipelineLogButton output={data.output}></PipelineLogButton> 
      </Col>      
      <Col md="auto me-auto">
        <RefreshButton pipeline_id={data.pipeline_id} size="large"></RefreshButton> 
      </Col>
    </Row>
    <Row >
      {error_status}      
    </Row>
    <Row >
      {git_error_status}      
    </Row>

    {branch_tables}
    
    {no_workflows}
    
 </Container>);
}

class PipelineLogButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }  
  handleShow(){
      this.setState({show:true});
  } 
  handleClose(){
      this.setState({show:false});
  }    
    render(){
      let log_screen =     <>
        <Button variant="primary btn"  onClick={() => this.handleShow()} >
          Pipeline log 
        </Button>  
        <Modal dialogClassName="modal-xl" show={this.state.show} onHide={() => this.handleClose()}>
          <Modal.Header closeButton>
            <Modal.Title>Pipeline log</Modal.Title>
          </Modal.Header>
          <Modal.Body>
            <div className="card mb-4">
              <Card.Body className="card-body">
                  <pre className="  language-markup">{this.props.output}</pre>
              </Card.Body>
            </div>  
          </Modal.Body>
          <Modal.Footer>
          <CloseForm onHide={this.handleClose}  closeModal={() => this.handleClose()}/>           
          </Modal.Footer>
        </Modal>
      </>;
      return(<div>
      {log_screen}
      </div>);
    }
}


class ConfigButton extends React.Component {
  constructor(props) {
    super(props);

  }
  
  handleSubmit(e) {
    // e.preventDefault();
    let data = JSON.stringify({
        name: "config_pipeline_",
        branch_: e, 
        pipeline_id_: this.props.pipeline_id
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            .then(response =>  {
              // console.log(response)
              window.location.href = response.url;
              });             
  } 

  render(){
    var eventKey=this.props.branch_name;
    let edit_button =     <>
      <Button variant="primary btn-sm px-2 p-1"  onClick={()=> {this.handleSubmit(eventKey)}} >
      <img src={config_icon} alt="config_icon" /> 
      </Button>


    </>;
    return(<div>
    {edit_button}
    </div>);
  }

}  

class BranchDropdown extends React.Component {
  render(){
  let branches=this.props.branches;
  let dropDownItems=[];
  dropDownItems.push(
    <Dropdown.Item key='*allBranches' eventKey='*allBranches'><img src={git_branch} alt="git-branch" style={ {width: '13px'}} />&nbsp;All branches</Dropdown.Item>
    );
  for (var i = 0; i < branches.length; i++) {
    dropDownItems.push(
      <Dropdown.Item key={branches[i].branch} eventKey={branches[i].branch}><img src={git_branch} alt="git-branch" style={ {width: '13px'}} />&nbsp; {branches[i].branch}</Dropdown.Item>
      );
    
  }
  let filterName='';
  if (this.props.branchFilter=='*allBranches'){
    filterName='All branches';
  } else{
    filterName=this.props.branchFilter;
  }
  return(
    <Dropdown onSelect={(eventKey)=>
      {this.props.handleSelect(eventKey);
      const url = new URL(window.location.href);
      url.searchParams.set('branchfilters', eventKey);
      window.history.replaceState(null, null, url); 
      }
    }>
      <Dropdown.Toggle variant="primary" id="dropdown-basic">
        <img src={git_branch_white} alt="git-branch-white" style={ {width: '14px'}} />&nbsp; {filterName}
      </Dropdown.Toggle>

      <Dropdown.Menu>
        {dropDownItems}
      </Dropdown.Menu>
    </Dropdown>
    );
  }
}

class WorkflowRows extends React.Component {
    
    render(){
      let workflows=this.props.workflows;
      let workflow_rows=[];
      
      // console.log(workflows)
          for (var i = 0; i < workflows.length; i++) {
            
                // console.log(workflows[i].runtime)
                
                // let trigger_date = new Date(workflows[i].last_run_rawdata);
                // let trigger_date_string=formatDate(trigger_date);
                let stopwatch_status='RU';
                let status_image='None';
                let status_span=<span className="border"></span>;
                let log_button = <LogButton output={workflows[i].output}></LogButton>;
                if (workflows[i].status=='ID'){
                  status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '25px'}} />;
                  stopwatch_status='ID';
                  status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} Idle</div>;
    
                 } else if (workflows[i].status=='IN'){
                  status_image=<img src={static_pending_icon} alt="static_pending_icon" style={ {width: '25px'}} />;
                  stopwatch_status='ID';
                  status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} Idle</div>;
                  
                 } else if (workflows[i].status=='SC'){
                  status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '25px'}} />;
                  stopwatch_status='PE';
                  status_span=<div className="border rounded" style={{"padding": "5px", "width": "150px",  "backgroundColor": "#F4F4F4"}}>{status_image} {workflows[i].schedule}</div>;
                  log_button = 'N/A';
                  
                 } else if (workflows[i].status=='PA'){
                  status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '25px'}} />;
                  stopwatch_status='PA';
                  status_span=<div className="border rounded border-success text-success" style={{"padding": "5px", "width": "150px", "backgroundColor": "#EAFAF4"}}>{status_image} Passed</div>;
                 } else if (workflows[i].status=='PE'){
                  status_image=<img src={running_icon} alt="loading_icon" style={ {width: '25px'}} />;
                  stopwatch_status='RU';
                  status_span=<div className="border rounded border-primary text-primary" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F4F8FF"}}>{status_image} Running</div>;
                 } else if (workflows[i].status=='FA'){
                  stopwatch_status='FA';
                  status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '25px'}} />;
                  status_span=<div className="border rounded border-danger text-danger" style={{"padding": "5px", "width": "150px", "backgroundColor": "#F8DFE2"}}>{status_image} Failed</div>;                
                }
                workflow_rows.push(
                  <tr role="row" className="odd" key={i}>
                    <td className="align-middle"><a href={workflows[i].workflow_url}>{workflows[i].workflow_name}</a></td>
                    <td className="align-middle"><StopWatch status={stopwatch_status} start_time={workflows[i].runtime}></StopWatch></td>
                    <td className="align-middle">{workflows[i].last_run}</td>
                    <td className="align-middle">{workflows[i].commit_hash.slice(0, 7)}</td>
                    <td className="align-middle">{status_span}</td>
                    <td className="align-middle">{log_button}</td>
                    <td><StopButton workflow_id={workflows[i].workflow_id}></StopButton> </td>                
                  </tr>
                  );
          }
          return(
                  <PaginatedTable rows={workflow_rows} number_of_rows_per_page="5" />
              );
      
    }
}



class PaginatedTable extends React.Component{
  constructor(props) {
    super(props);
    this.state = {
      currentPageIndex: 0,
      modalEllipseShow : false
    };
  }
  

  handleEllipseClose(){
    this.setState({modalEllipseShow : false});
  }
  
  handleEllipseOpen(){
    this.setState({modalEllipseShow : true});
  }
  
  handleSubmit(event){
    let rows=this.props.rows;
    let userSetPage=parseInt(event.target.pageNumber.value-1);
    let number_of_pages=Math.ceil(rows.length / this.props.number_of_rows_per_page);
    if (number_of_pages >= userSetPage) {
      this.setState({currentPageIndex : parseInt(event.target.pageNumber.value-1)});
    }
    this.setState({modalEllipseShow : false});
    event.preventDefault();
  }
  
  
  paginationEllipseClicked(number){
    this.handleEllipseOpen();
  }
  
  paginationClicked(number){
    this.setState({currentPageIndex : number-1});
  }
  
  paginationFirstClicked(){
    this.setState({currentPageIndex : 0});
  }
  
  paginationPreviousClicked(){
    if (this.state.currentPageIndex==0){
      this.setState({currentPageIndex : 0});
    } else{
      let tempNumber=this.state.currentPageIndex-1;
      this.setState({currentPageIndex : tempNumber});
    }
  }
  
  paginationNextClicked(number_of_pages){
    if (this.state.currentPageIndex==number_of_pages-1){
      this.setState({currentPageIndex : number_of_pages-1});
    } else{
      let tempNumber=this.state.currentPageIndex+1;
      this.setState({currentPageIndex : tempNumber});
    }  }
  
  paginationLastClicked(number_of_pages){
    this.setState({currentPageIndex : number_of_pages-1});
  }
  
  render(){
    let rows=this.props.rows;
    let current_page_list=[];
    let starting_index=this.props.number_of_rows_per_page*this.state.currentPageIndex;
    for (var i = 0; i < this.props.number_of_rows_per_page; i++) {
      let current_index=starting_index+i;
      if (current_index < rows.length){
        current_page_list.push(rows[current_index]);
      }
    }
    
    let number_of_pages=Math.ceil(rows.length / this.props.number_of_rows_per_page);
    let items = [];
    if (number_of_pages<8){
      for (let number = 1; number <= number_of_pages; number++) {
        items.push(
          <Pagination.Item key={number} onClick={() => this.paginationClicked(number)} active={number === parseInt(this.state.currentPageIndex+1)}>
            {number}
          </Pagination.Item>,
        );
      }
    } else {
      for (let number = 1; number <= 8; number++) {
        items.push(
          <Pagination.Item key={number} onClick={() => this.paginationClicked(number)} active={number === parseInt(this.state.currentPageIndex+1)}>
            {number}
          </Pagination.Item>,
        );
      }
      if (8<this.state.currentPageIndex+1){
          items.push(<Pagination.Ellipsis key="ellipse1" onClick={() => this.paginationEllipseClicked()}/>);
          items.push(<Pagination.Item key="current" onClick={() => this.paginationClicked(this.state.currentPageIndex+1)} active={true}>{this.state.currentPageIndex+1} </Pagination.Item>);
          if ((this.state.currentPageIndex+1)!=number_of_pages){
          items.push(<Pagination.Ellipsis key="ellipse2" onClick={() => this.paginationEllipseClicked()}/>);
          }
      }
      else{
        items.push(<Pagination.Ellipsis key="ellipse1" onClick={() => this.paginationEllipseClicked()}/>);

      }


    }
    
    
    return(
      <div>
      <Row>
        <Table className="table dataTable" id="projectTable" role="grid">
            <thead>
            <tr>
              <th style={{width: "16.66%"}}> Workflow </th>
              <th style={{width: "16.66%"}}> Run Time </th>
              <th style={{width: "16.66%"}}> Last Run </th>
              <th style={{width: "16.66%"}}> Commit </th>
              <th style={{width: "16.66%"}}> Status </th>
              <th style={{width: "8.333%"}}> Log </th>
              <th style={{width: "8.333%"}}> Actions </th>
            </tr>
        </thead>
    <tbody>
    {current_page_list}
   </tbody>
    </Table>
    </Row>
    <Row className="justify-content-md-center">
      <Col md="auto">
      <Pagination>
        <Pagination.First onClick={() => this.paginationFirstClicked()}  />
        <Pagination.Prev onClick={() => this.paginationPreviousClicked()}  />
        {items}
        <Pagination.Next onClick={() => this.paginationNextClicked(number_of_pages)}  />
        <Pagination.Last onClick={() => this.paginationLastClicked(number_of_pages)}  />
      </Pagination>
      </Col>
    </Row>
      
      <Modal size="sm" show={this.state.modalEllipseShow} onHide={ () => this.handleEllipseClose()}>
        <form onSubmit={ (event) => {this.handleSubmit(event)}}>
        <Modal.Header closeButton>
          <Modal.Title>Page Select</Modal.Title>
        </Modal.Header>
        <Modal.Body>
        <label>
          <input type="text" name= "pageNumber" />
        </label>
        </Modal.Body>
        <Modal.Footer>
          <Button variant="danger" onClick={() => this.handleEllipseClose()}>
            Close
          </Button>
          <Button variant="primary" type="submit" value="Submit">
            Submit
          </Button>
        </Modal.Footer>
        </form>
      </Modal>
  </div>

);
  }
}


class RunButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }
    
    render(){
    let run_confirmation_screen =     <>
      <Button variant="primary btn-sm px-2 p-1"  onClick={() => this.handleShow()} >
        <img src={play_icon} alt="play_icon" /> 
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header closeButton>
          <Modal.Title>Run Workflow</Modal.Title>
        </Modal.Header>
        <Modal.Body>Are you sure you want to run this workflow?</Modal.Body>
        <Modal.Footer>
        <RunForm workflowid={this.props.workflowid} onHide={this.handleClose}  closeModal={() => this.handleClose()}/> 

        </Modal.Footer>
      </Modal>
    </>;
    return(<div>
    {run_confirmation_screen}
    </div>);
    }
}

function RunForm(props) {
  function handleSubmit(e) {
    e.preventDefault();
    let data = JSON.stringify({
        name: "run_workflow_",
        id_: props.workflowid
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            // .then(response => response.json())
            .then();
    props.closeModal();
  }

  return (
    <form onSubmit={handleSubmit}>
      <button className="btn btn-danger " type="submit" id="run_workflow_" name="run_workflow_">Yes</button>
    </form>
  );
}

class StopButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }
    
    render(){
    let stop_confirmation_screen =     <>
      <Button variant="primary btn-sm  px-2 p-1"  onClick={() => this.handleShow()} >
        <img src={stop_icon} alt="stop_icon" /> 
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header closeButton>
          <Modal.Title>Stop Workflow</Modal.Title>
        </Modal.Header>
        <Modal.Body>Are you sure you want to stop all current jobs in this workflow?</Modal.Body>
        <Modal.Footer>
        <Form workflow_id={this.props.workflow_id} onHide={this.handleClose}  closeModal={() => this.handleClose()}/> 

        </Modal.Footer>
      </Modal>
    </>;
    return(<div>
    {stop_confirmation_screen}
    </div>);
    }
}

function Form(props) {
  function handleSubmit(e) {
    e.preventDefault();
    let data = JSON.stringify({
        name: "stop_workflow_",
        id_: props.workflow_id
    });
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            // .then(response => response.json())
            .then();
    props.closeModal();
  }

  return (
    <form onSubmit={handleSubmit}>
      <button className="btn btn-danger " type="submit" id="stop_workflow_" name="stop_workflow_">Yes</button>
    </form>
  );
}
