import React from 'react';
import ReactDOM from "react-dom";
import styled from "styled-components";
import running_icon from './img/beetlebox-running-icon.png';
import static_running_icon from './img/beetleboxci-static-running-icon.svg';
import pending_icon from './img/beetleboxci-pending-icon.png';
import static_pending_icon from './img/beetleboxci-static-pending-icon.svg';
import failure_icon from './img/beetleboxci-failure-icon.svg';
import pass_icon from './img/beetleboxci-pass-icon.svg';
import stop_icon from './img/stop.svg';
import refresh from './img/refresh-cw.svg';
import git_commit from './img/git-commit.svg';
import git_branch from './img/git-branch-svg-repo.svg'

import {Button, Modal, Table, Container, Row, Col, Card} from 'react-bootstrap';

import StopWatch from "./stopwatch.js"
import {formatDate, getCookie, humanFileSize} from "./shared_functions.js"
import {Breadcrumb} from './shared_classes.js'


import {
   useQuery,
   useMutation,
   useQueryClient,
   QueryClient,
   QueryClientProvider,
 } from 'react-query'
 
const queryClient = new QueryClient();

export default class WorkflowView extends React.Component {
      constructor(props) {
        super(props);
        var pathArray = window.location.pathname.split('/');
        var workflow_no=pathArray[2];
        this.workflow_url = window.location.protocol + '//' + window.location.host + '/api/workflow/' + workflow_no;
        this.steps_jsx=[];
    }
      
      render(){
          return(
      <div>
          <QueryClientProvider client={queryClient}>
            <FetchWorkflow workflow_url={this.workflow_url} />
          </QueryClientProvider>
      </div>
              );
      }
     
}

function FetchWorkflow(api_url) {
   const { isLoading, error, data } = useQuery('workflow_data', () =>
     fetch(api_url.workflow_url).then(res =>
       res.json()
     ),
      {
      // Refetch the data every three seconds
      refetchInterval: 3000,
    }
   );
   
     if (isLoading) return (<div className="container-xl px-4 mt-5">
      <div className='row justify-content-md-center'>
        <img src={running_icon} alt="loading_icon" style={ {width: '75px'}} /> Loading
      </div>
     </div>);
 
    if (error) return 'An error has occurred';
    if (data.error) return data.error.name + ': ' + data.error.message;
    
    var status_image, supporting_text;
    let stopwatch_status;
    let trigger_date = new Date(data.last_run);
    let trigger_date_string=formatDate(trigger_date);
    //  console.log(data)
 
    if (data.status=='SC'){
        status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '50px'}} />;
        stopwatch_status='PE';
        supporting_text='Last Triggered:';
    } else if (data.status=='IN'){
        status_image=<img src={static_pending_icon} alt="static_pending_icon" style={ {width: '50px'}} />;
        stopwatch_status='ID';
        supporting_text='Triggered on:';          
    } else if (data.status=='ID'){
        status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '50px'}} />;
        stopwatch_status='ID';
        supporting_text='Triggered on:';  
    } else if (data.status=='PA'){
        status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '50px'}} />;
        stopwatch_status='PA';
        supporting_text='Triggered on:';
    } else if (data.status=='PE'){
        status_image=<img src={running_icon} alt="loading_icon" style={ {width: '50px'}} />;
        stopwatch_status='RU';
        supporting_text='Triggered on:';
    } else if (data.status=='FA'){
        stopwatch_status='FA';
        status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '50px'}} />;
        supporting_text='Triggered on:';
    }
    let status_card= 
        <Col xl="3" className="mb-4 pr-0 align-items-stretch">
        <Card className="h-100">
          <Card.Body className="card-body">
           <Row>
              <div className="h3">Status</div>
           </Row>
           <Row>
          <Col md="auto">
            {status_image}
          </Col>
          <Col md="auto" className="align-self-center">
            <StopWatch start_time={data.runtime} status={stopwatch_status}></StopWatch>
          </Col>
           </Row>
          <Row className="align-self-center">
          <p>
          {supporting_text} <br></br>
          {trigger_date_string} </p>
         </Row>
         </Card.Body>
       </Card>
       </Col>;
    let repo_card= 
        <Col xl="3" className="mb-4 align-items-start">
        <Card className="h-100">
          <Card.Body className="card-body">
           <Row>
              <div className="h3">Repo info</div>
           </Row>
           <Row>
              <div className="h5">Branch</div>
           </Row>
           <Row>
              <Col md="1">
                <img src={git_branch} alt="git_branch" style={ {width: '15px', stroke: "maroon"}} /> 
              </Col>
              <Col md="9" className="align-self-center">
                {data.git_branch}
              </Col>

              
           </Row>
           <Row>
              <div className="h5">Commit</div>
           </Row>
           <Row>
            <Col md="1">
              <img src={git_commit} alt="git_commit" style={ {width: '15px', stroke: "maroon"}} /> 
            </Col>
            <Col md="9" className="align-self-center">
              {data.commit_hash.slice(0, 7)}
            </Col>
           </Row>
         </Card.Body>
       </Card>
       </Col>;
       
    let resource_card= 
        <Col xl="3" className="mb-4 align-items-start">
        <Card className="h-100">
          <Card.Body className="card-body">
           <Row>
              <div className="h3">Resource Usage</div>
           </Row>
           <Row>
              <div className="h5">Requested</div>
           </Row>
           <Row>
              <Col md="6">
                CPU: {data.CPU_request}
              </Col>
              <Col md="6" className="align-self-center">
                RAM: {data.RAM_request}
              </Col>
           </Row>
           

           <Row>
              <div className="h5">Current Usage</div>
           </Row>
           <Row>
          <Col md="6">
            CPU: {data.CPU_usage}
          </Col>
          <Col md="6" className="align-self-center">
            RAM: {data.RAM_usage}
          </Col>
           </Row>
         </Card.Body>
       </Card>
       </Col>;
       
       
    let human_storage=humanFileSize(data.storage_usage)
    let artifact_card= 
        <Col xl="3" className="mb-4 align-items-stretch">
        <Card className="h-100">
          <Card.Body className="card-body">
           <Row>
              <div className="h3">Artifacts</div>
           </Row>
           <Row>
              <div className="h5">Number of Artifacts</div>
           </Row>
           <Row>
              <div className="h7">{data.no_of_artifacts}</div>
           </Row>
           <Row>
              <div className="h5">Current Usage</div>
           </Row>
           <Row>
              <div className="h7">{human_storage}</div>
           </Row>
         </Card.Body>
       </Card>
       </Col>;
       
    let table = <div>
        <Table className="table dataTable" id="user-table" role="grid">
          <thead>
          <tr>
            <th style={{width: "16.66%"}}> Job Name </th>
            <th style={{width: "16.66%"}}> Run Time </th>
            <th style={{width: "16.66%"}}> Last Run </th>
            <th style={{width: "21.66%"}}> Container </th>
            <th style={{width: "21.66%"}}> Status </th>
            <th style={{width: "6.66%"}}> Actions </th>
          </tr>
        </thead>
        <JobsRows jobs={data.jobs} workflow_status={data.status}/>
        </Table>
    </div>;


    let pageList=[['Pipelines', data.pipelines_url], [data.project_name, data.project_url], [data.branch, data.project_url + "?branchfilters=" + data.branch], [data.workflow_name, window.location.pathname] ];

    return(  <Container fluid className="px-4 mt-5">
    <Row>
        <Breadcrumb pageList={pageList} />
    </Row>
    <Row>
        <h1 className="mb-4" >{data.workflow_name}</h1>
    </Row>
    <Row>
    {status_card}
    {repo_card}
    {resource_card}
    {artifact_card}
    </Row>
    <Row className="mx-0">
        <div className="card p-0 mb-4">
            <div className="card-body p-0">
            {table}
            </div>
        </div>
    </Row>
    </Container>);
  
}


class JobsRows extends React.Component {
      render(){
        let jobs=this.props.jobs;
        let workflow_status=this.props.workflow_status;
        
        let status_image;
        let stopwatch_status;
        let svg_size=20;
        let gif_size=20;
        let supporting_text;
        let job_rows=[]
        let status_span=<span className="border"></span>
          for (var i = 0; i < jobs.length; i++) {
              
            if (workflow_status=='IN'){
              status_image=<img src={static_pending_icon} alt="static_pending_icon" style={ {width: '30px'}} />
              stopwatch_status='PA';
              supporting_text='Idle'
              status_span=<div className="border rounded" style={{"padding": "5px", "width": "200px",  "backgroundColor": "#F4F4F4"}}>{status_image} {supporting_text}</div>
             } else if (jobs[i].status=='PE'||jobs[i].status=='PR'){
              status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '30px'}} />
              stopwatch_status='PE';
              supporting_text='Steps Pending: ' + jobs[i].no_of_idle_steps
              status_span=<div className="border rounded" style={{"padding": "5px", "width": "200px",  "backgroundColor": "#F4F4F4"}}>{status_image} {supporting_text}</div>
             } else if (jobs[i].status=='ID'){
              status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '30px'}} />
              stopwatch_status='ID'
              supporting_text='Idle'
              status_span=<div className="border rounded" style={{"padding": "5px", "width": "200px",  "backgroundColor": "#F4F4F4"}}>{status_image} {supporting_text}</div>
             } else if (jobs[i].status=='PD'||jobs[i].status=='QU'||jobs[i].status=='Q1'||jobs[i].status=='Q2'||jobs[i].status=='Q3'||jobs[i].status=='Q4'||jobs[i].status=='Q5'||jobs[i].status=='Q6'){
              status_image=<img src={pending_icon} alt="pending_icon" style={ {width: '30px'}} />
              stopwatch_status='QU'
              supporting_text='Queued'
              status_span=<div className="border rounded" style={{"padding": "5px", "width": "200px",  "backgroundColor": "#F4F4F4"}}>{status_image} {supporting_text}</div>
             } else if (jobs[i].status=='PA'){
              status_image=<img src={pass_icon} alt="pass_icon" style={ {width: '30px'}} />
              stopwatch_status='PA'
              supporting_text='Steps Passed: ' + jobs[i].no_of_passed_steps
              status_span=<div className="border rounded border-success text-success" style={{"padding": "5px", "width": "200px", "backgroundColor": "#EAFAF4"}}>{status_image} {supporting_text}</div>
             } else if (jobs[i].status=='RU'){
              status_image=<img src={running_icon} alt="loading_icon" style={ {width: '30px'}} />
              stopwatch_status='RU';
              supporting_text='Steps Running:  ' + jobs[i].no_of_pending_steps
              status_span=<div className="border rounded border-primary text-primary" style={{"padding": "5px", "width": "200px", "backgroundColor": "#F4F8FF"}}>{status_image} {supporting_text}</div>
             } else if (jobs[i].status=='ST'){
              status_image=<img src={running_icon} alt="loading_icon" style={ {width: '30px'}} />
              stopwatch_status='RU';
              supporting_text='Steps Running:  ' + jobs[i].no_of_pending_steps  
              status_span=<div className="border rounded border-primary text-primary" style={{"padding": "5px", "width": "200px", "backgroundColor": "#F4F8FF"}}>{status_image} {supporting_text}</div>
             } else if (jobs[i].status=='FA'){
              stopwatch_status='FA';
              status_image=<img src={failure_icon} alt="failure_icon" style={ {width: '30px'}} />
              supporting_text='Steps Failed: ' + jobs[i].no_of_failed_steps
              status_span=<div className="border rounded border-danger text-danger" style={{"padding": "5px", "width": "200px", "backgroundColor": "#F8DFE2"}}>{status_image} {supporting_text}</div>
             }
             let fulL_job_url=jobs[i].job_url
             let trigger_date = new Date(jobs[i].last_run);
            
             let trigger_date_string=formatDate(trigger_date);
             job_rows.push(
              <tr role="row" className="odd" key={i}>
                <td><a href={fulL_job_url}>{jobs[i].job_name}</a></td>
                <td><StopWatch status={stopwatch_status} start_time={jobs[i].runtime}></StopWatch></td>
                <td>{jobs[i].last_run}</td>
                <td>{jobs[i].container}</td>
                <td>{status_span}</td>
                <td>
                 <StopButton job_id={jobs[i].job_id}/>
                </td>
              </tr>
              );
          }
          return(
        <tbody>
         {job_rows}
        </tbody>
              );
      }
}

class StopButton extends React.Component {
  constructor(props) {
    super(props);
    this.handleClose = this.handleClose.bind(this);
    this.state = {
      show: false
    };
  }
  
  handleShow(){
      this.setState({show:true});
  }
 
  handleClose(){
      this.setState({show:false});
  }
    
    render(){
    let confirmation_screen_2 =     <>
      <Button variant="primary btn-sm" onClick={() => this.handleShow()}>
        <img src={stop_icon} alt="stop_icon" style={ {width: '20px'}} /> 
      </Button>

      <Modal show={this.state.show} onHide={() => this.handleClose()}>
        <Modal.Header closeButton>
          <Modal.Title>Stop Job</Modal.Title>
        </Modal.Header>
        <Modal.Body>Are you sure you want to stop this job?</Modal.Body>
        <Modal.Footer>
        <Form job_id={this.props.job_id} onHide={this.handleClose} closeModal={() => this.handleClose()}/> 

        </Modal.Footer>
      </Modal>
    </>
    return(<div>
    {confirmation_screen_2}
    </div>)
    }
}

function Form(props) {
  function handleSubmit(e) {
    e.preventDefault();
    let data = JSON.stringify({
        name: "stop_job_",
        job_id: props.job_id
    })
     const requestOptions = {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
              'X-CSRFToken': getCookie('csrftoken'),
            },
            credentials: 'same-origin',
            body: data
            };
        fetch('', requestOptions)
            // .then(response => response.json())
            .then();
    props.closeModal();
            
  }

  return (
    <form onSubmit={handleSubmit}>
      <button className="btn btn-danger"  type="submit" id="stop_job_btn" name="stop_job_">Yes</button>
    </form>
  );
}
